@@ -1,11 +1,8 @@
 /*
- * $Id: clock.h,v 1.1 2004/08/25 05:45:04 rmanfredi Exp $
+ * $Id: clock.h,v 1.4 2005/08/17 13:44:05 cbiere Exp $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * Maintain an accurate clock skew of our host's clock with respect
- * to the absolute time.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -26,11 +23,22 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Maintain an accurate clock skew of our host's clock with respect
+ * to the absolute time.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #ifndef _core_clock_h_
 #define _core_clock_h_
 
 #include "common.h"
-
+#include "lib/host_addr.h"
 
 /*
  * Public interface.
@@ -39,7 +47,7 @@
 void clock_init(void);
 void clock_close(void);
 
-void clock_update(time_t update, gint precision, guint32 ip);
+void clock_update(time_t update, gint precision, const host_addr_t addr);
 
 time_t clock_loc2gmt(time_t stamp);
 time_t clock_gmt2loc(time_t stamp);
