@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c,v 1.14 2004/11/22 08:35:21 rmanfredi Exp $
+ * $Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -22,28 +22,37 @@
  *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *----------------------------------------------------------------------
  */
+
 /**
+ * @ingroup core
  * @file
  *
  * Host cache management.
  *
- * TODO: 
- *  - finer grained stats: 
- *      hits/misses while adding, 
- *      hits/misses while bad checking
- *      how many hosts were tried to connect to?
- *  - display stats about gwcache usage:
- *      how often
- *      how many hosts got
- *  - move unstable servant code from nodes.c to hcache.c
- *  - make sure hosts we are currently connected too are also saved
- *    to disk on exit!
- *  - save more metadata if we can make use of it.
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2002-2003
+ *
+ * @todo
+ * TODO:
+ *
+ *	- finer grained stats:
+ *		-# hits/misses while adding,
+ *		-# hits/misses while bad checking
+ *		-# how many hosts were tried to connect to?
+ *	- display stats about gwcache usage:
+ *		-# how often
+ *		-# how many hosts got
+ *	- move unstable servant code from nodes.c to hcache.c
+ *	- make sure hosts we are currently connected too are also saved
+ *		to disk on exit!
+ *	- save more metadata if we can make use of it.
+ *
  */
 
 #include "common.h"
 
-RCSID("$Id: hcache.c,v 1.14 2004/11/22 08:35:21 rmanfredi Exp $");
+RCSID("$Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $");
 
 #include "hosts.h"
 #include "hcache.h"
@@ -56,14 +65,15 @@ RCSID("$Id: hcache.c,v 1.14 2004/11/22 08:35:21 rmanfredi Exp $");
 #include "lib/bg.h"
 #include "lib/file.h"
 #include "lib/hashlist.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/override.h"			/* Must be the last header included */
 
-#define MIN_RESERVE_SIZE	1024	/* we'd like that many pongs in reserve */
+#define MIN_RESERVE_SIZE	1024	/**< we'd like that many pongs in reserve */
 
 /**
  * An entry within the hostcache.
@@ -73,16 +83,16 @@ RCSID("$Id: hcache.c,v 1.14 2004/11/22 08:35:21 rmanfredi Exp $");
  * the host.
  */
 typedef struct hostcache_entry {
-    hcache_type_t type;        /**< Hostcache which contains this host */
-    time_t        time_added;  /**< Time when entry was added */
+    hcache_type_t type;					/**< Hostcache which contains this host */
+    time_t        time_added;			/**< Time when entry was added */
 #if 0
-	guint32       avg_uptime;  /**< Reported average uptime (seconds) */
-	gchar *       vendor;	   /**< Latest known vendor name (atom) */
+	guint32       avg_uptime;			/**< Reported average uptime (seconds) */
+	gchar *       vendor;				/**< Latest known vendor name (atom) */
 #endif
 } hostcache_entry_t;
 
 /** No metadata for host */
-#define NO_METADATA			GINT_TO_POINTER(1)	
+#define NO_METADATA			GINT_TO_POINTER(1)
 
 /**
  * A hostcache table.
@@ -91,7 +101,7 @@ typedef struct hostcache {
 	gchar *         name;		        /**< Name of the cache */
 	hcache_type_t   type;				/**< Cache type */
 
-    gboolean        ip_only;            /**< Use IP only, port always 0 */
+    gboolean        addr_only;          /**< Use IP only, port always 0 */
     gboolean        dirty;            	/**< If updated since last disk flush */
     hash_list_t *   hostlist;           /**< Host list: IP/Port  */
 
@@ -109,25 +119,26 @@ static hostcache_t *caches[HCACHE_MAX];
 static gboolean hcache_close_running = FALSE;
 
 /**
- * Names of the host caches. 
+ * Names of the host caches.
  *
- * @note has to be in the same order as in the hcache_type_t definition
- *       in gnet_nodes.h.
+ * @note
+ * Has to be in the same order as in the hcache_type_t definition
+ * in gnet_nodes.h.
  */
-static const gchar * const names[HCACHE_MAX] = { 
-    N_("fresh regular"), 
-    N_("valid regular"),
-    N_("fresh ultra"),
-    N_("valid ultra"),
-    N_("timeout"),
-    N_("busy"),
-    N_("unstable"),
+static const gchar * const names[HCACHE_MAX] = {
+    "fresh regular",
+    "valid regular",
+    "fresh ultra",
+    "valid ultra",
+    "timeout",
+    "busy",
+    "unstable",
     "none",
 };
 
-static const gchar * const host_type_names[HOST_MAX] = { 
-    N_("any"), 
-    N_("ultra"),
+static const gchar * const host_type_names[HOST_MAX] = {
+    "any",
+    "ultra",
 };
 
 
@@ -195,14 +206,14 @@ stop_mass_update(hostcache_t *hc)
 /**
  * Hashtable: IP/Port -> Metadata
  */
-static GHashTable * ht_known_hosts = NULL;	
+static GHashTable * ht_known_hosts = NULL;
 
 static void
 hcache_update_low_on_pongs(void)
 {
-    host_low_on_pongs = (guint) ( 
+    host_low_on_pongs = (guint) (
             caches[HCACHE_FRESH_ANY]->host_count +
-            caches[HCACHE_VALID_ANY]->host_count 
+            caches[HCACHE_VALID_ANY]->host_count
         ) < (max_hosts_cached >> 3);
 }
 
@@ -231,7 +242,7 @@ hcache_dump_info(const struct hostcache *hc, const gchar *what)
 {
     g_message("[%s|%s] %u (%d) hosts (%u hits, %u misses)",
         hc->name, what, hc->host_count,
-        hash_list_length(hc->hostlist), 
+        hash_list_length(hc->hostlist),
         hc->hits, hc->misses);
 }
 
@@ -242,19 +253,22 @@ hcache_dump_info(const struct hostcache *hc, const gchar *what)
 /**
  * Get information about the host entry, both the host and the metadata.
  *
+ * @param addr	the address of the host
+ * @param port	the port used by the host
  * @param h		filled with the host entry in the table
  * @param e		filled with the meta data of the host, as held in table
  *
  * @return FALSE if entry was not found in the cache.
  */
 static gboolean
-hcache_ht_get(guint32 ip, guint16 port, gnet_host_t **h, hostcache_entry_t **e)
+hcache_ht_get(const host_addr_t addr, guint16 port,
+	gnet_host_t **h, hostcache_entry_t **e)
 {
 	gnet_host_t host;
 	gpointer k, v;
 	gboolean found;
 
-	host.ip = ip;
+	host.addr = addr;
 	host.port = port;
 
 	found = g_hash_table_lookup_extended(ht_known_hosts, &host, &k, &v);
@@ -267,12 +281,13 @@ hcache_ht_get(guint32 ip, guint16 port, gnet_host_t **h, hostcache_entry_t **e)
 }
 
 /**
- * Add host to the hash table host cache. 
- * Also creates a metadata struct unless the host was added to HL_CAUGHT 
- * in which case we cannot know anything about the host. Yet we cannot 
- * assert that HL_CAUGHT never contains a host with metadata because when 
- * HL_CAUGHT becomes empty, move all hosts from HL_VALID to HL_CAUGHT. We 
- * can however assert that any host which does not have metadata is in 
+ * Add host to the hash table host cache.
+ *
+ * Also creates a metadata struct unless the host was added to HL_CAUGHT
+ * in which case we cannot know anything about the host. Yet we cannot
+ * assert that HL_CAUGHT never contains a host with metadata because when
+ * HL_CAUGHT becomes empty, move all hosts from HL_VALID to HL_CAUGHT. We
+ * can however assert that any host which does not have metadata is in
  * HL_CAUGHT.
  *
  * @return Pointer to metadata struct for the added host or NO_METADATA
@@ -285,7 +300,7 @@ hcache_ht_add(hcache_type_t type, gnet_host_t *host)
 
     hce = hce_alloc();
     hce->type = type;
-    hce->time_added = time((time_t *) NULL);
+    hce->time_added = tm_time();
 
 	g_hash_table_insert(ht_known_hosts, host, hce);
 
@@ -310,7 +325,7 @@ hcache_ht_remove(gnet_host_t *host)
 
 	if (!found) {
 		g_warning("hcache_ht_remove: attempt to remove unknown host: %s",
-				  ip_port_to_gchar(host->ip, host->port));
+				  host_addr_port_to_string(host->addr, host->port));
 		return;
 	}
 
@@ -336,12 +351,20 @@ hcache_get_metadata(gnet_host_t *host)
  * @return TRUE if the host is in one of the "bad hosts" caches.
  */
 gboolean
-hcache_node_is_bad(guint32 ip)
+hcache_node_is_bad(const host_addr_t addr)
 {
     hostcache_entry_t *hce;
     gnet_host_t h;
 
-    h.ip = ip;
+	/*
+	 * When we're low on pongs, we cannot afford the luxury of discarding
+	 * any IP address, or we'll end up contacting web caches for more.
+	 */
+
+	if (host_low_on_pongs)
+		return FALSE;
+
+    h.addr = addr;
     h.port = 0;
     hce = hcache_get_metadata(&h);
 
@@ -380,16 +403,16 @@ hcache_move_entries(hostcache_t *to, hostcache_t *from)
     from->hostlist = hash_list_new();
     from->host_count = 0;
 
-    /* 
+    /*
      * Make sure that after switching hce->list points to the new
-     * list HL_CAUGHT 
-     */        
+     * list HL_CAUGHT
+     */
 
 	iter = hash_list_iterator(to->hostlist);
 
 	while (NULL != (item = hash_list_next(iter))) {
         hostcache_entry_t *hce;
-    
+
         hce = hcache_get_metadata((gnet_host_t *) item);
         if (hce == NULL || hce == NO_METADATA)
             continue;
@@ -403,14 +426,14 @@ hcache_move_entries(hostcache_t *to, hostcache_t *from)
  * Make sure we have some host available in HCACHE_FRESH_ANY
  * and HCACHE_FRESH_ULTRA.
  *
- * If one of the both is empty, all hosts from HCACHE_VALID_XXX 
+ * If one of the both is empty, all hosts from HCACHE_VALID_XXX
  * are moved to HCACHE_VALID_XXX. When caught on other hcaches than
  * FRESH_ANY and FRESH_ULTRA nothing happens.
  *
  * @return TRUE if host are available in hc after the call.
  */
 static gboolean
-hcache_require_caught(hostcache_t *hc) 
+hcache_require_caught(hostcache_t *hc)
 {
     g_assert(NULL != hc);
 
@@ -467,30 +490,31 @@ hcache_unlink_host(hostcache_t *hc, gnet_host_t *host)
  * Convert host cache type to string.
  */
 const gchar *
-hcache_type_to_gchar(hcache_type_t type)
+hcache_type_to_string(hcache_type_t type)
 {
 	g_assert((guint) type < HCACHE_MAX);
 
-	return _(names[type]);
+	return names[type];
 }
 
 /**
  * Convert host type to string.
  */
 const gchar *
-host_type_to_gchar(hcache_type_t type)
+host_type_to_string(host_type_t type)
 {
 	g_assert((guint) type < HOST_MAX);
 
-	return _(host_type_names[type]);
+	return host_type_names[type];
 }
 
 
 /**
  * @return the number of slots which can be added to the given type.
  *
- * @note Several types share common pools. Adding a host of one type
- *       may affect the number of slots left on other types.
+ * @note
+ * Several types share common pools. Adding a host of one type may
+ * affect the number of slots left on other types.
  */
 static gint32
 hcache_slots_left(hcache_type_t type)
@@ -500,12 +524,12 @@ hcache_slots_left(hcache_type_t type)
     switch (type) {
     case HCACHE_FRESH_ANY:
     case HCACHE_VALID_ANY:
-        return max_hosts_cached - 
+        return max_hosts_cached -
             caches[HCACHE_FRESH_ANY]->host_count -
             caches[HCACHE_VALID_ANY]->host_count;
     case HCACHE_FRESH_ULTRA:
     case HCACHE_VALID_ULTRA:
-        return max_ultra_hosts_cached - 
+        return max_ultra_hosts_cached -
             caches[HCACHE_FRESH_ULTRA]->host_count -
             caches[HCACHE_VALID_ULTRA]->host_count;
 	case HCACHE_NONE:
@@ -529,7 +553,8 @@ hcache_slots_left(hcache_type_t type)
  *         was added to the cache. (See above)
  */
 gboolean
-hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
 	gnet_host_t *host;
 	hostcache_t *hc;
@@ -537,16 +562,25 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 
 	g_assert((guint) type < HCACHE_MAX && type != HCACHE_NONE);
 
-    if ((type == HCACHE_UNSTABLE) && (!node_monitor_unstable_ip)) {
-        return FALSE;
-    }
+	/*
+	 * Don't add anything to the "unstable" cache if they don't want to
+	 * monitor unstable servents or when we're low on pongs (thereby
+	 * automatically disabling this monitoring).  The aim is to avoid
+	 * the host discarding the last few IP addresses it has, forcing it
+	 * to contact the web caches...
+	 */
 
-	if (ip == listen_ip() && port == listen_port) {
+    if (type == HCACHE_UNSTABLE) {
+    	if (!node_monitor_unstable_ip || host_low_on_pongs)
+			return FALSE;
+	}
+
+	if (host_addr_equal(addr, listen_addr()) && port == listen_port) {
         stats[HCACHE_LOCAL_INSTANCE]++;
 		return FALSE;
     }
 
-	if (node_host_is_connected(ip, port)) {
+	if (node_host_is_connected(addr, port)) {
         stats[HCACHE_ALREADY_CONNECTED]++;
 		return FALSE;			/* Connected to that host? */
     }
@@ -554,12 +588,15 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 	hc = caches[type];
     g_assert(hc->type == type);
 
-    if (!ip_is_valid(ip) && (!hc->ip_only || !port_is_valid(port))) {
+    if (
+		!host_addr_is_routable(addr) &&
+		(!hc->addr_only || !port_is_valid(port))
+	) {
         stats[HCACHE_INVALID_HOST]++;
 		return FALSE;			/* Is host valid? */
     }
 
-    if (bogons_check(ip) || hostiles_check(ip)) {
+    if (bogons_check(addr) || hostiles_check(addr)) {
         stats[HCACHE_INVALID_HOST]++;
 		return FALSE;			/* Is host valid? */
     }
@@ -569,7 +606,7 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 	 * entry from one cache to another.
 	 */
 
-	if (hcache_ht_get(ip, port, &host, &hce)) {
+	if (hcache_ht_get(addr, port, &host, &hce)) {
         g_assert(hce != NULL);
 
         hc->hits++;
@@ -625,15 +662,15 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 		caches[hce->type]->dirty = hc->dirty = TRUE;
 
 		hce->type = type;
-		hce->time_added = time(NULL);
+		hce->time_added = tm_time();
 
 		return TRUE;
     }
 
 	/* Okay, we got a new host */
-	host = (gnet_host_t *) walloc(sizeof(*host));
+	host = walloc(sizeof *host);
 
-	host->ip   = ip;
+	host->addr = addr;
 	host->port = port;
 
 	hcache_ht_add(type, host);
@@ -657,7 +694,7 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
     default:
         /*
          * hcache_expire() depends on the fact that new entries are
-         * added to the beginning of the list 
+         * added to the beginning of the list
          */
         hash_list_prepend(hc->hostlist, host);
         break;
@@ -677,8 +714,9 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
     hcache_update_low_on_pongs();
 
     if (dbg > 8) {
-        printf("Added %s %s (%s)\n", what, ip_port_to_gchar(ip, port),
-            ((type == HCACHE_FRESH_ANY) || (type == HCACHE_VALID_ANY)) ? 
+        printf("Added %s %s (%s)\n",
+			what, host_addr_port_to_string(addr, port),
+            ((type == HCACHE_FRESH_ANY) || (type == HCACHE_VALID_ANY)) ?
                 (host_low_on_pongs ? "LOW" : "OK") : "");
     }
 
@@ -689,13 +727,14 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
  * Add a caught (fresh) host to the right list depending on the host type.
  */
 gboolean
-hcache_add_caught(host_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add_caught(host_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
     switch (type) {
     case HOST_ANY:
-    	return hcache_add(HCACHE_FRESH_ANY, ip, port, what);
+    	return hcache_add(HCACHE_FRESH_ANY, addr, port, what);
     case HOST_ULTRA:
-    	return hcache_add(HCACHE_FRESH_ULTRA, ip, port, what);
+    	return hcache_add(HCACHE_FRESH_ULTRA, addr, port, what);
     case HOST_MAX:
 		g_assert_not_reached();
     }
@@ -708,13 +747,14 @@ hcache_add_caught(host_type_t type, guint32 ip, guint16 port, const gchar *what)
  * Add a valid host to the right list depending on the host type.
  */
 gboolean
-hcache_add_valid(host_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add_valid(host_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
     switch (type) {
     case HOST_ANY:
-    	return hcache_add(HCACHE_VALID_ANY, ip, port, what);
+    	return hcache_add(HCACHE_VALID_ANY, addr, port, what);
     case HOST_ULTRA:
-    	return hcache_add(HCACHE_VALID_ULTRA, ip, port, what);
+    	return hcache_add(HCACHE_VALID_ULTRA, addr, port, what);
     case HOST_MAX:
 		g_assert_not_reached();
     }
@@ -733,11 +773,11 @@ hcache_remove(gnet_host_t *h)
 {
     hostcache_entry_t *hce;
     hostcache_t *hc;
-    
+
     hce = hcache_get_metadata(h);
     if (hce == NULL) {
 		g_warning("hcache_remove: attempt to remove unknown host: %s",
-				  ip_port_to_gchar(h->ip, h->port));           
+				  host_addr_port_to_string(h->addr, h->port));
         return; /* Host is not in hashtable */
     }
 
@@ -794,7 +834,7 @@ hcache_remove_all(hostcache_t *hc)
 }
 
 /**
- * Clear the whole host cache for a host type and the pong cache of 
+ * Clear the whole host cache for a host type and the pong cache of
  * the same type. Use this to clear the "ultra" and "any" host caches.
  */
 void
@@ -862,9 +902,8 @@ hcache_size(host_type_t type)
  * @return total number of expired entries
  */
 static guint32
-hcache_expire_cache(hostcache_t *hc)
+hcache_expire_cache(hostcache_t *hc, time_t now)
 {
-    time_t now = time((time_t *) NULL);
     gint32 secs_to_keep = 60 * 30; /* 30 minutes */
     guint32 expire_count = 0;
 	gnet_host_t *h;
@@ -900,14 +939,14 @@ hcache_expire_cache(hostcache_t *hc)
  *
  * @return total number of expired entries
  */
-guint32
-hcache_expire_all(void)
+static guint32
+hcache_expire_all(time_t now)
 {
     guint32 expire_count = 0;
 
-    expire_count += hcache_expire_cache(caches[HCACHE_TIMEOUT]);
-    expire_count += hcache_expire_cache(caches[HCACHE_BUSY]);
-    expire_count += hcache_expire_cache(caches[HCACHE_UNSTABLE]);
+    expire_count += hcache_expire_cache(caches[HCACHE_TIMEOUT], now);
+    expire_count += hcache_expire_cache(caches[HCACHE_BUSY], now);
+    expire_count += hcache_expire_cache(caches[HCACHE_UNSTABLE], now);
 
     return expire_count;
 }
@@ -922,7 +961,7 @@ hcache_expire_all(void)
  * to HCACHE_FRESH_XXX and then it is purged.
  */
 void
-hcache_prune(hcache_type_t type) 
+hcache_prune(hcache_type_t type)
 {
 	hostcache_t *hc;
     gint extra;
@@ -961,7 +1000,7 @@ hcache_prune(hcache_type_t type)
         return;
 
     start_mass_update(hc);
-    
+
     hcache_require_caught(hc);
 	while (extra > 0) {
 		gnet_host_t *h = hash_list_first(hc->hostlist);
@@ -1015,7 +1054,7 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
 	for (i = 0; i < hcount; i++) {
 		gnet_host_t host;
 
-		if (!pcache_get_recent(type, &host.ip, &host.port))
+		if (!pcache_get_recent(type, &host.addr, &host.port))
 			break;
 
 		if (g_hash_table_lookup(seen_host, &host))
@@ -1036,8 +1075,8 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
 	iter = hash_list_iterator_last(hc->hostlist);
 
 	for (
-        h = hash_list_previous(iter); 
-        i < hcount && h != NULL; 
+        h = hash_list_previous(iter);
+        i < hcount && h != NULL;
         i++, h = hash_list_previous(iter)
     ) {
 		if (g_hash_table_lookup(seen_host, h))
@@ -1057,17 +1096,17 @@ done:
 }
 
 /**
- * Finds a host in either the pong_cache or the host_cache that is in 
- * one of the local networks. 
+ * Finds a host in either the pong_cache or the host_cache that is in
+ * one of the local networks.
  *
  * @return TRUE if host is found
  */
 gboolean
-hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
+hcache_find_nearby(host_type_t type, host_addr_t *addr, guint16 *port)
 {
 	gnet_host_t *h;
 	static int alternate = 0;
-	guint32 first_ip;
+	host_addr_t first_addr;
 	guint16 first_port;
 	gboolean got_recent;
 	hostcache_t *hc = NULL;
@@ -1086,39 +1125,39 @@ hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
 		g_assert_not_reached();
     }
 
-	if (!hc)	
+	if (!hc)
         g_error("hcache_get_caught: unknown host type: %d", type);
 
     gnet_prop_get_boolean_val(hc->reading, &reading);
 
 	if (alternate++ & 1) {
 		/* Iterate through all recent pongs */
-		for (
-			*ip = 0,
-				got_recent = pcache_get_recent(type, &first_ip, &first_port);
-			got_recent && (*ip != first_ip || *port != first_port); 
-			got_recent = pcache_get_recent(type, ip, port)
-		) {
-			if (host_is_nearby(*ip))
+
+		*addr = zero_host_addr;
+		got_recent = pcache_get_recent(type, &first_addr, &first_port);
+
+		while (got_recent) {
+			if (host_addr_equal(*addr, first_addr) && *port == first_port)
+				break;
+
+			if (host_is_nearby(*addr))
 				return TRUE;
+
+			got_recent = pcache_get_recent(type, addr, port);
 		}
 	}
 
 	/* iterate through whole list */
 
-	iter = reading ?  hash_list_iterator(hc->hostlist) :
+	iter = reading ? hash_list_iterator(hc->hostlist) :
 		hash_list_iterator_last(hc->hostlist);
 
-	for (
-		h = hash_list_follower(iter);
-		NULL != h; 
-        h = hash_list_follower(iter)
-	) {
+	for (h = hash_list_follower(iter); h; h = hash_list_follower(iter)) {
 
-		if (host_is_nearby(h->ip)) {
-            *ip = h->ip;
+		if (host_is_nearby(h->addr)) {
+            *addr = h->addr;
             *port = h->port;
-            
+
             result = TRUE;
 			break;
 		}
@@ -1137,7 +1176,7 @@ hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
  * recent pong cache, in alternance.
  */
 void
-hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
+hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
 {
 	static guint alternate = 0;
 	hostcache_t *hc = NULL;
@@ -1167,13 +1206,13 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 
     hcache_update_low_on_pongs();
 
-	/* 
-	 * First, try to find a local host 
+	/*
+	 * First, try to find a local host
 	 */
 
 	if (
 		use_netmasks && number_local_networks &&
-		hcache_find_nearby(type, ip, port)
+		hcache_find_nearby(type, addr, port)
 	)
 		return;
 
@@ -1181,7 +1220,7 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 	 * Try the recent pong cache when `alternate' is odd.
 	 */
 
-	if (alternate++ & 0x1 && pcache_get_recent(type, ip, port))
+	if (alternate++ & 0x1 && pcache_get_recent(type, addr, port))
 		return;
 
 	/*
@@ -1191,7 +1230,7 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 
 	h = reading ? hash_list_first(hc->hostlist) : hash_list_last(hc->hostlist);
 
-	*ip = h->ip;
+	*addr = h->addr;
 	*port = h->port;
 
     hcache_unlink_host(hc, h);
@@ -1221,14 +1260,14 @@ hcache_alloc(hcache_type_t type, gnet_property_t reading,
 
 	g_assert((guint) type < HCACHE_MAX);
 
-	hc = g_malloc0(sizeof(*hc));
+	hc = g_malloc0(sizeof *hc);
 
 	hc->hostlist = hash_list_new();
 	hc->name = name;
 	hc->type = type;
 	hc->reading = reading;
     hc->hosts_in_catcher = catcher;
-    hc->ip_only = FALSE;
+    hc->addr_only = FALSE;
 
 	return hc;
 }
@@ -1256,12 +1295,12 @@ hcache_free(hostcache_t *hc)
  */
 
 #define READ_MAGIC		0x3d00003d
-#define HOST_READ_CNT	20			/* Amount of hosts to read each tick */
+#define HOST_READ_CNT	20			/**< Amount of hosts to read each tick */
 
 struct read_ctx {
-	gint magic;						/* Magic number */
-	FILE *fd;						/* File descriptor to read from */
-	hostcache_t *hc;				/* Hostcache to fill */
+	gint magic;						/**< Magic number */
+	FILE *fd;						/**< File descriptor to read from */
+	hostcache_t *hc;				/**< Hostcache to fill */
 };
 
 /**
@@ -1292,7 +1331,7 @@ read_done(hostcache_t *hc)
      * hosts in the catcher.
      *      -- Richard, 6/8/2002
      */
-    
+
     gnet_prop_set_boolean_val(hc->reading, FALSE);
 }
 
@@ -1302,7 +1341,7 @@ read_done(hostcache_t *hc)
 static bgret_t
 read_step(gpointer unused_h, gpointer u, gint ticks)
 {
-	struct read_ctx *rctx = (struct read_ctx *) u;
+	struct read_ctx *rctx = u;
 	hostcache_t *hc;
 	gint max_read;
 	gint count;
@@ -1310,7 +1349,7 @@ read_step(gpointer unused_h, gpointer u, gint ticks)
 	static gchar h_tmp[1024];
 
 	(void) unused_h;
-	g_assert(rctx->magic == READ_MAGIC);
+	g_assert(READ_MAGIC == rctx->magic);
 	g_assert(rctx->fd);
 
 	hc = rctx->hc;
@@ -1324,11 +1363,11 @@ read_step(gpointer unused_h, gpointer u, gint ticks)
 
 	for (i = 0; i < count; i++) {
 		if (fgets(h_tmp, sizeof(h_tmp) - 1, rctx->fd)) { /* NUL appended */
-			guint32 ip;
+			host_addr_t addr;
 			guint16 port;
 
-			if (gchar_to_ip_port(h_tmp, &ip, &port))
-                hcache_add(hc->type, ip, port, "on-disk cache");
+			if (string_to_host_addr_port(h_tmp, NULL, &addr, &port))
+                hcache_add(hc->type, addr, port, "on-disk cache");
 		} else
 			goto done;
 	}
@@ -1378,7 +1417,7 @@ hcache_retrieve(hostcache_t *hc, const gchar *filename)
 
     {
 		file_path_t fp;
-		
+
 		file_path_set(&fp, settings_config_dir(), filename);
 		fd = file_config_open_read("hosts", &fp, 1);
 	}
@@ -1411,7 +1450,7 @@ hcache_write(FILE *f, hostcache_t *hc)
 	iter = hash_list_iterator(hc->hostlist);
 
 	for (h = hash_list_next(iter); h != NULL; h = hash_list_next(iter))
-		fprintf(f, "%s\n", ip_port_to_gchar(h->ip, h->port));
+		fprintf(f, "%s\n", host_addr_port_to_string(h->addr, h->port));
 
 	hash_list_release(iter);
 }
@@ -1431,9 +1470,7 @@ hcache_store(hcache_type_t type, const gchar *filename, hcache_type_t extra)
 	g_assert(caches[type] != NULL);
 	g_assert(extra == HCACHE_NONE || caches[extra] != NULL);
 
-	fp.dir = settings_config_dir();
-	fp.name = filename;
-
+	file_path_set(&fp, settings_config_dir(), filename);
 	f = file_config_open_write(filename, &fp);
 
 	if (!f)
@@ -1450,20 +1487,20 @@ hcache_store(hcache_type_t type, const gchar *filename, hcache_type_t extra)
 /**
  * Get statistical information about the caches.
  *
- * @param stats must point to an hcache_stats_t[HCACHE_MAX] array.
+ * @param s must point to an hcache_stats_t[HCACHE_MAX] array.
  */
 void
-hcache_get_stats(hcache_stats_t *stats)
+hcache_get_stats(hcache_stats_t *s)
 {
     guint n;
 
     for (n = 0; n < HCACHE_MAX; n++) {
 		if (n == HCACHE_NONE)
 			continue;
-        stats[n].host_count = caches[n]->host_count;
-        stats[n].hits       = caches[n]->hits;
-        stats[n].misses     = caches[n]->misses;
-        stats[n].reading    = FALSE;
+        s[n].host_count = caches[n]->host_count;
+        s[n].hits       = caches[n]->hits;
+        s[n].misses     = caches[n]->misses;
+        s[n].reading    = FALSE;
     }
 }
 
@@ -1471,20 +1508,20 @@ hcache_get_stats(hcache_stats_t *stats)
  * Host cache timer.
  */
 void
-hcache_timer(void)
+hcache_timer(time_t now)
 {
-    hcache_expire_all();
+    hcache_expire_all(now);
 
     if (dbg >= 15) {
-        hcache_dump_info(caches[HCACHE_FRESH_ANY],   "timer");    
-        hcache_dump_info(caches[HCACHE_VALID_ANY],   "timer");    
+        hcache_dump_info(caches[HCACHE_FRESH_ANY],   "timer");
+        hcache_dump_info(caches[HCACHE_VALID_ANY],   "timer");
 
-        hcache_dump_info(caches[HCACHE_FRESH_ULTRA], "timer");    
-        hcache_dump_info(caches[HCACHE_VALID_ULTRA], "timer");    
+        hcache_dump_info(caches[HCACHE_FRESH_ULTRA], "timer");
+        hcache_dump_info(caches[HCACHE_VALID_ULTRA], "timer");
 
-        hcache_dump_info(caches[HCACHE_TIMEOUT],  "timer");    
-        hcache_dump_info(caches[HCACHE_BUSY],     "timer");    
-        hcache_dump_info(caches[HCACHE_UNSTABLE], "timer");    
+        hcache_dump_info(caches[HCACHE_TIMEOUT],  "timer");
+        hcache_dump_info(caches[HCACHE_BUSY],     "timer");
+        hcache_dump_info(caches[HCACHE_UNSTABLE], "timer");
 
         g_message("Hcache global: local %u   alrdy connected %u   invalid %u",
             stats[HCACHE_LOCAL_INSTANCE], stats[HCACHE_ALREADY_CONNECTED],
@@ -1514,32 +1551,32 @@ hcache_init(void)
         "hosts.fresh.ultra");
 
     caches[HCACHE_VALID_ANY] = hcache_alloc(
-        HCACHE_VALID_ANY, PROP_READING_HOSTFILE, 
+        HCACHE_VALID_ANY, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_CATCHER,
         "hosts.valid.any");
 
     caches[HCACHE_VALID_ULTRA] = hcache_alloc(
-        HCACHE_VALID_ULTRA, PROP_READING_ULTRAFILE, 
+        HCACHE_VALID_ULTRA, PROP_READING_ULTRAFILE,
         PROP_HOSTS_IN_ULTRA_CATCHER,
         "hosts.valid.ultra");
 
     caches[HCACHE_TIMEOUT] = hcache_alloc(
-        HCACHE_TIMEOUT, PROP_READING_HOSTFILE, 
+        HCACHE_TIMEOUT, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.timeout");
-    caches[HCACHE_TIMEOUT]->ip_only = TRUE;
+    caches[HCACHE_TIMEOUT]->addr_only = TRUE;
 
     caches[HCACHE_BUSY] = hcache_alloc(
         HCACHE_BUSY, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.busy");
-    caches[HCACHE_BUSY]->ip_only = TRUE;
+    caches[HCACHE_BUSY]->addr_only = TRUE;
 
     caches[HCACHE_UNSTABLE] = hcache_alloc(
         HCACHE_UNSTABLE, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.unstable");
-    caches[HCACHE_UNSTABLE]->ip_only = TRUE;
+    caches[HCACHE_UNSTABLE]->addr_only = TRUE;
 }
 
 /**
@@ -1659,8 +1696,8 @@ hcache_shutdown(void)
  */
 void hcache_close(void)
 {
-	static const hcache_type_t types[] = { 
-        HCACHE_FRESH_ANY, 
+	static const hcache_type_t types[] = {
+        HCACHE_FRESH_ANY,
         HCACHE_VALID_ANY,
         HCACHE_FRESH_ULTRA,
         HCACHE_VALID_ULTRA,
@@ -1670,12 +1707,12 @@ void hcache_close(void)
     };
 	guint i;
 
-	g_assert(!hcache_close_running);	
-	hcache_close_running = TRUE;	
-	
+	g_assert(!hcache_close_running);
+	hcache_close_running = TRUE;
+
     /*
-     * First we stop all background processes and remove all hosts, 
-     * only then we free the hcaches. This is important because 
+     * First we stop all background processes and remove all hosts,
+     * only then we free the hcaches. This is important because
      * hcache_require_caught will crash if we free certain hostcaches.
      */
 
@@ -1689,9 +1726,9 @@ void hcache_close(void)
 		/* Make sure all previous caches have been cleared */
 		for (j = 0; j < type; j++)
     		g_assert(caches[j]->host_count == 0);
-		
+
         hcache_remove_all(caches[type]);
-		
+
 		/* Make sure no caches have been refilled */
 		for (j = 0; j <= type; j++)
     		g_assert(caches[j]->host_count == 0);
