@@ -1,10 +1,8 @@
 /*
- * $Id: move.c,v 1.3 2004/10/24 01:32:21 cbiere Exp $
+ * $Id: move.c,v 1.7 2005/08/31 21:37:28 rmanfredi Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Asychronous file moving operations.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Asychronous file moving operations.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: move.c,v 1.3 2004/10/24 01:32:21 cbiere Exp $");
+RCSID("$Id: move.c,v 1.7 2005/08/31 21:37:28 rmanfredi Exp $");
 
 #include "downloads.h"
 #include "fileinfo.h"
@@ -36,46 +44,47 @@ RCSID("$Id: move.c,v 1.3 2004/10/24 01:32:21 cbiere Exp $");
 #include "lib/atoms.h"
 #include "lib/bg.h"
 #include "lib/file.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/override.h"	/* Must be the last header included */
 
-#define COPY_BLOCK_SHIFT	12			/* Power of two of copy unit credit */
-#define COPY_BUF_SIZE		65536		/* Size of the reading buffer */
+#define COPY_BLOCK_SHIFT	12			/**< Power of two of copy unit credit */
+#define COPY_BUF_SIZE		65536		/**< Size of the reading buffer */
 
 static gpointer move_daemon = NULL;
 
 #define MOVED_MAGIC	0x00c0b100
 
-/*
+/**
  * Moving daemon context.
  */
 struct moved {
-	gint magic;				/* Magic number */
-	struct download *d;		/* Download for which we're moving file */
-	gint rd;				/* Opened file descriptor for read, -1 if none */
-	gint wd;				/* Opened file descriptor for write, -1 if none */
-	time_t start;			/* Start time, to determine copying rate */
-	filesize_t size;		/* Size of file */
-	filesize_t copied;		/* Amount of data copied so far */
-	gchar *buffer;			/* Large buffer, where data is read */
-	gchar *target;			/* Target file name, in case an error occurs */
-	gint error;				/* Error code */
+	gint magic;				/**< Magic number */
+	struct download *d;		/**< Download for which we're moving file */
+	gint rd;				/**< Opened file descriptor for read, -1 if none */
+	gint wd;				/**< Opened file descriptor for write, -1 if none */
+	time_t start;			/**< Start time, to determine copying rate */
+	filesize_t size;		/**< Size of file */
+	filesize_t copied;		/**< Amount of data copied so far */
+	gchar *buffer;			/**< Large buffer, where data is read */
+	gchar *target;			/**< Target file name, in case an error occurs */
+	gint error;				/**< Error code */
 };
 
-/*
+/**
  * Work queue entry.
  */
 struct work {
-	struct download *d;		/* Download to move */
-	gchar *dest;			/* Target directory (atom) */
-	gchar *ext;				/* Trailing extension (atom) */
+	struct download *d;		/**< Download to move */
+	gchar *dest;			/**< Target directory (atom) */
+	gchar *ext;				/**< Trailing extension (atom) */
 };
 
-/*
+/**
  * we_alloc
  *
  * Allocate work queue entry.
@@ -93,7 +102,7 @@ we_alloc(struct download *d, const gchar *dest, const gchar *ext)
 	return we;
 }
 
-/*
+/**
  * we_free
  *
  * Freeing of work queue entry.
@@ -135,7 +144,7 @@ d_sighandler(gpointer unused_h, gpointer u, bgsig_t sig)
 	}
 }
 
-/*
+/**
  * d_free
  *
  * Freeing of computation context.
@@ -225,7 +234,7 @@ d_start(gpointer h, gpointer ctx, gpointer item)
 	if (md->wd == -1)
 		goto abort_read;
 
-	md->start = time(NULL);
+	md->start = tm_time();
 	md->size = download_filesize(d);
 	md->copied = 0;
 	md->error = 0;
@@ -286,7 +295,7 @@ d_end(gpointer h, gpointer ctx, gpointer item)
 		gchar *source;
 
 		g_assert(md->copied == md->size);
-		
+
 		source = make_pathname(download_path(md->d), download_outname(md->d));
 		if (NULL == source || -1 == unlink(source))
 			g_warning("cannot unlink \"%s\": %s",
@@ -299,7 +308,7 @@ d_end(gpointer h, gpointer ctx, gpointer item)
 				md->target, g_strerror(errno));
 	}
 
-	elapsed = delta_time(time(NULL), md->start);
+	elapsed = delta_time(tm_time(), md->start);
 	elapsed = MAX(1, elapsed);		/* time warp? clock not monotic? */
 
 	if (dbg > 1)
