@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c,v 1.5 2004/10/20 15:38:57 cbiere Exp $
+ * $Id: mq.c,v 1.12 2005/08/07 22:32:09 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -24,14 +24,18 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Message queues, common code between TCP and UDP sending stacks.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: mq.c,v 1.5 2004/10/20 15:38:57 cbiere Exp $");
+RCSID("$Id: mq.c,v 1.12 2005/08/07 22:32:09 cbiere Exp $");
 
 #include "nodes.h"
 #include "mq.h"
@@ -218,7 +222,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * message at the tail of the queue, even if it is less prioritary
 		 * than our comparison point.
 		 */
-			
+
 		q->header.function = GTA_MSG_SEARCH_RESULTS;
 
 		/*
@@ -318,7 +322,7 @@ mq_enter_flowc(mqueue_t *q)
 
 	if (dbg > 4)
 		printf("entering FLOWC for node %s (%d bytes queued)\n",
-			node_ip(q->node), q->size);
+			node_addr(q->node), q->size);
 }
 
 /**
@@ -332,7 +336,7 @@ mq_leave_flowc(mqueue_t *q)
 	if (dbg > 4)
 		printf("leaving %s for node %s (%d bytes queued)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			node_ip(q->node), q->size);
+			node_addr(q->node), q->size);
 
 	q->flags &= ~(MQ_FLOWC|MQ_SWIFT);	/* Under low watermark, clear */
 	if (q->qlink)
@@ -423,13 +427,24 @@ mq_clear(mqueue_t *q)
  * Forbid further writes to the queue.
  */
 void
-mq_shutdown(mqueue_t *q)
+mq_discard(mqueue_t *q)
 {
 	g_assert(q);
 
 	q->flags |= MQ_DISCARD;
 }
 
+/**
+ * Disable all further writes from the queue.
+ */
+void
+mq_shutdown(mqueue_t *q)
+{
+	g_assert(q);
+
+	tx_shutdown(q->tx_drv);		/* No further output will be made */
+}
+
 /**
  * Compare two pointers to links based on their relative priorities, then
  * based on their held Gnutella messages.
@@ -737,7 +752,7 @@ qlink_remove(mqueue_t *q, GList *l)
  * up to the caller to initialize it with -1 and check whether it has been
  * set.
  *
- * Returns TRUE if we were able to make enough room.
+ * @returns TRUE if we were able to make enough room.
  */
 static gboolean
 make_room(mqueue_t *q, pmsg_t *mb, gint needed, gint *offset)
@@ -764,7 +779,7 @@ make_room_header(
 	if (dbg > 5)
 		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			needed, (gulong) q, node_ip(q->node));
+			needed, (gulong) q, node_addr(q->node));
 
 	if (q->qhead == NULL)			/* Queue is empty */
 		return FALSE;
@@ -853,7 +868,7 @@ make_room_header(
 			gmsg_log_dropped(pmsg_start(cmb),
 				"to %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-				node_ip(q->node), gmsg_infostr(header));
+				node_addr(q->node), gmsg_infostr(header));
 
 		gnet_stats_count_flowc(pmsg_start(cmb));
 		cmb_size = pmsg_size(cmb);
@@ -870,7 +885,7 @@ make_room_header(
 	if (dbg > 5)
 		printf("%s end purge: %d bytes (count=%d) for node %s, need=%d\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			q->size, q->count, node_ip(q->node), needed);
+			q->size, q->count, node_addr(q->node), needed);
 
 	/*
 	 * In case we emptied the whole queue, disable servicing.
@@ -928,7 +943,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		if (dbg > 4)
 			gmsg_log_dropped(pmsg_start(mb),
 				"to FLOWC node %s, %d bytes queued",
-				node_ip(q->node), q->size);
+				node_addr(q->node), q->size);
 
 		gnet_stats_count_flowc(pmsg_start(mb));
 		pmsg_free(mb);
@@ -966,16 +981,24 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 			if (dbg > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [FULL]",
-					node_ip(q->node), q->size);
+					node_addr(q->node), q->size);
 
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
 			if (dbg > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [KILLING]",
-					node_ip(q->node), q->size);
+					node_addr(q->node), q->size);
 
-			node_bye(q->node, 502, "Send queue reached %d bytes", q->maxsize);
+			/* XXX: Is the check for UDP the correct fix or just a
+			 *		workaround? node_bye_v() asserts that the node isn't
+			 *		the UDP node.
+			 *		-- cbiere, 2004-12-20
+			 */
+			if (!NODE_IS_UDP(q->node)) {
+				node_bye(q->node, 502, "Send queue reached %d bytes",
+					q->maxsize);
+			}
 		}
 
 		pmsg_free(mb);
@@ -1012,7 +1035,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 
 		for (l = q->qtail; l; l = l->prev) {
 			pmsg_t *m = (pmsg_t *) l->data;
-			
+
 			if (
 				pmsg_is_unread(m) &&			/* Not partially written */
 				pmsg_prio(m) < prio				/* Reached insert point */
@@ -1099,10 +1122,10 @@ mq_putq(mqueue_t *q, pmsg_t *mb)
 }
 
 static const struct mq_cops mq_cops = {
-	mq_puthere,				/* puthere */
-	qlink_remove,			/* qlink_remove */
-	mq_rmlink_prev,			/* rmlink_prev */
-	mq_update_flowc,		/* update_flowc */
+	mq_puthere,				/**< puthere */
+	qlink_remove,			/**< qlink_remove */
+	mq_rmlink_prev,			/**< rmlink_prev */
+	mq_update_flowc,		/**< update_flowc */
 };
 
 /**
