@@ -1,5 +1,5 @@
 /*
- * $Id: mq_tcp.c,v 1.2 2004/10/03 10:04:12 rmanfredi Exp $
+ * $Id: mq_tcp.c,v 1.10 2005/12/17 09:25:44 daichik Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -24,14 +24,18 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Message queues, writing to a TCP stack.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: mq_tcp.c,v 1.2 2004/10/03 10:04:12 rmanfredi Exp $");
+RCSID("$Id: mq_tcp.c,v 1.10 2005/12/17 09:25:44 daichik Exp $");
 
 #include "nodes.h"
 #include "mq.h"
@@ -47,23 +51,27 @@ RCSID("$Id: mq_tcp.c,v 1.2 2004/10/03 10:04:12 rmanfredi Exp $");
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define MQ_MAXIOV		256		/* Our limit on the I/O vectors we build */
-#define MQ_MINIOV		2		/* Minimum amount of I/O vectors in service */
-#define MQ_MINSEND		256		/* Minimum size we try to send */
+#define MQ_MAXIOV		256		/**< Our limit on the I/O vectors we build */
+#define MQ_MINIOV		2		/**< Minimum amount of I/O vectors in service */
+#define MQ_MINSEND		256		/**< Minimum size we try to send */
 
 static void mq_tcp_service(gpointer data);
-static struct mq_ops mq_tcp_ops;
+static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb);
+static const struct mq_ops mq_tcp_ops = {
+	mq_tcp_putq,			/* putq */
+};
+
 
 /**
  * Create new message queue capable of holding `maxsize' bytes, and
  * owned by the supplied node.
  */
-mqueue_t *mq_tcp_make(
-	gint maxsize, struct gnutella_node *n, struct txdriver *nd)
+mqueue_t *
+mq_tcp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 {
 	mqueue_t *q;
 
-	q = walloc0(sizeof(*q));
+	q = walloc0(sizeof *q);
 
 	q->node = n;
 	q->tx_drv = nd;
@@ -78,19 +86,18 @@ mqueue_t *mq_tcp_make(
 	return q;
 }
 
-/*
- * mq_tcp_service
- *
+/**
  * Service routine for TCP message queue.
  */
-static void mq_tcp_service(gpointer data)
+static void
+mq_tcp_service(gpointer data)
 {
 	mqueue_t *q = (mqueue_t *) data;
 	static struct iovec iov[MQ_MAXIOV];
 	gint iovsize;
 	gint iovcnt;
 	gint sent;
-	gint r;
+	ssize_t r;
 	GList *l;
 	gint dropped;
 	gint maxsize;
@@ -171,15 +178,20 @@ again:
 
 	r = tx_writev(q->tx_drv, iov, iovcnt);
 
-	if (r <= 0) {
+	g_assert((ssize_t) -1 == r || !tx_has_error(q->tx_drv));
+
+	if ((ssize_t) -1 == r || r == 0) {
 		q->last_written = 0;
 		if (r == 0)
 			goto update_servicing;
 		return;
 	}
 
-	if (has_prioritary)
+	if (has_prioritary) {
 		tx_flush(q->tx_drv);
+		if (tx_has_error(q->tx_drv))
+			return;
+	}
 
 	node_add_tx_given(q->node, r);
 	q->last_written = r;
@@ -279,7 +291,8 @@ update_servicing:
 /**
  * Enqueue message, which becomes owned by the queue.
  */
-static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
+static void
+mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 {
 	gint size = pmsg_size(mb);
 	gchar *mbs = pmsg_start(mb);
@@ -308,7 +321,7 @@ static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 	 */
 
 	if (q->qhead == NULL) {
-		gint written;
+		ssize_t written;
 
 		if (pmsg_check(mb, q)) {
 			if (prioritary)
@@ -316,9 +329,19 @@ static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 
 			written = tx_write(q->tx_drv, mbs, size);
 
+			/**
+			 * FIXME: The following assertion check failed.
+			 *			--cbiere, 2005-11-01
+			 */
+			g_assert((ssize_t) -1 == written || !tx_has_error(q->tx_drv));
+
+			if ((ssize_t) -1 == written)
+				goto cleanup;
+
 			if (prioritary && written == size) {
 				tx_flush(q->tx_drv);
-				node_unflushq(q->node);
+				if (tx_has_error(q->tx_drv))
+					goto cleanup;
 			}
 		} else {
 			gnet_stats_count_flowc(mbs);
@@ -366,8 +389,4 @@ cleanup:
 	return;
 }
 
-static struct mq_ops mq_tcp_ops = {
-	mq_tcp_putq,			/* putq */
-};
-
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
