@@ -1,10 +1,8 @@
 /*
- * $Id: rx_link.h,v 1.2 2004/09/03 15:05:07 cbiere Exp $
+ * $Id: rx_link.h,v 1.6 2005/11/26 16:29:07 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Network driver -- link layer.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,6 +23,16 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Network driver -- link layer.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_rx_link_h_
 #define _core_rx_link_h_
 
@@ -32,6 +40,25 @@
 
 const struct rxdrv_ops *rx_link_get_ops(void);
 
+/**
+ * Callbacks used by the link layer.
+ */
+struct rx_link_cb {
+	void (*add_rx_given)(gpointer owner, gint amount);
+	void (*read_error)(gpointer owner,
+			const gchar *reason, ...) PRINTF_FUNC_PTR(2, 3);
+	void (*got_eof)(gpointer owner);
+};
+
+/**
+ * Arguments to be passed when the layer is intantiated.
+ */
+struct rx_link_args {
+	const struct rx_link_cb *cb;	/**< Callbacks */
+	struct bsched *bs;				/**< Bandwidth scheduler to use */
+	struct wrap_io *wio;			/**< I/O wrapping routines */
+};
+
 #endif	/* _core_rx_link_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
