@@ -1,7 +1,7 @@
 /*
- * $Id: shell.c,v 1.4 2004/10/25 04:29:16 cbiere Exp $
+ * $Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $
  *
- *   Copyright (c) 2002-2003, Richard Eckart
+ * Copyright (c) 2002-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -27,7 +27,7 @@
 
 #ifdef USE_REMOTE_CTRL
 
-RCSID("$Id: shell.c,v 1.4 2004/10/25 04:29:16 cbiere Exp $");
+RCSID("$Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $");
 
 #include "shell.h"
 #include "sockets.h"
@@ -37,6 +37,8 @@ RCSID("$Id: shell.c,v 1.4 2004/10/25 04:29:16 cbiere Exp $");
 #include "version.h"
 
 #include "if/bridge/c2ui.h"
+#include "if/bridge/ui2c.h"
+
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
@@ -44,10 +46,12 @@ RCSID("$Id: shell.c,v 1.4 2004/10/25 04:29:16 cbiere Exp $");
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/inputevt.h"
+#include "lib/tm.h"
+#include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 #define CMD_MAX_SIZE 1024
-#define OUTPUT_BUFFER_SIZE 64000
+#define SHELL_BUFFER_SIZE 64000
 
 #define IS_PROCESSING(sh) (sh->outpos > 0)
 #define SHELL_TIMEOUT 60
@@ -60,27 +64,26 @@ static GSList *sl_shells = NULL;
 
 typedef struct gnutella_shell {
 	struct gnutella_socket *socket;
-	gchar  outbuf[OUTPUT_BUFFER_SIZE];
+	gchar  *outbuf;	/* FIXME: Always use a newly walloc()ed buffer instead */
 	gint32 outpos;
-	guint  write_tag;
-	time_t last_update; /* Last update (needed for timeout) */
-	gchar *msg;         /* Additional information to reply code */
-	gboolean shutdown;  /* In shutdown mode? */
+	time_t last_update; /**< Last update (needed for timeout) */
+	const gchar *msg;   /**< Additional information to reply code */
+	gboolean shutdown;  /**< In shutdown mode? */
 } gnutella_shell_t;
 
-/* Don't refer to OUTPUT_BUFFER_SIZE, use sizeof */
-#undef OUTPUT_BUFFER_SIZE
-
 static gchar auth_cookie[SHA1_RAW_SIZE];
 
+static void shell_shutdown(gnutella_shell_t *sh);
 static void shell_destroy(gnutella_shell_t *sh);
-void shell_shutdown(gnutella_shell_t *sh);
 static gboolean shell_write(gnutella_shell_t *sh, const gchar *s);
-void print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
-	int triples, hsep_triple *nonhsep);
+static void print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
+	int triples, hsep_triple *non_hsep);
+static void shell_handle_data(gpointer data, gint unused_source,
+	inputevt_cond_t cond);
 
 enum {
 	CMD_UNKNOWN,
+	CMD_NOOP,
 	CMD_QUIT,
 	CMD_SEARCH,
 	CMD_NODE,
@@ -89,40 +92,42 @@ enum {
 	CMD_PRINT,
 	CMD_SET,
 	CMD_WHATIS,
-	CMD_HORIZON
+	CMD_HORIZON,
+	CMD_RESCAN
 };
 
 static const struct {
 	const gint id;
 	const gchar * const cmd;
 } commands[] = {
-	{CMD_QUIT,   "QUIT"},
-	{CMD_SEARCH, "SEARCH"},
-	{CMD_NODE,   "NODE"},
-	{CMD_ADD,    "ADD"},
-	{CMD_HELP,   "HELP"},
-	{CMD_PRINT,  "PRINT"},
-	{CMD_SET,    "SET"},
-	{CMD_WHATIS, "WHATIS"},
-	{CMD_HORIZON, "HORIZON"}
+	{	CMD_QUIT,		"QUIT"		},
+	{	CMD_SEARCH,		"SEARCH"	},
+	{	CMD_NODE,		"NODE"		},
+	{	CMD_ADD,		"ADD"		},
+	{	CMD_HELP,		"HELP"		},
+	{	CMD_PRINT,		"PRINT"		},
+	{	CMD_SET,		"SET"		},
+	{	CMD_WHATIS,		"WHATIS"	},
+	{	CMD_HORIZON,	"HORIZON"	},
+	{	CMD_RESCAN,		"RESCAN"	}
 };
 
 
 static gint
 get_command(const gchar *cmd)
 {
-	guint n;
+	guint i;
 
-	for (n = 0; n < G_N_ELEMENTS(commands); n ++) {
-		if (g_ascii_strcasecmp(commands[n].cmd, cmd) == 0)
-			return commands[n].id;
+	for (i = 0; i < G_N_ELEMENTS(commands); i++) {
+		if (ascii_strcasecmp(commands[i].cmd, cmd) == 0)
+			return commands[i].id;
 	}
 
 	return CMD_UNKNOWN;
 }
 
 /**
- * Returns a pointer to the end of the first token within s. If
+ * @returns a pointer to the end of the first token within s. If
  * s only consists of a single token, it returns a pointer to the
  * terminating \0 in the string.
  */
@@ -140,7 +145,7 @@ gchar *shell_token_end(const gchar *s)
 		if (*cur == '\0')
 			return cur; /* ran into end of string */
 
-		if (escape || (*cur == '\\')) {
+		if (escape || *cur == '\\') {
 			escape = !escape;
 			cur ++;
 			continue;
@@ -159,7 +164,7 @@ gchar *shell_token_end(const gchar *s)
 				done = TRUE;
 			break;
 		}
-		
+
 		if (!done)
 			cur ++;
 	}
@@ -191,14 +196,13 @@ shell_unescape(gchar *s)
 }
 
 /**
- * Return the next token from s starting from position pos. Make sure
+ * @return the next token from s starting from position pos. Make sure
  * that pos is 0 or something sensible when calling this the first time!.
  * The returned string needs to be g_free-ed when no longer needed.
  */
 static gchar *
 shell_get_token(const gchar *s, gint *pos) {
-	const gchar *start;
-	const gchar *end;
+	const gchar *start, *end;
 	gchar *retval;
 
 	g_assert(pos);
@@ -209,26 +213,90 @@ shell_get_token(const gchar *s, gint *pos) {
 	if (*start == '\0')
 		*pos = -1;
 
-	if (*pos == -1) 
+	if (*pos == -1)
 		return NULL; /* nothing more to get */
-	
+
 	end = shell_token_end(start);
 
 	/* update position before removing quotes */
-	*pos = (*end == '\0') ? -1 : end-s+1;
+	*pos = *end == '\0' ? -1 : end - s + 1;
 
 	/* don't return enclosing quotes */
-	if ((*start == '"') && (*end == '"'))
+	if (*start == '"' && *end == '"')
 		start ++;
 
-	retval = g_strndup(start, end-start);
+	retval = g_strndup(start, end - start);
 	shell_unescape(retval);
-	
+
 	return retval;
 }
 
 static guint
-shell_exec_node(gnutella_shell_t *sh, const gchar *cmd) 
+shell_exec_node(gnutella_shell_t *sh, const gchar *cmd)
+{
+	gchar *tok;
+	gint pos = 0;
+
+	g_assert(sh);
+	g_assert(cmd);
+
+	tok = shell_get_token(cmd, &pos);
+	if (!tok)
+		goto error;
+
+	switch (get_command(tok)) {
+	case CMD_ADD: {
+		gchar *tok_buf;
+		host_addr_t addr;
+		guint32 port = GTA_PORT;
+
+		tok_buf = shell_get_token(cmd, &pos);
+
+		if (tok_buf) {
+			addr = name_to_host_addr(tok_buf);
+			G_FREE_NULL(tok_buf);
+		} else
+			goto error;
+
+		tok_buf = shell_get_token(cmd, &pos);
+
+		if (tok_buf) {
+			gint error;
+			guint32 u;
+			
+			u = parse_uint32(tok_buf, NULL, 10, &error);
+			port = error || u > 65535 ? 0 : u;
+			G_FREE_NULL(tok_buf);
+		}
+
+		if (is_host_addr(addr) && port) {
+			node_add(addr, port, CONNECT_F_FORCE);
+			sh->msg = _("Node added");
+		} else {
+			sh->msg = _("Invalid IP/Port");
+			goto error;
+		}
+		break;
+	}
+	default:
+		sh->msg = _("Unknown operation");
+		goto error;
+	}
+
+	G_FREE_NULL(tok);
+
+	return REPLY_READY;
+
+error:
+	G_FREE_NULL(tok);
+	if (sh->msg == NULL)
+		sh->msg = _("Malformed command");
+
+	return REPLY_ERROR;
+}
+
+static guint
+shell_exec_search(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok;
 	gint pos = 0;
@@ -242,80 +310,18 @@ shell_exec_node(gnutella_shell_t *sh, const gchar *cmd)
 		goto error;
 
 	switch (get_command(tok)) {
-	case CMD_ADD: {
-		gchar *tok_buf;
-		guint32 ip = 0;
-		guint32 port = GTA_PORT;
-
-		tok_buf = shell_get_token(cmd, &pos);
-
-		if (tok_buf) {
-			ip = host_to_ip(tok_buf);
-			G_FREE_NULL(tok_buf);
-		} else
-			goto error;
-
-		tok_buf = shell_get_token(cmd, &pos);
-
-		if (tok_buf) {
-			port = atol(tok_buf);
-			G_FREE_NULL(tok_buf);
-		}
-
-		if (ip && port) {
-			node_add(ip, port);
-			sh->msg = _("Node added");
-			reply_code = REPLY_READY;
-		} else {
-			sh->msg = _("Invalid IP/Port");
-			goto error;
-		}
-		break;
-	}
-	default:
-		sh->msg = _("Unknown operation");
-		goto error;
-	}
-
-	G_FREE_NULL(tok);
-	
-	return REPLY_READY;
-
-error:
-	G_FREE_NULL(tok);
-	if (sh->msg == NULL)
-		sh->msg = _("Malformed command");
-	
-	return REPLY_ERROR;
-}
-
-static guint
-shell_exec_search(gnutella_shell_t *sh, const gchar *cmd) 
-{
-	gchar *tok;
-	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
-
-	g_assert(sh);
-	g_assert(cmd);
-
-	tok = shell_get_token(cmd, &pos);
-	if (!tok)
-		goto error;
-
-	switch(get_command(tok)) {
 	case CMD_ADD: {
 		gchar *tok_query;
-		
+
 		tok_query = shell_get_token(cmd, &pos);
 		if (!tok_query) {
 			sh->msg = _("Query string missing");
 			goto error;
 		}
-	
+
 		gcu_search_gui_new_search(tok_query, 0);
 		G_FREE_NULL(tok_query);
-	   
+
 		sh->msg = _("Search added");
 		reply_code = REPLY_READY;
 		break;
@@ -326,31 +332,56 @@ shell_exec_search(gnutella_shell_t *sh, const gchar *cmd)
 	}
 
 	G_FREE_NULL(tok);
-	
+
 	return reply_code;
 
 error:
 	G_FREE_NULL(tok);
 	if (sh->msg == NULL)
 		sh->msg = _("Malformed command");
-	
+
 	return REPLY_ERROR;
 }
 
+static property_t
+get_prop_stub_by_name(const gchar *tok_prop, prop_set_stub_t **stub_ptr)
+{
+	prop_set_get_stub_t stub_getter[] = {
+#if !defined(USE_TOPLESS)
+		gui_prop_get_stub,
+#endif
+		gnet_prop_get_stub,
+	};
+	guint i;
+
+	g_return_val_if_fail(NULL != tok_prop, NO_PROP);
+	g_return_val_if_fail(NULL != stub_ptr, NO_PROP);
+
+	for (i = 0; i < G_N_ELEMENTS(stub_getter); i++) {
+		property_t prop;
+		prop_set_stub_t *stub;
+
+		stub = (stub_getter[i])();
+		if (NO_PROP != (prop = stub->get_by_name(tok_prop))) {
+			*stub_ptr = stub;
+			return prop;
+		}
+		G_FREE_NULL(stub);
+	}
+
+	*stub_ptr = NULL;
+	return NO_PROP;
+}
+
+
 static guint
-shell_exec_print(gnutella_shell_t *sh, const gchar *cmd) 
+shell_exec_print(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok_prop;
 	gint pos = 0;
 	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
 	property_t prop;
-	prop_set_get_stub_t stub_getter[] = {
-		gui_prop_get_stub,
-		gnet_prop_get_stub,
-		NULL
-	};
-	guint n;
 
 	g_assert(sh);
 	g_assert(cmd);
@@ -361,14 +392,7 @@ shell_exec_print(gnutella_shell_t *sh, const gchar *cmd)
 		goto error;
 	}
 
-	n = 0; prop = NO_PROP;
-	while((stub_getter[n] != NULL) && (prop == NO_PROP)) {
-		G_FREE_NULL(stub);
-		stub = (stub_getter[n])();
-		prop = stub->get_by_name(tok_prop);
-		n ++;
-	}
-			
+	prop = get_prop_stub_by_name(tok_prop, &stub);
 	if (prop == NO_PROP) {
 		sh->msg = _("Unknown property");
 		goto error;
@@ -383,7 +407,7 @@ shell_exec_print(gnutella_shell_t *sh, const gchar *cmd)
 
 	G_FREE_NULL(stub);
 	G_FREE_NULL(tok_prop);
-	
+
 	return reply_code;
 
 error:
@@ -391,12 +415,12 @@ error:
 	G_FREE_NULL(tok_prop);
 	if (sh->msg == NULL)
 		sh->msg = _("Malformed command");
-	
+
 	return REPLY_ERROR;
 }
 
 static guint
-shell_exec_set(gnutella_shell_t *sh, const gchar *cmd) 
+shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok_prop;
 	gchar *tok_value;
@@ -404,13 +428,7 @@ shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
 	property_t prop;
-	prop_set_get_stub_t stub_getter[] = {
-		gui_prop_get_stub,
-		gnet_prop_get_stub,
-		NULL
-	};
 	prop_def_t *prop_buf = NULL;
-	guint n;
 
 	g_assert(sh);
 	g_assert(cmd);
@@ -418,20 +436,13 @@ shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 	tok_prop = shell_get_token(cmd, &pos);
 	if (!tok_prop) {
 		sh->msg = _("Property missing");
-		goto error;
+		goto failure;
 	}
 
-	n = 0; prop = NO_PROP;
-	while ((stub_getter[n] != NULL) && (prop == NO_PROP)) {
-		G_FREE_NULL(stub);
-		stub = (stub_getter[n])();
-		prop = stub->get_by_name(tok_prop);
-		n ++;
-	}
-			
+	prop = get_prop_stub_by_name(tok_prop, &stub);
 	if (prop == NO_PROP) {
 		sh->msg = _("Unknown property");
-		goto error;
+		goto failure;
 	}
 
 	prop_buf = stub->get_def (prop);
@@ -442,51 +453,70 @@ shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 	if (!tok_value) {
 		prop_free_def (prop_buf);
 		sh->msg = _("Value missing");
-		goto error;
+		goto failure;
 	}
 
 	switch (prop_buf->type) {
-	case PROP_TYPE_BOOLEAN: {
-		gboolean val;
-		if (g_ascii_strcasecmp(tok_value, "true") == 0) {
-			val = TRUE;
+	case PROP_TYPE_BOOLEAN:
+		{
+			gboolean val;
+			
+			if (0 == ascii_strcasecmp(tok_value, "true")) {
+				val = TRUE;
+			} else if (0 == ascii_strcasecmp(tok_value, "false")) {
+				val = FALSE;
+			} else {
+				guint64 u;
+				gint error;
+			
+				u = parse_uint64(tok_value, NULL, 10, &error);
+				if (error)
+					goto failure;
+
+				val = u ? TRUE : FALSE;
+			}
+			stub->boolean.set(prop, &val, 0, 1);
 		}
-		else if (g_ascii_strcasecmp(tok_value, "false") == 0) {
-			val = FALSE;
-		}
-		else {
-			val = (atol(tok_value) != 0) ? TRUE : FALSE;
-		}
-		stub->boolean.set (prop, &val, 0, 1);
 		break;
-	}
 	case PROP_TYPE_MULTICHOICE:
-	case PROP_TYPE_GUINT32: {
-		guint32 val;
-		val = atol(tok_value);
-		stub->guint32.set (prop, &val, 0, 1);
+	case PROP_TYPE_GUINT32:
+		{
+			guint32 val;
+			gint error;
+
+			val = parse_uint32(tok_value, NULL, 10, &error);
+			if (error)
+				goto failure;
+		
+			stub->guint32.set(prop, &val, 0, 1);
+		}
 		break;
-	}
-	case PROP_TYPE_GUINT64: {
-		guint32 val;
-		val = atol(tok_value);
-		stub->guint32.set (prop, &val, 0, 1);
+	case PROP_TYPE_GUINT64:
+		{
+			guint64 val;
+			gint error;
+
+			val = parse_uint64(tok_value, NULL, 10, &error);
+			if (error)
+				goto failure;
+
+			stub->guint64.set(prop, &val, 0, 1);
+		}
 		break;
-	}
-	case PROP_TYPE_STRING: {
-		stub->string.set (prop, tok_value);
+	case PROP_TYPE_STRING:
+		stub->string.set(prop, tok_value);
 		break;
-	}
-	case PROP_TYPE_STORAGE: {
-		gchar guid[16];
-		hex_to_guid(tok_value, guid);
-		stub->storage.set (prop, guid, prop_buf->vector_size);
+	case PROP_TYPE_STORAGE:
+		{
+			gchar guid[GUID_RAW_SIZE];
+			hex_to_guid(tok_value, guid);
+			stub->storage.set (prop, guid, prop_buf->vector_size);
+		}
 		break;
-	}
 	default:
 		prop_free_def (prop_buf);
 		sh->msg = _("Type not supported");
-		goto error;
+		goto failure;
 	}
 
 	sh->msg = _("Value found and set");
@@ -496,15 +526,15 @@ shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 	G_FREE_NULL(tok_prop);
 	G_FREE_NULL(tok_value);
 	prop_free_def (prop_buf);
-	
+
 	return reply_code;
 
-error:
+failure:
 	G_FREE_NULL(stub);
 	G_FREE_NULL(tok_prop);
-	if (sh->msg == NULL)
+	if (!sh->msg)
 		sh->msg = _("Malformed command");
-	
+
 	return REPLY_ERROR;
 }
 
@@ -512,20 +542,14 @@ error:
  * Takes a whatis command and tries to execute it.
  */
 static guint
-shell_exec_whatis(gnutella_shell_t *sh, const gchar *cmd) 
+shell_exec_whatis(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok_prop;
 	gint pos = 0;
 	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
-	property_t prop;
-	prop_set_get_stub_t stub_getter[] = {
-		gui_prop_get_stub,
-		gnet_prop_get_stub,
-		NULL
-	};
 	prop_def_t *prop_buf = NULL;
-	guint n;
+	property_t prop;
 
 	g_assert(sh);
 	g_assert(cmd);
@@ -536,14 +560,7 @@ shell_exec_whatis(gnutella_shell_t *sh, const gchar *cmd)
 		goto error;
 	}
 
-	n = 0; prop = NO_PROP;
-	while ((stub_getter[n] != NULL) && (prop == NO_PROP)) {
-		G_FREE_NULL(stub);
-		stub = (stub_getter[n])();
-		prop = stub->get_by_name(tok_prop);
-		n ++;
-	}
-			
+	prop = get_prop_stub_by_name(tok_prop, &stub);
 	if (prop == NO_PROP) {
 		sh->msg = _("Unknown property");
 		goto error;
@@ -563,7 +580,7 @@ shell_exec_whatis(gnutella_shell_t *sh, const gchar *cmd)
 	G_FREE_NULL(stub);
 	G_FREE_NULL(tok_prop);
 	prop_free_def (prop_buf);
-	
+
 	return reply_code;
 
 error:
@@ -571,22 +588,39 @@ error:
 	G_FREE_NULL(tok_prop);
 	if (sh->msg == NULL)
 		sh->msg = _("Malformed command");
-	
+
 	return REPLY_ERROR;
 }
 
+/**
+ * Rescan the shared directories for added/removed files.
+ */
+static guint
+shell_exec_rescan(gnutella_shell_t *sh, const gchar *cmd)
+{
+	g_assert(sh);
+	g_assert(cmd);
+
+	shell_write(sh, "100-Scanning shared directories...\n");
+	guc_share_scan();
+	shell_write(sh, "100-Finished\n");
+	sh->msg = "";
+
+	return REPLY_READY;
+}
+
+
 /**
  * Displays horizon size information.
  */
 static guint
-shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd) 
+shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar buf[200];
 	gchar *tok;
 	gint pos = 0;
 	hsep_triple globaltable[HSEP_N_MAX + 1];
-	hsep_triple nonhsep;
-	guint64 *globalt;
+	hsep_triple non_hsep[1];
 	gboolean all;
 
 	g_assert(sh);
@@ -597,49 +631,46 @@ shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd)
     if (tok != NULL) {
 		shell_write(sh, tok);
 		shell_write(sh, "\n");
-		if (0 == strcasecmp(tok, "ALL"))
-			all = 1;
-		else {
+		if (0 == ascii_strcasecmp(tok, "ALL")) {
+			all = TRUE;
+		} else {
         	sh->msg = _("Unknown parameter");
 	        goto error;
 		}
+	} else {
+		all = FALSE;
 	}
-	else
-		all = 0;
 
 	sh->msg = "";
 
-	hsep_get_global_table(globaltable, G_N_ELEMENTS(globaltable));	
-	hsep_get_non_hsep_triple(&nonhsep);
+	hsep_get_global_table(globaltable, G_N_ELEMENTS(globaltable));
+	hsep_get_non_hsep_triple(non_hsep);
 
-	gm_snprintf(buf, sizeof(buf),
+	gm_snprintf(buf, sizeof buf,
 		_("Total horizon size (%u/%u nodes support HSEP):"),
-		(unsigned int)globaltable[1][HSEP_IDX_NODES],
-		(unsigned int)(globaltable[1][HSEP_IDX_NODES] +
-		nonhsep[HSEP_IDX_NODES]));
+		(guint)globaltable[1][HSEP_IDX_NODES],
+		(guint)(globaltable[1][HSEP_IDX_NODES] + non_hsep[0][HSEP_IDX_NODES]));
 
 	shell_write(sh, buf);
 	shell_write(sh, "\n\n");
 
-	globalt = (gint64 *) &globaltable[1];
+	print_hsep_table(sh, globaltable, HSEP_N_MAX, non_hsep);
 
-	print_hsep_table(sh, globaltable, HSEP_N_MAX, &nonhsep);
-
-	if(all) {
-		GSList *sl;
+	if (all) {
+		const GSList *sl;
 		hsep_triple table[HSEP_N_MAX + 1];
 
-		for (sl = (GSList *) node_all_nodes(); sl; sl = g_slist_next(sl)) {
-			struct gnutella_node *n = (struct gnutella_node *) sl->data;
+		for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
+			const struct gnutella_node *n = sl->data;
 
 			if ((!NODE_IS_ESTABLISHED(n)) || !(n->attrs & NODE_A_CAN_HSEP))
 				continue;
 
 			shell_write(sh, "\n");
 
-			gm_snprintf(buf, sizeof(buf),
+			gm_snprintf(buf, sizeof buf,
 				_("Horizon size via HSEP node %s (%s):"),
-				node_ip(n),
+				node_addr(n),
 				NODE_IS_LEAF(n) ? _("leaf") :
 					(NODE_IS_ULTRA(n) ? _("ultrapeer") : _("normal node")));
 
@@ -662,59 +693,60 @@ error:
 	return REPLY_ERROR;
 }
 
-void
+static void
 print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
-	int triples, hsep_triple *nonhsepptr)
+	int triples, hsep_triple *non_hsep_ptr)
 {
-	gint i;
-	char *hopsstr = _("Hops");
-	char *nodesstr = _("Nodes");
-	char *filesstr = _("Files");
-	char *sizestr = _("Size");
-	guint64 *t;
-	hsep_triple nonhsep;
-	static hsep_triple emptynonhsep = {0, 0, 0};
+	static const hsep_triple empty_non_hsep = {0, 0, 0};
+	const gchar *hops_str = _("Hops");
+	const gchar *nodes_str = _("Nodes");
+	const gchar *files_str = _("Files");
+	const gchar *size_str = _("Size");
+	hsep_triple non_hsep, *t;
 	gchar buf[200];
 	guint maxlen[4];
+	gint i;
 
-	if (nonhsepptr != NULL)
-		memcpy(nonhsep, *nonhsepptr, sizeof(nonhsep));
+	if (NULL != non_hsep_ptr)
+		memcpy(non_hsep, non_hsep_ptr, sizeof non_hsep);
 	else
-		memcpy(nonhsep, emptynonhsep, sizeof(nonhsep));
-
-	t = (guint64 *) &table[1];
+		memcpy(non_hsep, empty_non_hsep, sizeof non_hsep);
+	t = &table[1];
 
 	/*
 	 * Determine maximum width of each column.
 	 */
-	
-	maxlen[0] = strlen(hopsstr);   /* length of Hops */
-	maxlen[1] = strlen(nodesstr);  /* length of Nodes */
-	maxlen[2] = strlen(filesstr);  /* length of Files */
-	maxlen[3] = strlen(sizestr);   /* length of Size */
+
+	maxlen[0] = strlen(hops_str);   /* length of Hops */
+	maxlen[1] = strlen(nodes_str);  /* length of Nodes */
+	maxlen[2] = strlen(files_str);  /* length of Files */
+	maxlen[3] = strlen(size_str);   /* length of Size */
 
 	for (i = 0; i < triples * 4; i++) {
 		size_t n;
 		guint m = i % 4;
-	       
+
 		switch (m) {
 		case 0:
-			n = gm_snprintf(buf, sizeof(buf), "%d", i / 4 + 1);
+			n = strlen(uint64_to_string(i / 4 + 1));
 			break;
+
 		case 1:
-			n = gm_snprintf(buf, sizeof(buf), "%" PRIu64,
-					*t + nonhsep[HSEP_IDX_NODES]);
-			t++;
-			break;
 		case 2:
-			n = gm_snprintf(buf, sizeof(buf), "%" PRIu64,
-					*t + nonhsep[HSEP_IDX_FILES]);
-			t++;
-			break;
 		case 3:
-			n = strlen(short_kb_size(*t + nonhsep[HSEP_IDX_KIB]));
-			t++;
+			{
+				guint j = 0;
+
+				switch (m) {
+				case 1: j = HSEP_IDX_NODES; break;
+				case 2: j = HSEP_IDX_FILES; break;
+				case 3: j = HSEP_IDX_KIB; break;
+				}
+
+				n = strlen(uint64_to_string(t[i][j] + non_hsep[j]));
+			}
 			break;
+
 		default:
 			n = 0;
 			g_assert_not_reached();
@@ -724,8 +756,11 @@ print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
 			maxlen[m] = n;
 	}
 
-	gm_snprintf(buf, sizeof(buf), "%*s  %*s  %*s  %*s\n", maxlen[0], hopsstr,
-		maxlen[1], nodesstr, maxlen[2], filesstr, maxlen[3], sizestr);
+	gm_snprintf(buf, sizeof buf, "%*s  %*s  %*s  %*s\n",
+		maxlen[0], hops_str,
+		maxlen[1], nodes_str,
+		maxlen[2], files_str,
+		maxlen[3], size_str);
 
 	shell_write(sh, buf);
 
@@ -734,30 +769,33 @@ print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
 
 	shell_write(sh, "\n");
 
-	t = (gint64 *) &table[1];
-	
+	t = &table[1];
+
 	for (i = 0; i < triples; i++) {
-		gm_snprintf(buf, sizeof(buf), "%*d  %*" PRIu64 "  %*" PRIu64 "  %*s\n",
+		const gchar *s1, *s2, *s3;
+
+		s1 = uint64_to_string(t[i][HSEP_IDX_NODES] + non_hsep[HSEP_IDX_NODES]);
+		s2 = uint64_to_string2(t[i][HSEP_IDX_FILES] + non_hsep[HSEP_IDX_FILES]);
+		s3 = short_kb_size(t[i][HSEP_IDX_KIB] + non_hsep[HSEP_IDX_KIB]);
+
+		gm_snprintf(buf, sizeof buf, "%*d  %*s  %*s  %*s\n",
 			maxlen[0], i + 1,
-			maxlen[1], t[HSEP_IDX_NODES] + nonhsep[HSEP_IDX_NODES],
-			maxlen[2], t[HSEP_IDX_FILES] + nonhsep[HSEP_IDX_FILES],
-			maxlen[3], short_kb_size(t[HSEP_IDX_KIB] +
-			    nonhsep[HSEP_IDX_KIB]));
+			maxlen[1], s1,
+			maxlen[2], s2,
+			maxlen[3], s3);
 
 		shell_write(sh, buf);
-		t += 3;
-	}		
+	}
 
 }
 
 
 
-/*
- * shell_exec:
- *
+/**
  * Takes a command string and tries to parse and execute it.
  */
-static guint shell_exec(gnutella_shell_t *sh, const gchar *cmd) 
+static guint
+shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok;
 	gint pos = 0;
@@ -768,11 +806,11 @@ static guint shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 
 	tok = shell_get_token(cmd, &pos);
 	if (!tok)
-		goto error;
+		return CMD_NOOP;
 
 	switch (get_command(tok)) {
 	case CMD_HELP:
-		shell_write(sh, 
+		shell_write(sh,
 			"100-Help:\n"
 			"100-SEARCH ADD <query>\n"
 			"100-NODE ADD <ip> [port]\n"
@@ -780,6 +818,7 @@ static guint shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 			"100-SET <property> <value>\n"
 			"100-WHATIS <property>\n"
 			"100-HORIZON [ALL]\n"
+			"100-RESCAN\n"
 			"100-QUIT\n"
 			"100-HELP\n");
 		reply_code = REPLY_READY;
@@ -790,91 +829,96 @@ static guint shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 		shell_shutdown(sh);
 		break;
 	case CMD_SEARCH:
-		reply_code = shell_exec_search(sh, cmd+pos);
+		reply_code = shell_exec_search(sh, &cmd[pos]);
 		break;
 	case CMD_NODE:
-		reply_code = shell_exec_node(sh, cmd+pos);
+		reply_code = shell_exec_node(sh, &cmd[pos]);
 		break;
 	case CMD_PRINT:
-		reply_code = shell_exec_print(sh, cmd+pos);
+		reply_code = shell_exec_print(sh, &cmd[pos]);
 		break;
 	case CMD_SET:
-		reply_code = shell_exec_set(sh, cmd+pos);
+		reply_code = shell_exec_set(sh, &cmd[pos]);
 		break;
 	case CMD_WHATIS:
-		reply_code = shell_exec_whatis(sh, cmd+pos);
+		reply_code = shell_exec_whatis(sh, &cmd[pos]);
 		break;
 	case CMD_HORIZON:
-		reply_code = shell_exec_horizon(sh, cmd+pos);
+		reply_code = shell_exec_horizon(sh, &cmd[pos]);
+		break;
+	case CMD_RESCAN:
+		reply_code = shell_exec_rescan(sh, &cmd[pos]);
 		break;
 	default:
 		goto error;
 	}
 
 	G_FREE_NULL(tok);
-	
+
 	return reply_code;
 
 error:
 	G_FREE_NULL(tok);
 	if (sh->msg == NULL)
 		sh->msg = _("Unknown command");
-	
+
 	return REPLY_ERROR;
 }
 
-/*
- * shell_write_data:
- *
+/**
  * Called when data can be written to the shell connection. If the
  * connection is in shutdown mode, it is destroyed when the output
  * buffer is empty.
  */
-static void shell_write_data(gnutella_shell_t *sh)
+static void
+shell_write_data(gnutella_shell_t *sh)
 {
 	struct gnutella_socket *s;
-
 	ssize_t written;
 
 	g_assert(sh);
 	g_assert(sh->socket);
+	g_assert(sh->outbuf);
 	g_assert(IS_PROCESSING(sh));
 
-	sh->last_update = time((time_t *) NULL);
+	sh->last_update = tm_time();
 
 	s = sh->socket;
 	written = s->wio.write(&s->wio, sh->outbuf, sh->outpos);
-	if (written < 0) {
-		if (errno == EAGAIN)
+	switch (written) {
+	case (ssize_t) -1:
+		if (errno == VAL_EAGAIN || errno == EINTR)
 			return;
 
-		shell_shutdown (sh);
+		/* FALL THRU */
+	case 0:
+		if (!sh->shutdown)
+			shell_shutdown(sh);
 		sh->outpos = 0;
-	}
+		break;
 
-	memmove(sh->outbuf, sh->outbuf + written, sh->outpos-written);
-	sh->outpos -= written;
+	default:
+		memmove(sh->outbuf, &sh->outbuf[written], sh->outpos - written);
+		sh->outpos -= written;
+	}
 
 	g_assert(sh->outpos >= 0);
 
-	if (sh->outpos == 0) {
-		if (sh->write_tag) {
-			g_assert(inputevt_remove(sh->write_tag));
-			sh->write_tag = 0;
-		}
+	if (0 == sh->outpos) {
+		socket_evt_clear(sh->socket);
+		socket_evt_set(sh->socket, INPUT_EVENT_RX, shell_handle_data, sh);
 	}
 }
 
-/*
- * shell_read_data:
- *
+/**
  * Called when data is available on a shell connection. Uses getline to
  * read the available data line by line.
  */
-static void shell_read_data(gnutella_shell_t *sh)
+static void
+shell_read_data(gnutella_shell_t *sh)
 {
 	struct gnutella_socket *s;
-	guint parsed;
+	gint parsed;
 	ssize_t rc = -1;
 
 	g_assert(sh);
@@ -882,7 +926,7 @@ static void shell_read_data(gnutella_shell_t *sh)
 	g_assert(sh->socket);
 	g_assert(sh->socket->getline);
 
-	sh->last_update = time((time_t *) NULL);
+	sh->last_update = tm_time();
 	s = sh->socket;
 
 	if (s->pos >= sizeof(s->buffer))
@@ -890,7 +934,7 @@ static void shell_read_data(gnutella_shell_t *sh)
 	else {
 		gchar *p = s->buffer + s->pos;
 		size_t size = sizeof(s->buffer) - s->pos - 1;
-		
+
 		rc = s->wio.read(&s->wio, p, size);
 		if (rc <= 0) {
 			if (rc == 0) {
@@ -911,25 +955,24 @@ static void shell_read_data(gnutella_shell_t *sh)
 
 	while (s->pos) {
 		guint reply_code;
-		GString *buf; 
 
 		g_assert (s->pos > 0);
 
 		switch (getline_read(s->getline, s->buffer, s->pos, &parsed)) {
 		case READ_OVERFLOW:
 			g_warning("Line is too long (from shell at %s)\n",
-				ip_port_to_gchar(s->ip, s->port));
+				host_addr_port_to_string(s->addr, s->port));
 			shell_destroy(sh);
 			return;
 		case READ_DONE:
-			if (s->pos != parsed)
-				memmove(s->buffer, s->buffer+parsed, s->pos-parsed);
+			if (s->pos != (size_t) parsed)
+				memmove(s->buffer, &s->buffer[parsed], s->pos - parsed);
 			s->pos -= parsed;
 			break;
 		case READ_MORE:
 		default:
-			g_assert(parsed == s->pos);
-		
+			g_assert((size_t) parsed == s->pos);
+
 			return;
 		}
 
@@ -937,14 +980,20 @@ static void shell_read_data(gnutella_shell_t *sh)
 		 * We come here everytime we get a full line.
 		 */
 
-		buf = g_string_sized_new(100);
 		reply_code = shell_exec(sh, getline_str(s->getline));
-		g_string_printf(buf, "%u %s\n", reply_code, sh->msg ? sh->msg : "");
-		shell_write(sh, buf->str);
-		g_string_free(buf, TRUE);
-		sh->msg = NULL;
-		buf = NULL;
+		if (CMD_NOOP != reply_code) {
+			gchar *buf = NULL;
+			size_t size;
 
+			size = w_concat_strings(&buf, uint32_to_string(reply_code),
+					" ", sh->msg ? sh->msg : "", "\n", (void *) 0);
+
+			shell_write(sh, buf); /* XXX: Let shell_write() own ``buf'' */
+			wfree(buf, size);
+			buf = NULL;
+		}
+
+		sh->msg = NULL;
 		getline_reset(s->getline);
 	}
 
@@ -956,7 +1005,7 @@ static void shell_read_data(gnutella_shell_t *sh)
 static void
 shell_handle_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 {
-	gnutella_shell_t *sh = (gnutella_shell_t *) data;
+	gnutella_shell_t *sh = data;
 
 	(void) unused_source;
 	g_assert(sh);
@@ -967,7 +1016,7 @@ shell_handle_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	if ((cond & INPUT_EVENT_WRITE) && IS_PROCESSING(sh))
+	if ((cond & INPUT_EVENT_W) && IS_PROCESSING(sh))
 		shell_write_data(sh);
 
 	if (sh->shutdown) {
@@ -976,7 +1025,7 @@ shell_handle_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	if (cond & INPUT_EVENT_READ)
+	if (cond & INPUT_EVENT_R)
 		shell_read_data(sh);
 
 }
@@ -985,29 +1034,31 @@ static gboolean
 shell_write(gnutella_shell_t *sh, const gchar *s)
 {
 	size_t len;
+	gboolean writing;
 
 	g_assert(sh);
 	g_assert(s);
 
 	g_return_val_if_fail(sh->outpos >= 0, FALSE);
-	g_return_val_if_fail((size_t) sh->outpos < sizeof sh->outbuf, FALSE);
+	g_return_val_if_fail((size_t) sh->outpos < SHELL_BUFFER_SIZE, FALSE);
 	len = strlen(s);
 	g_return_val_if_fail((ssize_t) len >= 0, FALSE);
-	g_return_val_if_fail(len <= sizeof sh->outbuf, FALSE);
+	g_return_val_if_fail(len <= SHELL_BUFFER_SIZE, FALSE);
 
-	if (len + sh->outpos >= sizeof(sh->outbuf)) {
+	if (len + sh->outpos >= SHELL_BUFFER_SIZE) {
+		/* XXX: This is ridiculous */
 		g_warning("Line is too long (for shell at %s)",
-			ip_port_to_gchar(sh->socket->ip, sh->socket->port));
+			host_addr_port_to_string(sh->socket->addr, sh->socket->port));
 		return FALSE;
 	}
 
-	memcpy(sh->outbuf + sh->outpos, s, len);
+	writing = 0 != sh->outpos;
+	memcpy(&sh->outbuf[sh->outpos], s, len);
 	sh->outpos += len;
 
-	if (sh->write_tag == 0) {
-		sh->write_tag = inputevt_add(sh->socket->wio.fd(&sh->socket->wio), 
-			INPUT_EVENT_EXCEPTION | INPUT_EVENT_WRITE,
-			shell_handle_data, (gpointer) sh);
+	if (!writing) {
+		socket_evt_clear(sh->socket);
+		socket_evt_set(sh->socket, INPUT_EVENT_WX, shell_handle_data, sh);
 	}
 
 	return TRUE;
@@ -1015,8 +1066,9 @@ shell_write(gnutella_shell_t *sh, const gchar *s)
 
 /**
  * Takes a HELO command string and checks whether the connection
- * is allowed using the specified credentials. Returns true if
- * the connection is allowed.
+ * is allowed using the specified credentials.
+ *
+ * @return TRUE if the connection is allowed.
  */
 static gboolean
 shell_auth(const gchar *str)
@@ -1032,9 +1084,8 @@ shell_auth(const gchar *str)
 	g_warning("auth: [%s] [<cookie not displayed>]", tok_helo);
 
 	if (tok_helo && tok_cookie) {
-		ok = 
-			(strcmp("HELO", tok_helo) == 0) &&
-			(strcmp(sha1_base32(auth_cookie), tok_cookie) == 0);
+		ok = strcmp("HELO", tok_helo) == 0 &&
+			strcmp(sha1_base32(auth_cookie), tok_cookie) == 0;
 	} else {
 		ok = FALSE;
 	}
@@ -1055,10 +1106,10 @@ shell_new(struct gnutella_socket *s)
 
 	g_assert(s);
 
-	sh = g_new0(gnutella_shell_t, 1);
+	sh = walloc0(sizeof *sh);
 	sh->socket = s;
+	sh->outbuf = walloc(SHELL_BUFFER_SIZE);
 	sh->outpos = 0;
-	sh->write_tag = 0;
 	sh->msg = NULL;
 	sh->shutdown = FALSE;
 
@@ -1071,9 +1122,10 @@ shell_new(struct gnutella_socket *s)
 static void
 shell_free(gnutella_shell_t *sh)
 {
-	g_assert(sh->socket == NULL); /* must have called shell_destroy before */
+	g_assert(NULL == sh->socket); /* must have called shell_destroy before */
+	g_assert(NULL == sh->outbuf); /* must have called shell_destroy before */
 
-	G_FREE_NULL(sh);
+	wfree(sh, sizeof *sh);
 }
 
 /**
@@ -1081,25 +1133,29 @@ shell_free(gnutella_shell_t *sh)
  * removed from sl_shells, so don't call this while iterating over sl_shells.
  */
 static void
-shell_destroy(gnutella_shell_t *s)
+shell_destroy(gnutella_shell_t *sh)
 {
-	g_assert(s);
-	g_assert(s->socket);
+	g_assert(sh);
+	g_assert(sh->socket);
 
 	if (dbg > 0)
 		g_warning("shell_destroy");
 
-	sl_shells = g_slist_remove(sl_shells, s);
+	sl_shells = g_slist_remove(sl_shells, sh);
 
-	if (s->write_tag)
-		g_assert(inputevt_remove(s->write_tag));
+	socket_evt_clear(sh->socket);
 
-	socket_free(s->socket);
-	s->socket = NULL;
-	shell_free(s);
+	if (sh->outbuf) {
+		wfree(sh->outbuf, SHELL_BUFFER_SIZE);
+		sh->outbuf = NULL;
+	}
+
+	socket_free(sh->socket);
+	sh->socket = NULL;
+	shell_free(sh);
 }
 
-void
+static void
 shell_shutdown(gnutella_shell_t *sh)
 {
 	g_assert(sh);
@@ -1117,21 +1173,19 @@ shell_add(struct gnutella_socket *s)
 	gnutella_shell_t *sh;
 
 	g_assert(s);
-	g_assert(s->gdk_tag == 0);
+	g_assert(0 == s->gdk_tag);
 	g_assert(s->getline);
 
-	g_warning("Incoming shell connection from %s\n",
-		ip_port_to_gchar(s->ip, s->port));
+	g_message("Incoming shell connection from %s",
+		host_addr_port_to_string(s->addr, s->port));
 
 	s->type = SOCK_TYPE_SHELL;
-	socket_tos_default(s);			/* Set proper Type of Service */
+	socket_tos_lowdelay(s);			/* Set proper Type of Service */
 
 	sh = shell_new(s);
-	
-	g_assert(s->gdk_tag == 0);
-	s->gdk_tag = inputevt_add(s->wio.fd(&s->wio),
-		INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
-		shell_handle_data, (gpointer) sh);
+
+	socket_evt_clear(s);
+	socket_evt_set(s, INPUT_EVENT_RX, shell_handle_data, sh);
 
 	sl_shells = g_slist_prepend(sl_shells, sh);
 
@@ -1180,18 +1234,19 @@ shell_dump_cookie(void)
 void
 shell_timer(time_t now)
 {
-	GSList *to_remove = NULL;
-	GSList *sl;
+	GSList *sl, *to_remove = NULL;
 
 	for (sl = sl_shells; sl != NULL; sl = g_slist_next(sl)) {
-		gnutella_shell_t *sh = (gnutella_shell_t *) sl->data;
-		
-		if (now - sh->last_update > SHELL_TIMEOUT)
-			g_slist_prepend(to_remove, sh);
+		gnutella_shell_t *sh = sl->data;
+
+		if (delta_time(now, sh->last_update) > SHELL_TIMEOUT)
+			to_remove = g_slist_prepend(to_remove, sh);
 	}
 
-	for (sl = to_remove; sl != NULL; sl = g_slist_next(sl)) 
-		shell_destroy((gnutella_shell_t *) sl->data);
+	for (sl = to_remove; sl != NULL; sl = g_slist_next(sl)) {
+		gnutella_shell_t *sh = sl->data;
+		shell_destroy(sh);
+	}
 
 	g_slist_free(to_remove);
 }
@@ -1202,7 +1257,7 @@ shell_init(void)
 	gint n;
 
 	for (n = 0; n < SHA1_RAW_SIZE; n ++) {
-		guint32 v = random_value(~0);
+		guint32 v = random_value(~0U);
 
 		v ^= (v >> 24) ^ (v >> 16) ^ (v >> 8);
 		auth_cookie[n] = v & 0xff;
@@ -1214,12 +1269,13 @@ shell_init(void)
 void
 shell_close(void)
 {
-	GSList *sl;
-	GSList *to_remove;
+	GSList *sl, *to_remove;
 
 	to_remove = g_slist_copy(sl_shells);
-	for (sl = to_remove; sl; sl = g_slist_next(sl))
-		shell_destroy((gnutella_shell_t *) sl->data);
+	for (sl = to_remove; sl; sl = g_slist_next(sl)) {
+		gnutella_shell_t *sh = sl->data;
+		shell_destroy(sh);
+	}
 
 	g_slist_free(to_remove);
 	g_assert(NULL == sl_shells);
