@@ -1,10 +1,8 @@
 /*
- * $Id: udp.h,v 1.4 2004/11/14 21:35:27 rmanfredi Exp $
+ * $Id: udp.h,v 1.8 2005/09/14 17:57:42 rmanfredi Exp $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
- * Handling of UDP datagrams.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,10 +23,21 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Handling of UDP datagrams.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
+ */
+
 #ifndef _core_udp_h_
 #define _core_udp_h_
 
 #include <glib.h>
+#include "lib/host_addr.h"
 
 /*
  * Public interface.
@@ -37,10 +46,10 @@
 struct gnutella_socket;
 struct gnutella_node;
 
-void udp_received(struct gnutella_socket *s);
-void udp_connect_back(guint32 ip, guint16 port, const gchar *muid);
+void udp_received(struct gnutella_socket *s, gboolean truncated);
+void udp_connect_back(const host_addr_t addr, guint16 port, const gchar *muid);
 void udp_send_msg(struct gnutella_node *n, gpointer buf, gint len);
-void udp_send_ping(guint32 ip, guint16 port);
+void udp_send_ping(const host_addr_t addr, guint16 port);
 
 #endif /* _core_udp_h_ */
 
