@@ -1,5 +1,5 @@
 /*
- * $Id: version.c,v 1.4 2004/11/22 14:28:12 rmanfredi Exp $
+ * $Id: version.c,v 1.19 2005/12/21 13:43:04 rmanfredi Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -24,16 +24,22 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Version management.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: version.c,v 1.4 2004/11/22 14:28:12 rmanfredi Exp $");
+RCSID("$Id: version.c,v 1.19 2005/12/21 13:43:04 rmanfredi Exp $");
 
+#ifdef I_SYS_UTSNAME
 #include <sys/utsname.h>		/* For uname() */
+#endif
 
 #include "version.h"
 #include "token.h"
@@ -46,8 +52,12 @@ RCSID("$Id: version.c,v 1.4 2004/11/22 14:28:12 rmanfredi Exp $");
 #include "lib/getdate.h"
 #include "lib/misc.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
+#include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
+#define SECS_PER_DAY	86400
+
 gchar *version_string = NULL;
 gchar *version_short_string = NULL;
 
@@ -87,7 +97,7 @@ version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
 }
 
 /**
- * Return a user-friendly description of the version.
+ * @return a user-friendly description of the version.
  * NB: returns pointer to static data.
  */
 const gchar *
@@ -156,7 +166,7 @@ version_stamp(const gchar *str, version_t *ver)
 			 */
 
 			g_strlcpy(stamp, p, MIN(size, sizeof(stamp)));
-			ver->timestamp = date2time(stamp, time(NULL));
+			ver->timestamp = date2time(stamp, tm_time());
 
 			if (ver->timestamp == -1) {
 				ver->timestamp = 0;
@@ -172,7 +182,7 @@ version_stamp(const gchar *str, version_t *ver)
  * Parse gtk-gnutella's version number in User-Agent/Server string `str'
  * and extract relevant information into `ver'.
  *
- * Returns TRUE if we parsed a gtk-gnutella version correctly, FALSE if we
+ * @returns TRUE if we parsed a gtk-gnutella version correctly, FALSE if we
  * were not facing a gtk-gnutella version, or if we did not recognize it.
  */
 static gboolean
@@ -199,11 +209,9 @@ version_parse(const gchar *str, version_t *ver)
 	 * where the third number is the "patchlevel".
 	 */
 
-	if (0 != strncmp(str, "gtk-gnutella/", 13))
+	if (NULL == (v = is_strprefix(str, "gtk-gnutella/")))
 		return FALSE;
 
-	v = str + 13;
-
 	if (
 		5 == sscanf(v, "%u.%u.%u%c%u",
 			&ver->major, &ver->minor, &ver->patchlevel,
@@ -262,7 +270,7 @@ ok:
 
 /**
  * Compare two tag chars, assuming version numbers are equal.
- * Returns -1, 0 or +1 depending on the sign of "a - b".
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 static gint
 version_tagcmp(guchar a, guchar b)
@@ -287,7 +295,7 @@ version_tagcmp(guchar a, guchar b)
 
 /**
  * Compare two gtk-gnutella versions, timestamp not withstanding.
- * Returns -1, 0 or +1 depending on the sign of "a - b".
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 gint
 version_cmp(const version_t *a, const version_t *b)
@@ -311,7 +319,8 @@ version_cmp(const version_t *a, const version_t *b)
 
 /**
  * Parse vendor string and fill supplied version structure `vs'.
- * Returns OK if we were able to parse correctly.
+ *
+ * @returns OK if we were able to parse correctly.
  */
 gboolean
 version_fill(const gchar *version, version_t *vs)
@@ -332,53 +341,41 @@ version_new_found(const gchar *text, gboolean stable)
 {
     static gchar last_stable[256] = "";
     static gchar last_dev[256] = "";
-    static gchar s[1024];
+	gchar s[1024];
 
     if (stable)
-        g_strlcpy(last_stable, text, sizeof(last_stable));
+        utf8_strlcpy(last_stable, text, sizeof last_stable);
     else
-        g_strlcpy(last_dev, text, sizeof(last_dev));
+        utf8_strlcpy(last_dev, text, sizeof last_dev);
 
-	gm_snprintf(s, sizeof s,
-		"%s - Newer version%s available: %s%s%s%s%s",
-		GTA_WEBSITE,
-		last_stable[0] && last_dev[0] ? "s" : "",
-		last_stable[0] ? "release " : "",
-		last_stable[0] ? last_stable : "",
-		last_stable[0] && last_dev[0] ? " / " : "",
-		last_dev[0] ? "from CVS " : "",
-		last_dev[0] ? last_dev : "");
+	if ('\0' != last_stable[0] && '\0' != last_dev[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer versions available: release %s / from CVS %s"),
+			last_stable, last_dev);
+	} else if ('\0' != last_stable[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer version available: release %s"),
+			last_stable);
+	} else if ('\0' != last_dev[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer version available: from CVS %s"),
+			last_dev);
+	}
 
     gnet_prop_set_string(PROP_NEW_VERSION_STR, s);
 }
 
-/**
- * Check whether GTKG version is newer than supplied timestamp.
- */
-gboolean
-version_newer(const gchar *str, time_t stamp)
-{
-	version_t version;
-
-	if (!version_parse(str, &version))
-		return FALSE;			/* Not gtk-gnutella, or unparseable */
-
-	version_stamp(str, &version);
-
-	return version.timestamp >= stamp;
-}
-
 /**
  * Check version of servent, and if it's a gtk-gnutella more recent than we
  * are, record that fact and change the status bar.
  *
- * The `ip' is being passed solely for the tok_version_valid() call.
+ * The `addr' is being passed solely for the tok_version_valid() call.
  *
- * Returns TRUE if we properly checked the version, FALSE if we got something
+ * @returns TRUE if we properly checked the version, FALSE if we got something
  * looking as gtk-gnutella but which failed the token-based sanity checks.
  */
 gboolean
-version_check(const gchar *str, const gchar *token, guint32 ip)
+version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 {
 	version_t their_version;
 	version_t *target_version;
@@ -428,12 +425,26 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 			return FALSE;	/* Can't be correct */
 		}
 
-		error = tok_version_valid(str, token, strlen(token), ip);
+		error = tok_version_valid(str, token, strlen(token), addr);
+
+		/*
+		 * Unfortunately, if our token has expired, we can no longer
+		 * validate the tokens of the remote peers, since they are using
+		 * a different set of keys.
+		 *
+		 * This means an expired GTKG will blindly trust well-formed remote
+		 * tokens at face value.  But it's their fault, they should not run
+		 * an expired version!
+		 *		--RAM, 2005-12-21
+		 */
+
+		if (error == TOK_BAD_KEYS && tok_is_ancient(tm_time()))
+			error = TOK_OK;		/* Our keys have expired, cannot validate */
 
 		if (error != TOK_OK) {
             if (dbg) {
                 g_message("vendor string \"%s\" [%s] has wrong token "
-                    "\"%s\": %s ", str, ip_to_gchar(ip), token, 
+                    "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
             }
 			return FALSE;
@@ -471,15 +482,15 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 		target_version = &last_dev_version;
 	}
 
-	/* 
+	/*
 	 * Only compare a development version which has the same version
 	 * number with a more up to date version.
 	 * This will also avoid a 0.93.4u to be listed out of date if also a 0.94u
 	 * is available, while both are actually in development.
 	 * 		-- JA 15/04/2004
 	 */
-	
-	/* 
+
+	/*
 	 * Their version is more recent, but is unstable
 	 */
 	if (their_version.tag == 'u' && cmp < 0)
@@ -521,27 +532,38 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 
 	return TRUE;
 }
- 
+
 /**
  * Initialize version string.
  */
 void
 version_init(void)
 {
-	struct utsname un;
 	gchar buf[128];
+	time_t now = tm_time();
+	const gchar *sysname = "Unknown";
+	const gchar *machine = NULL;
 	gboolean ok;
-	time_t now = time(NULL);
 
-	if (uname(&un)) {
-		memset(&un, 0, sizeof un);
-		g_warning("uname() failed: %s:", g_strerror(errno));
+#ifdef HAS_UNAME
+	{
+		static struct utsname un;	/* Must survive this scope */
+	
+		if (0 == uname(&un)) {
+			sysname = un.sysname;
+			machine = un.machine;
+		} else {
+			g_warning("uname() failed: %s:", g_strerror(errno));
+		}
 	}
+#endif /* HAS_UNAME */
 
-	gm_snprintf(buf, sizeof(buf),
-		"gtk-gnutella/%s (%s; %s; %s %s)",
+	gm_snprintf(buf, sizeof buf,
+		"gtk-gnutella/%s (%s; %s; %s%s%s)",
 		GTA_VERSION_NUMBER, GTA_RELEASE, GTA_INTERFACE,
-		un.sysname, un.machine);
+		sysname,
+		machine && machine[0] ? " " : "",
+		machine ? machine : "");
 
 	version_string = atom_str_get(buf);
 	ok = version_parse(version_string, &our_version);
@@ -630,8 +652,9 @@ version_maybe_refuse(gint overtime)
 void
 version_ancient_warn(void)
 {
-	time_t now = time(NULL);
+	time_t now = tm_time();
 	gint lifetime, remain, elapsed;
+	time_t s;
 
 	g_assert(our_version.timestamp != 0);	/* version_init() called */
 
@@ -670,9 +693,30 @@ version_ancient_warn(void)
 
 	g_assert(remain >= 0);		/* None of the checks above have fired */
 
+	/*
+	 * Try to see whether the token will expire within the next
+	 * VERSION_ANCIENT_REMIND secs, looking for the minimum cutoff date.
+	 *
+	 * Indeed, it is possible to emit new versions without issuing a
+	 * new set of token keys, thereby constraining the lifetime of the
+	 * version.  This is usually what happens for bug-fixing releases
+	 * that do not introduce significant Gnutella features.
+	 */
+
+	for (s = now + VERSION_ANCIENT_REMIND; s > now; s -= SECS_PER_DAY) {
+		if (!tok_is_ancient(s))
+			break;
+	}
+
+	remain = MIN(remain, s - now);
+
+	/*
+	 * Let them know when version will expire soon...
+	 */
+
 	if (remain < VERSION_ANCIENT_REMIND)
         gnet_prop_set_guint32_val(PROP_ANCIENT_VERSION_LEFT_DAYS,
-			remain / 86400);
+			remain / SECS_PER_DAY);
 }
 
 /**
@@ -683,7 +727,7 @@ gboolean
 version_is_too_old(const gchar *vendor)
 {
 	version_t ver;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 	gint d;
 
 	version_stamp(vendor, &ver);		/* Fills ver->timestamp */
@@ -719,4 +763,4 @@ version_close(void)
 			version_str(&last_dev_version));
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
