@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.h,v 1.1 2004/08/25 05:40:39 rmanfredi Exp $
+ * $Id: hosts.h,v 1.4 2005/08/17 13:44:05 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -26,11 +26,13 @@
 #ifndef _if_core_hosts_h_
 #define _if_core_hosts_h_
 
-/*
+#include "lib/host_addr.h" /* For host_addr_t addr */
+
+/**
  * A gnutella host.
  */
 typedef struct gnutella_host {
-	guint32 ip;
+	host_addr_t addr;
 	guint16 port;
 } gnet_host_t;
 
