@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.h,v 1.1 2004/08/25 05:40:39 rmanfredi Exp $
+ * $Id: sockets.h,v 1.5 2005/08/14 12:31:49 rmanfredi Exp $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -38,6 +38,20 @@ enum {
     PROXY_SOCKSV5 = 5
 };
 
+/***
+ *** Network protocols
+ ***/
+enum {
+    NET_USE_BOTH  = 0,
+    NET_USE_IPV4  = 4,
+    NET_USE_IPV6  = 6,
+};
+
+enum {
+	CONNECT_F_FORCE	= (1 << 0),		/* Bypass limitation checks */
+	CONNECT_F_TLS	= (1 << 1),		/* Initiate a TLS connection */
+};
+
 #endif /* _if_core_sockets_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
