@@ -1,10 +1,8 @@
 /*
- * $Id: adns.c,v 1.4 2004/10/21 15:14:59 cbiere Exp $
+ * $Id: adns.c,v 1.27 2005/12/17 10:37:15 cbiere Exp $
  *
  * Copyright (c) 2004, Christian Biere
  *
- * Asynchronous DNS lookup.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,48 +23,62 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Asynchronous DNS lookup.
+ *
+ * @author Christian Biere
+ * @date 2004
+ */
+
 #include "common.h"
 
-RCSID("$Id: adns.c,v 1.4 2004/10/21 15:14:59 cbiere Exp $");
-
-#include <signal.h> /* signal(), SIGCHLD, SIG_IGN */
+RCSID("$Id: adns.c,v 1.27 2005/12/17 10:37:15 cbiere Exp $");
 
 #include "adns.h"
 #include "atoms.h"
 #include "inputevt.h"
 #include "misc.h"
 #include "glib-missing.h"
+#include "tm.h"
 #include "walloc.h"
+#include "socket.h"
 
 #include "override.h"		/* Must be the last header included */
 
-static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
+static guint32 common_dbg = 0;	/**< @bug XXX need to init lib's props --RAM */
 
 /* private data types */
 
 typedef struct adns_query {
-	GFunc user_callback;
+	void (*user_callback)(void);
 	gpointer user_data;
-	guint32 ip;
+	host_addr_t addr;
 	gboolean reverse;
 	gchar hostname[MAX_HOSTLEN + 1];
 } adns_query_t;
 
 typedef struct adns_async_write {
-	adns_query_t *query;	/* Original query */
-	gchar *buf;				/* Remaining data to write */
-	gint n;					/* Amount to write still */
+	adns_query_t *query;	/**< Original query */
+	gchar *buf;				/**< Remaining data to write */
+	gint n;					/**< Amount to write still */
 } adns_async_write_t;
 
 typedef struct adns_cache_entry {
-	gchar *hostname; /* atom */
-    guint32 ip;
+	gchar *hostname;		/**< atom */
+	host_addr_t addr;
 	time_t timestamp;
 } adns_cache_entry_t;
 
-/* Cache entries will expire after ADNS_CACHE_TIMEOUT seconds */
+/**
+ * Cache entries will expire after ADNS_CACHE_TIMEOUT seconds.
+ */
 #define ADNS_CACHE_TIMEOUT (5 * 60)
-/* Cache max. ADNS_CACHED_NUM of adns_cache_entry_t entries */
+/**
+ * Cache max. ADNS_CACHED_NUM of adns_cache_entry_t entries.
+ */
 #define ADNS_CACHED_NUM (1024)
 
 #define ADNS_PROCESS_TITLE "DNS helper for gtk-gnutella"
@@ -86,11 +98,11 @@ static adns_cache_t *adns_cache = NULL;
 static gint adns_query_fd = -1;
 static guint adns_query_event_id = 0;
 static guint adns_reply_event_id = 0;
-static gboolean is_helper = FALSE;		/* Are we the DNS helper process? */
+static gboolean is_helper = FALSE;		/**< Are we the DNS helper process? */
 
-static gboolean adns_helper_alive = TRUE;
-
-/* private macros */
+/**
+ * Private macros.
+ */
 
 #define CLOSE_IF_VALID(fd)	\
 do {						\
@@ -98,9 +110,11 @@ do {						\
 		close(fd);			\
 		fd = -1;			\
 	} 						\
-} while(0) 					
+} while(0)
 
-/* private functions */
+/**
+ * Private functions.
+ */
 
 static adns_cache_t *
 adns_cache_init(void)
@@ -108,7 +122,7 @@ adns_cache_init(void)
 	adns_cache_t *cache;
 	guint i;
 
-	cache = g_malloc(sizeof(*cache)); 
+	cache = g_malloc(sizeof(*cache));
 	cache->size = G_N_ELEMENTS(cache->entries);
 	cache->oldest = 0;
 	cache->timeout = ADNS_CACHE_TIMEOUT;
@@ -119,9 +133,9 @@ adns_cache_init(void)
 }
 
 /**
- *	Frees all memory allocated by the cache and returns NULL. 
+ * Frees all memory allocated by the cache and returns NULL.
  */
-adns_cache_t *
+void
 adns_cache_free(adns_cache_t *cache)
 {
 	guint i;
@@ -132,8 +146,7 @@ adns_cache_free(adns_cache_t *cache)
 		if (NULL != cache->entries[i].hostname)
 			atom_str_free(cache->entries[i].hostname);
 	cache->size = 0;
-	g_free(cache);
-	return NULL;
+	G_FREE_NULL(cache);
 }
 
 /* these are not needed anywhere else so undefine them */
@@ -141,13 +154,13 @@ adns_cache_free(adns_cache_t *cache)
 #undef ADNS_CACHE_TIMEOUT
 
 /**
- * Adds ``hostname'' and ``ip'' to the cache. The cache is implemented
+ * Adds ``hostname'' and ``addr'' to the cache. The cache is implemented
  * as a wrap-around FIFO. In case it's full, the oldest entry will be
- * overwritten. 
+ * overwritten.
  */
 static void
 adns_cache_add(adns_cache_t *cache, time_t now,
-	const gchar *hostname, guint32 ip)
+	const gchar *hostname, const host_addr_t addr)
 {
 	adns_cache_entry_t *entry;
 	gchar *atom;
@@ -163,7 +176,7 @@ adns_cache_add(adns_cache_t *cache, time_t now,
 	}
 	entry->hostname = atom;
 	entry->timestamp = now;
-	entry->ip = ip;
+	entry->addr = addr;
 	g_hash_table_insert(cache->hashtab, entry->hostname,
 		GUINT_TO_POINTER(cache->oldest));
 
@@ -174,13 +187,13 @@ adns_cache_add(adns_cache_t *cache, time_t now,
 /**
  * Looks for ``hostname'' in ``cache'' wrt to cache->timeout. If
  * ``hostname'' is not found or the entry is expired, FALSE will be
- * returned. Expired entries will be removed! ``ip'' is allowed to
+ * returned. Expired entries will be removed! ``addr'' is allowed to
  * be NULL, otherwise the cached IP will be stored into the variable
- * ``ip'' points to. 
+ * ``addr'' points to.
  */
 static gboolean
 adns_cache_lookup(adns_cache_t *cache, time_t now,
-	const gchar *hostname, guint32 *ip)
+	const gchar *hostname, host_addr_t *addr)
 {
 	guint i;
 	gpointer key;
@@ -195,23 +208,25 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
 	atom = atom_str_get(hostname);
 	found = g_hash_table_lookup_extended(cache->hashtab, atom, &key, &val);
 	g_assert(!found || atom == key);
-	atom_str_free(atom); /* Do not yet nullify for the assertion below */
 
-	if (!found)
+	if (!found) {
+		atom_str_free(atom);
 		return FALSE;
+	}
 
 	i = GPOINTER_TO_UINT(val);
 	g_assert(i < cache->size);
 	entry = &cache->entries[i];
 	g_assert(atom == entry->hostname);
+	atom_str_free(atom);
 	atom = NULL;
 
 	if (delta_time(now, entry->timestamp) < cache->timeout) {
-		if (NULL != ip)
-			*ip = entry->ip;
+		if (NULL != addr)
+			*addr = entry->addr;
 		if (common_dbg > 0)
-			g_warning("adns_cache_lookup: \"%s\" cached (ip=%s)",
-				entry->hostname, ip_to_gchar(entry->ip));
+			g_warning("adns_cache_lookup: \"%s\" cached (addr=%s)",
+				entry->hostname, host_addr_to_string(entry->addr));
 		return TRUE;
 	} else {
 		if (common_dbg > 0)
@@ -221,7 +236,7 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
 		atom_str_free(key);
 		entry->hostname = NULL;
 		entry->timestamp = 0;
-		entry->ip = 0;
+		entry->addr = zero_host_addr;
 	}
 
 	return FALSE;
@@ -246,10 +261,10 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
 
 		if (do_write)
 			ret = write(fd, buf, n);
-		else 
+		else
 			ret = read(fd, buf, n);
-	
-		if ((ssize_t) -1 == ret && errno != EAGAIN && errno != EINTR) {
+
+		if ((ssize_t) -1 == ret && errno != VAL_EAGAIN && errno != EINTR) {
             /* Ignore the failure, if the parent process is gone.
                This prevents an unnecessary warning when quitting. */
             if (!is_helper || getppid() != 1)
@@ -258,7 +273,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
 			return FALSE;
 		} else if (0 == ret) {
 			/*
-			 * Don't warn on EOF if we're the children process and our
+			 * Don't warn on EOF if this is the child process and the
 			 * parent is gone.
 			 */
 			if (!do_write && !(is_helper && getppid() == 1))
@@ -275,7 +290,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
 }
 
 /**
- * read the complete buffer ``buf'' of size ``len'' from file descriptor ``fd''
+ * Read the complete buffer ``buf'' of size ``len'' from file descriptor ``fd''
  *
  * @return TRUE on success, FALSE if the operation failed
  */
@@ -286,7 +301,7 @@ adns_do_read(gint fd, gpointer buf, size_t len)
 }
 
 /**
- * write the complete buffer ``buf'' of size ``len'' to file descriptor ``fd''
+ * Write the complete buffer ``buf'' of size ``len'' to file descriptor ``fd''
  *
  * @return TRUE on success, FALSE if the operation failed
  */
@@ -297,36 +312,43 @@ adns_do_write(gint fd, gpointer buf, size_t len)
 }
 
 /**
- * copies user_callback and user_data from the query buffer to the
+ * Copies user_callback and user_data from the query buffer to the
  * reply buffer. This function won't fail. However, if gethostbyname()
- * fails ``reply->ip'' will be set to zero.
+ * fails ``reply->addr'' will be set to zero.
  */
 static void
 adns_gethostbyname(const adns_query_t *query, adns_query_t *reply)
 {
 	const gchar *host;
-	
+
 	g_assert(NULL != query);
 	g_assert(NULL != reply);
 
 	if (common_dbg > 1) {
 		g_message("adns_gethostbyname: Resolving \"%s\" ...",
-				query->reverse ? ip_to_gchar(query->ip) : query->hostname);
+			query->reverse
+				? host_addr_to_string(query->addr)
+				: query->hostname);
 	}
 
 	reply->user_callback = query->user_callback;
 	reply->user_data = query->user_data;
 	reply->reverse = query->reverse;
-	reply->ip = query->reverse ? query->ip : host_to_ip(query->hostname);
-	host = query->reverse ? ip_to_host(query->ip) : query->hostname;
+	reply->addr = query->reverse
+		? query->addr
+		: name_to_host_addr(query->hostname);
+	host = query->reverse
+		? host_addr_to_name(query->addr)
+		: query->hostname;
 	g_strlcpy(reply->hostname, host ? host : "", sizeof reply->hostname);
 }
 
 /**
  * The ``main'' function of the adns helper process (server).
+ *
  * Simply reads requests (queries) from fd_in, performs a DNS lookup for it
  * and writes the result to fd_out. All operations should be blocking. Exits
- * in case of non-recoverable error during read or write.  
+ * in case of non-recoverable error during read or write.
  */
 static void
 adns_helper(gint fd_in, gint fd_out)
@@ -336,7 +358,9 @@ adns_helper(gint fd_in, gint fd_out)
 	g_set_prgname(ADNS_PROCESS_TITLE);
 	gm_setproctitle(g_get_prgname());
 
-	signal(SIGQUIT, SIG_IGN);		/* Avoid core dumps on SIGQUIT */
+#ifdef SIGQUIT 
+	set_signal(SIGQUIT, SIG_IGN);	/* Avoid core dumps on SIGQUIT */
+#endif
 
 	is_helper = TRUE;
 
@@ -358,22 +382,22 @@ adns_invoke_user_callback(adns_query_t *reply)
 {
 	if (reply->reverse) {
 		adns_reverse_callback_t func;
-	   
+
 		func = (adns_reverse_callback_t) reply->user_callback;
 		func(reply->hostname[0] != '\0' ? reply->hostname : NULL,
 			reply->user_data);
 	} else {
 		adns_callback_t func;
-	   
+
 		func = (adns_callback_t) reply->user_callback;
-		func(reply->ip, reply->user_data);
+		func(is_host_addr(reply->addr) ? &reply->addr : NULL, reply->user_data);
 	}
 }
 
 /**
  * Handles the query in synchronous (blocking) mode and is used if the
  * dns helper is busy i.e., the pipe buffer is full or in case the dns
- * helper is dead.  
+ * helper is dead.
  */
 static void
 adns_fallback(const adns_query_t *query)
@@ -398,27 +422,28 @@ adns_reply_callback(gpointer data, gint source, inputevt_cond_t condition)
 	static size_t n = 0;
 	static adns_query_t reply;
 
-	g_assert(NULL == data);	
-	g_assert(condition & (INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION));
-	g_assert(sizeof(reply) >= n);
+	g_assert(NULL == data);
+	g_assert(condition & INPUT_EVENT_RX);
+	g_assert(sizeof reply >= n);
 
 again:
-	if (sizeof(reply) > n) {
-		gpointer buf = (gchar *) &reply + n;
+	if (sizeof reply > n) {
+		gpointer buf = cast_to_gchar_ptr(&reply) + n;
 		ssize_t ret;
 
-		ret = read(source, buf, sizeof(reply)-n);
-		
+		ret = read(source, buf, sizeof reply - n);
+
 		if (0 == ret) {
-			errno = ECONNRESET;	
+			errno = ECONNRESET;
 			ret = (ssize_t) -1;
 		}
 		/* FALL THROUGH */
 		if ((ssize_t) -1 == ret) {
-			if (errno != EAGAIN && errno != EINTR) {
+			if (errno != VAL_EAGAIN && errno != EINTR) {
 				g_warning("adns_reply_callback: read() failed: %s",
 					g_strerror(errno));
 				inputevt_remove(adns_reply_event_id);
+				adns_reply_event_id = 0;
 				g_warning("adns_reply_callback: removed myself");
 				close(source);
 				return;
@@ -430,24 +455,24 @@ again:
 		n += (size_t) ret;
 	}
 	/* FALL THROUGH */
-	if (sizeof(reply) == n) {
-		time_t now = time(NULL);
+	if (sizeof reply == n) {
+		time_t now = tm_time();
 
 		if (common_dbg > 1) {
-			const gchar *host, *ip;
+			const gchar *host, *addr;
 
 			host = reply.hostname;
-			ip = ip_to_gchar(reply.ip);
+			addr = host_addr_to_string(reply.addr);
 			g_warning("adns_reply_callback: Resolved \"%s\" to \"%s\".",
-				reply.reverse ? ip : host, reply.reverse ? host : ip);
+				reply.reverse ? addr : host, reply.reverse ? host : addr);
 		}
 
 		g_assert(NULL != reply.user_callback);
-		if (	
+		if (
 				!reply.reverse &&
 				!adns_cache_lookup(adns_cache, now, reply.hostname, NULL)
 		) {
-			adns_cache_add(adns_cache, now, reply.hostname, reply.ip);
+			adns_cache_add(adns_cache, now, reply.hostname, reply.addr);
 		}
 
 		adns_invoke_user_callback(&reply);
@@ -464,13 +489,13 @@ adns_async_write_alloc(const adns_query_t *query, size_t n)
 {
 	adns_async_write_t *remain;
 
-	g_assert(n < sizeof(*query));
+	g_assert(n < sizeof *query);
 
-	remain = walloc(sizeof(*remain));
-	remain->query = walloc(sizeof(*query));
-	memcpy(remain->query, query, sizeof(*query));
-	remain->buf = (gchar *) remain->query + n;
-	remain->n = sizeof(*query) - n;
+	remain = walloc(sizeof *remain);
+	remain->query = walloc(sizeof *query);
+	memcpy(remain->query, query, sizeof *query);
+	remain->buf = cast_to_gchar_ptr(remain->query) + n;
+	remain->n = sizeof *query - n;
 
 	return remain;
 }
@@ -481,8 +506,8 @@ adns_async_write_alloc(const adns_query_t *query, size_t n)
 static void
 adns_async_write_free(adns_async_write_t *remain)
 {
-	wfree(remain->query, sizeof(*remain->query));
-	wfree(remain, sizeof(*remain));
+	wfree(remain->query, sizeof *remain->query);
+	wfree(remain, sizeof *remain);
 }
 
 /**
@@ -509,19 +534,19 @@ adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
 		ssize_t ret;
 
 		ret = write(dest, remain->buf, remain->n);
-		
+
 		if (0 == ret) {
-			errno = ECONNRESET;	
+			errno = ECONNRESET;
 			ret = (ssize_t) -1;
 		}
 		/* FALL THROUGH */
 		if ((ssize_t) -1 == ret) {
-			if (errno != EAGAIN && errno != EINTR)
+			if (errno != VAL_EAGAIN && errno != EINTR)
 				goto error;
 			return;
 		}
 
-		g_assert(ret > 0);	
+		g_assert(ret > 0);
 		remain->n -= (size_t) ret;
 		remain->buf += (size_t) ret;
 		g_assert(remain->n >= 0);
@@ -529,6 +554,7 @@ adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
 
 	inputevt_remove(adns_query_event_id);
 	adns_query_event_id = 0;
+	
 	adns_async_write_free(remain);
 
 	return;
@@ -536,9 +562,9 @@ adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
 error:
 	g_warning("adns_query_callback: write() failed: %s", g_strerror(errno));
 abort:
-	inputevt_remove(adns_query_event_id);
 	g_warning("adns_query_callback: removed myself");
-	adns_helper_alive = FALSE;
+	inputevt_remove(adns_query_event_id);
+	adns_query_event_id = 0;
 	CLOSE_IF_VALID(adns_query_fd);
 	g_warning("adns_query_callback: using fallback");
 	adns_fallback(remain->query);
@@ -561,7 +587,11 @@ adns_init(void)
 		g_warning("adns_init: pipe() failed: %s", g_strerror(errno));
 		goto prefork_failure;
 	}
-	signal(SIGCHLD, SIG_IGN); /* prevent a zombie */
+	
+#ifdef SIGCHLD 
+	set_signal(SIGCHLD, SIG_IGN); /* prevent a zombie */
+#endif
+
 	pid = fork();
 	if ((pid_t) -1 == pid) {
 		g_warning("adns_init: fork() failed: %s", g_strerror(errno));
@@ -570,41 +600,45 @@ adns_init(void)
 	if (0 == pid) {
 		/* child process */
 		gint null;
-	
+
 		close(fd_query[1]);
 		close(fd_reply[0]);
    		close(STDIN_FILENO);  /* Just in case */
 		null = open("/dev/null", O_RDONLY);
 		if (-1 == null)
-			g_error("adns_init: Could not open() /dev/null");
+			g_error("adns_init: Could not open() /dev/null: %s",
+				g_strerror(errno));
 		g_assert(STDIN_FILENO == null);
-		adns_helper(fd_query[0], fd_reply[1]); 
+		adns_helper(fd_query[0], fd_reply[1]);
 		g_assert_not_reached();
 		_exit(EXIT_SUCCESS);
-	} 
+	}
 
 	/* parent process */
 	close(fd_query[0]);
 	close(fd_reply[1]);
 	adns_query_fd = fd_query[1];
-	fcntl(adns_query_fd, F_SETFL, O_NONBLOCK);
-	fcntl(fd_reply[0], F_SETFL, O_NONBLOCK);
-	adns_cache = adns_cache_init();
-	adns_reply_event_id = inputevt_add(fd_reply[0],
-		INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
-		adns_reply_callback, NULL);
-	return;
 
+	socket_set_nonblocking(adns_query_fd);
+	socket_set_nonblocking(fd_reply[0]);
+	
+	adns_reply_event_id = inputevt_add(fd_reply[0], INPUT_EVENT_RX,
+							adns_reply_callback, NULL);
+	/* FALL THROUGH */
 prefork_failure:
 
-	CLOSE_IF_VALID(fd_query[0]);
-	CLOSE_IF_VALID(fd_query[1]);
-	CLOSE_IF_VALID(fd_reply[0]);
-	CLOSE_IF_VALID(fd_reply[1]);
+	if (!adns_reply_event_id) {
+		g_warning("Cannot use ADNS; DNS lookups may cause stalling");
+		CLOSE_IF_VALID(fd_query[0]);
+		CLOSE_IF_VALID(fd_query[1]);
+		CLOSE_IF_VALID(fd_reply[0]);
+		CLOSE_IF_VALID(fd_reply[1]);
+	}
+
+	adns_cache = adns_cache_init();
 }
 
 /**
- *
  * @return TRUE on success, FALSE on failure.
  */
 static gboolean
@@ -613,9 +647,9 @@ adns_send_query(const adns_query_t *query)
 	ssize_t written;
 	adns_query_t q;
 
-	if (!adns_helper_alive || 0 != adns_query_event_id)
+	if (!adns_reply_event_id || 0 != adns_query_event_id)
 		return FALSE;
-		
+
 	g_assert(adns_query_fd >= 0);
 	q = *query;
 
@@ -623,12 +657,13 @@ adns_send_query(const adns_query_t *query)
 	 * Try to write the query atomically into the pipe.
 	 */
 
-	written = write(adns_query_fd, &q, sizeof(q));
+	written = write(adns_query_fd, &q, sizeof q);
 	if (written == (ssize_t) -1) {
-		if (errno != EINTR && errno != EAGAIN) {
+		if (errno != EINTR && errno != VAL_EAGAIN) {
 			g_warning("adns_resolve: write() failed: %s",
 				g_strerror(errno));
-			adns_helper_alive = FALSE;
+			inputevt_remove(adns_reply_event_id);
+			adns_reply_event_id = 0;
 			CLOSE_IF_VALID(adns_query_fd);
 			return FALSE;
 		}
@@ -643,26 +678,26 @@ adns_send_query(const adns_query_t *query)
 	 * can absorb new data.
 	 */
 
-	if (written < (ssize_t) sizeof(q)) {
+	if (written < (ssize_t) sizeof q) {
 		adns_async_write_t *aq = adns_async_write_alloc(&q, written);
 
-		adns_query_event_id = inputevt_add(adns_query_fd,
-			INPUT_EVENT_WRITE | INPUT_EVENT_EXCEPTION,
-			adns_query_callback, aq);
+		adns_query_event_id = inputevt_add(adns_query_fd, INPUT_EVENT_WX,
+								adns_query_callback, aq);
 	}
-	
+
 	return TRUE;
 }
 
 /**
- * Creates a DNS resolve query for ``hostname''. The given function
- * ``user_callback'' (which MUST NOT be NULL) will be invoked with
- * the resolved IP address and ``user_data'' as its parameters. The
- * IP address 0.0.0.0 i.e., ``(guint32) 0'' is used to indicate a
- * failure. In case the hostname is given as an IP string, it will
- * be directly converted and the callback immediately invoked. If
- * the adns helper process is ``out of service'' the query will be
- * resolved synchronously.
+ * Creates a DNS resolve query for ``hostname''.
+ *
+ * The given function ``user_callback'' (which MUST NOT be NULL)
+ * will be invoked with the resolved IP address and ``user_data''
+ * as its parameters. The IP address 0.0.0.0 i.e., ``(guint32) 0''
+ * is used to indicate a failure. In case the hostname is given as
+ * an IP string, it will be directly converted and the callback
+ * immediately invoked. If the adns helper process is ``out of service''
+ * the query will be resolved synchronously.
  *
  * @return TRUE if the resolution is asynchronous i.e., the callback
  * will be called AFTER adns_resolve() returned. If the resolution is
@@ -673,20 +708,20 @@ gboolean
 adns_resolve(const gchar *hostname,
 	adns_callback_t user_callback, gpointer user_data)
 {
-	gsize hostname_len;
 	static adns_query_t query, reply;
+	size_t hostname_len;
 
 	g_assert(NULL != hostname);
 	g_assert(NULL != user_callback);
 
-	query.user_callback = (GFunc) user_callback;
+	query.user_callback = (void (*)(void)) user_callback;
 	query.user_data = user_data;
 	query.reverse = FALSE;
-	query.ip = 0;
+	query.addr = zero_host_addr;
 	reply = query;
-	
-	reply.ip = gchar_to_ip(hostname);
-	if (0 != reply.ip) {
+
+	reply.addr = string_to_host_addr(hostname, NULL);
+	if (is_host_addr(reply.addr)) {
 		adns_invoke_user_callback(&reply);
 		return FALSE; /* synchronous */
 	}
@@ -694,22 +729,25 @@ adns_resolve(const gchar *hostname,
 	hostname_len = g_strlcpy(query.hostname, hostname, sizeof(query.hostname));
 	if (hostname_len >= sizeof(query.hostname)) {
 		/* truncation detected */
-		reply.ip = 0;
+		reply.addr = zero_host_addr;
 		adns_invoke_user_callback(&reply);
 		return FALSE; /* synchronous */
 	}
 
 	ascii_strlower(query.hostname, hostname);
-	if (adns_cache_lookup(adns_cache, time(NULL), query.hostname, &reply.ip)) {
+	if (
+		adns_cache_lookup(adns_cache, tm_time(), query.hostname, &reply.addr)
+	) {
 		adns_invoke_user_callback(&reply);
 		return FALSE; /* synchronous */
 	}
 
 	if (adns_send_query(&query))
-		return TRUE; /* asynchronous */ 
+		return TRUE; /* asynchronous */
 
-	g_warning("adns_resolve: using synchronous resolution for \"%s\"",
-		query.hostname);
+	if (adns_reply_event_id)
+		g_warning("adns_resolve: using synchronous resolution for \"%s\"",
+			query.hostname);
 
 	adns_fallback(&query);
 
@@ -717,7 +755,7 @@ adns_resolve(const gchar *hostname,
 }
 
 /**
- * Creates a DNS reverse lookup query for ``ip''. The given function
+ * Creates a DNS reverse lookup query for ``addr''. The given function
  * ``user_callback'' (which MUST NOT be NULL) will be invoked with
  * the resolved hostname and ``user_data'' as its parameters. If the lookup
  * failed, the callback will be invoked with ``hostname'' NULL. If the adns
@@ -730,24 +768,24 @@ adns_resolve(const gchar *hostname,
  * returned, adns_reverse_lookup() returns FALSE.
  */
 gboolean
-adns_reverse_lookup(guint32 ip,
+adns_reverse_lookup(const host_addr_t addr,
 	adns_reverse_callback_t user_callback, gpointer user_data)
 {
 	static adns_query_t query;
 
-	g_assert(NULL != user_callback);
+	g_assert(user_callback);
 
-	query.user_callback = (GFunc) user_callback;
+	query.user_callback = (void (*)(void)) user_callback;
 	query.user_data = user_data;
-	query.ip = ip;
+	query.addr = addr;
 	query.reverse = TRUE;
 	query.hostname[0] = '\0';
 
 	if (adns_send_query(&query))
-		return TRUE; /* asynchronous */ 
+		return TRUE; /* asynchronous */
 
 	g_warning("adns_reverse_lookup: using synchronous resolution for \"%s\"",
-		ip_to_gchar(query.ip));
+		host_addr_to_string(query.addr));
 
 	adns_fallback(&query);
 
@@ -760,8 +798,17 @@ adns_reverse_lookup(guint32 ip,
 void
 adns_close(void)
 {
-	inputevt_remove(adns_reply_event_id);
-	adns_cache = adns_cache_free(adns_cache);
+	if (adns_reply_event_id) {
+		inputevt_remove(adns_reply_event_id);
+		adns_reply_event_id = 0;
+	}
+	if (adns_query_event_id) {
+		inputevt_remove(adns_query_event_id);
+		adns_query_event_id = 0;
+	}
+	
+	adns_cache_free(adns_cache);
+	adns_cache = NULL;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
