@@ -1,14 +1,8 @@
 /*
- * $Id: bg.c,v 1.1 2004/08/24 21:01:24 rmanfredi Exp $
+ * $Id: bg.c,v 1.9 2005/08/31 21:37:32 rmanfredi Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Background task management.
- *
- * A background task is some CPU or I/O intensive operation that needs to
- * be split up in small chunks of processing because it would block the
- * process for too long if executed atomically.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -29,26 +23,38 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Background task management.
+ *
+ * A background task is some CPU or I/O intensive operation that needs to
+ * be split up in small chunks of processing because it would block the
+ * process for too long if executed atomically.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: bg.c,v 1.1 2004/08/24 21:01:24 rmanfredi Exp $");
-
-#include <setjmp.h>
-#include <glib.h>
+RCSID("$Id: bg.c,v 1.9 2005/08/31 21:37:32 rmanfredi Exp $");
 
 #include "bg.h"
+#include "tm.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
-#define BT_MAGIC		0xbacc931dU		/* Internal bgtask magic number */
+#define BT_MAGIC		0xbacc931dU		/**< Internal bgtask magic number */
 
-#define MAX_LIFE		150000			/* In useconds, MUST be << 1 sec */
-#define MIN_LIFE		40000			/* Min lifetime per task, in usecs */
-#define DELTA_FACTOR	4				/* Max variations are 400% */
+#define MAX_LIFE		150000			/**< In useconds, MUST be << 1 sec */
+#define MIN_LIFE		40000			/**< Min lifetime per task, in usecs */
+#define DELTA_FACTOR	4				/**< Max variations are 400% */
 
 static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
 
-/*
+/**
  * Internal representation of a user-defined task.
  *
  * `step' is the current processing step.  Several processing steps can be
@@ -62,54 +68,54 @@ static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
  * `stepvec' is the set of steps we have to run (normally in sequence).
  */
 struct bgtask {
-	guint magic;			/* Magic number */
-	guint32 flags;			/* Operating flags */
-	gchar *name;			/* Task name */
-	gint step;				/* Current processing step */
-	gint seqno;				/* Number of calls at same step */
-	bgstep_cb_t *stepvec;	/* Set of steps to run in sequence */
-	gint stepcnt;			/* Amount of steps in the `stepvec' array */
-	gpointer ucontext;		/* User context */
-	time_t ctime;			/* Creation time */
-	gint wtime;				/* Wall-clock run time sofar, in ms */
-	bgclean_cb_t uctx_free;	/* Free routine for context */
-	bgdone_cb_t done_cb;	/* Called when done */
-	gpointer done_arg;		/* "done" callback argument */
-	gint exitcode;			/* Final "exit" code */
-	bgsig_t signal;			/* Last signal delivered */
-	GSList *signals;		/* List of signals pending delivery */
-	jmp_buf env;			/* Only valid when TASK_F_RUNNING */
-	GTimeVal start;			/* Start time of scheduling "tick" */
-	gint ticks;				/* Scheduling ticks for time slice */
-	gint ticks_used;		/* Amount of ticks used by processing step */
-	gint prev_ticks;		/* Ticks used when measuring `elapsed' below */
-	gint elapsed;			/* Elapsed during last run, in usec */
-	gdouble tick_cost;		/* Time in ms. spent by each tick */
-	bgsig_cb_t sigh[BG_SIG_COUNT];	/* Signal handlers */
+	guint magic;			/**< Magic number */
+	guint32 flags;			/**< Operating flags */
+	gchar *name;			/**< Task name */
+	gint step;				/**< Current processing step */
+	gint seqno;				/**< Number of calls at same step */
+	bgstep_cb_t *stepvec;	/**< Set of steps to run in sequence */
+	gint stepcnt;			/**< Amount of steps in the `stepvec' array */
+	gpointer ucontext;		/**< User context */
+	time_t ctime;			/**< Creation time */
+	gint wtime;				/**< Wall-clock run time sofar, in ms */
+	bgclean_cb_t uctx_free;	/**< Free routine for context */
+	bgdone_cb_t done_cb;	/**< Called when done */
+	gpointer done_arg;		/**< "done" callback argument */
+	gint exitcode;			/**< Final "exit" code */
+	bgsig_t signal;			/**< Last signal delivered */
+	GSList *signals;		/**< List of signals pending delivery */
+	jmp_buf env;			/**< Only valid when TASK_F_RUNNING */
+	tm_t start;				/**< Start time of scheduling "tick" */
+	gint ticks;				/**< Scheduling ticks for time slice */
+	gint ticks_used;		/**< Amount of ticks used by processing step */
+	gint prev_ticks;		/**< Ticks used when measuring `elapsed' below */
+	gint elapsed;			/**< Elapsed during last run, in usec */
+	gdouble tick_cost;		/**< Time in ms. spent by each tick */
+	bgsig_cb_t sigh[BG_SIG_COUNT];	/**< Signal handlers */
 
 	/*
 	 * Daemon tasks.
 	 */
 
-	GSList *wq;				/* Work queue (daemon task only) */
-	bgstart_cb_t start_cb;	/* Called when starting working on an item */
-	bgend_cb_t end_cb;		/* Called when finished working on an item */
-	bgclean_cb_t item_free;	/* Free routine for work queue items */
-	bgnotify_cb_t notify;	/* Start/Stop notification (optional) */
+	GSList *wq;				/**< Work queue (daemon task only) */
+	bgstart_cb_t start_cb;	/**< Called when starting working on an item */
+	bgend_cb_t end_cb;		/**< Called when finished working on an item */
+	bgclean_cb_t item_free;	/**< Free routine for work queue items */
+	bgnotify_cb_t notify;	/**< Start/Stop notification (optional) */
 };
 
 /*
  * Operating flags.
  */
 
-#define TASK_F_EXITED		0x00000001	/* Exited */
-#define TASK_F_SIGNAL		0x00000002	/* Signal received */
-#define TASK_F_RUNNING		0x00000004	/* Task is running */
-#define TASK_F_ZOMBIE		0x00000008	/* Task waiting status collect */
-#define TASK_F_NOTICK		0x00000010	/* Do no recompute tick info */
-#define TASK_F_SLEEPING		0x00000020	/* Task is sleeping */
-#define TASK_F_RUNNABLE		0x00000040	/* Task is runnable */
-#define TASK_F_DAEMON		0x80000000	/* Task is a daemon */
+#define TASK_F_EXITED		0x00000001	/**< Exited */
+#define TASK_F_SIGNAL		0x00000002	/**< Signal received */
+#define TASK_F_RUNNING		0x00000004	/**< Task is running */
+#define TASK_F_ZOMBIE		0x00000008	/**< Task waiting status collect */
+#define TASK_F_NOTICK		0x00000010	/**< Do no recompute tick info */
+#define TASK_F_SLEEPING		0x00000020	/**< Task is sleeping */
+#define TASK_F_RUNNABLE		0x00000040	/**< Task is runnable */
+#define TASK_F_DAEMON		0x80000000	/**< Task is a daemon */
 
 /*
  * Access routines to internal fields.
@@ -125,7 +131,7 @@ static GSList *sleepq = NULL;
 static gint runcount = 0;
 static GSList *dead_tasks = NULL;
 
-/*
+/**
  * bg_sched_add
  *
  * Add new task to the scheduler (run queue).
@@ -143,7 +149,7 @@ static void bg_sched_add(struct bgtask *bt)
 	runq = g_slist_append(runq, bt);
 }
 
-/*
+/**
  * bg_sched_remove
  *
  * Remove task from the scheduler (run queue).
@@ -158,7 +164,7 @@ static void bg_sched_remove(struct bgtask *bt)
 	bt->flags &= ~TASK_F_RUNNABLE;
 }
 
-/*
+/**
  * bg_sched_pick
  *
  * Pick next task to schedule.
@@ -172,14 +178,14 @@ static struct bgtask *bg_sched_pick(void)
 	return (runq != NULL) ? (struct bgtask *) runq->data : NULL;
 }
 
-/*
+/**
  * bg_task_suspend
  *
  * Suspend task.
  */
 static void bg_task_suspend(struct bgtask *bt)
 {
-	GTimeVal end;
+	tm_t end;
 	gint elapsed;
 
 	g_assert(bt->flags & TASK_F_RUNNING);
@@ -191,7 +197,7 @@ static void bg_task_suspend(struct bgtask *bt)
 	 * Update task running time.
 	 */
 
-	g_get_current_time(&end);
+	tm_now_exact(&end);
 	elapsed = (glong) ((end.tv_sec - bt->start.tv_sec) * 1000 * 1000 +
 		(end.tv_usec - bt->start.tv_usec));
 
@@ -233,10 +239,10 @@ static void bg_task_suspend(struct bgtask *bt)
 	}
 }
 
-/*
+/**
  * bg_task_resume
  *
- * Suspend task.
+ * Resume task execution.
  */
 static void bg_task_resume(struct bgtask *bt)
 {
@@ -245,10 +251,10 @@ static void bg_task_resume(struct bgtask *bt)
 	bg_sched_remove(bt);
 	bt->flags |= TASK_F_RUNNING;
 
-	g_get_current_time(&bt->start);
+	tm_now_exact(&bt->start);
 }
 
-/*
+/**
  * bg_sched_sleep
  *
  * Add task to the sleep queue.
@@ -265,7 +271,7 @@ static void bg_sched_sleep(struct bgtask *bt)
 	sleepq = g_slist_prepend(sleepq, bt);
 }
 
-/*
+/**
  * bg_sched_wakeup
  *
  * Remove task from the sleep queue and insert it to the runqueue.
@@ -284,13 +290,13 @@ static void bg_sched_wakeup(struct bgtask *bt)
 
 static struct bgtask *current_task = NULL;
 
-/*
+/**
  * bg_task_switch
  *
  * Switch to new task `bt'.
  * If argument is NULL, suspends current task.
  *
- * Returns previously scheduled task, if any.
+ * @returns previously scheduled task, if any.
  */
 static struct bgtask *bg_task_switch(struct bgtask *bt)
 {
@@ -312,7 +318,7 @@ static struct bgtask *bg_task_switch(struct bgtask *bt)
 	return old;
 }
 
-/*
+/**
  * bg_task_create
  *
  * Create a new background task.
@@ -327,15 +333,15 @@ static struct bgtask *bg_task_switch(struct bgtask *bt)
  * The user-supplied argument `done_arg' will also be given to that callback.
  * Note that "done" does not necessarily mean success.
  *
- * Returns an opaque handle.
+ * @returns an opaque handle.
  */
 gpointer bg_task_create(
-	gchar *name,						/* Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/* Work to perform (copied) */
-	gpointer ucontext,					/* User context */
-	bgclean_cb_t ucontext_free,			/* Free routine for context */
-	bgdone_cb_t done_cb,				/* Notification callback when done */
-	gpointer done_arg)					/* Callback argument */
+	gchar *name,						/**< Task name (for tracing) */
+	bgstep_cb_t *steps, gint stepcnt,	/**< Work to perform (copied) */
+	gpointer ucontext,					/**< User context */
+	bgclean_cb_t ucontext_free,			/**< Free routine for context */
+	bgdone_cb_t done_cb,				/**< Notification callback when done */
+	gpointer done_arg)					/**< Callback argument */
 {
 	struct bgtask *bt;
 	gint stepsize;
@@ -363,7 +369,7 @@ gpointer bg_task_create(
 	return bt;
 }
 
-/*
+/**
  * bg_daemon_create
  *
  * A "daemon" is a task equipped with a work queue.
@@ -385,14 +391,14 @@ gpointer bg_task_create(
  * Use bg_daemon_enqueue() to enqueue more work to the daemon.
  */
 gpointer bg_daemon_create(
-	gchar *name,						/* Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/* Work to perform (copied) */
-	gpointer ucontext,					/* User context */
-	bgclean_cb_t ucontext_free,			/* Free routine for context */
-	bgstart_cb_t start_cb,				/* Starting working on an item */
-	bgend_cb_t end_cb,					/* Done working on an item */
-	bgclean_cb_t item_free,				/* Free routine for work queue items */
-	bgnotify_cb_t notify)				/* Start/Stop notify (optional) */
+	gchar *name,						/**< Task name (for tracing) */
+	bgstep_cb_t *steps, gint stepcnt,	/**< Work to perform (copied) */
+	gpointer ucontext,					/**< User context */
+	bgclean_cb_t ucontext_free,			/**< Free routine for context */
+	bgstart_cb_t start_cb,				/**< Starting working on an item */
+	bgend_cb_t end_cb,					/**< Done working on an item */
+	bgclean_cb_t item_free,				/**< Free routine for work queue items */
+	bgnotify_cb_t notify)				/**< Start/Stop notify (optional) */
 {
 	struct bgtask *bt;
 	gint stepsize;
@@ -423,7 +429,7 @@ gpointer bg_daemon_create(
 	return bt;
 }
 
-/*
+/**
  * bg_daemon_enqueue
  *
  * Enqueue work item to the daemon task.
@@ -449,7 +455,7 @@ void bg_daemon_enqueue(gpointer h, gpointer item)
 	}
 }
 
-/*
+/**
  * bg_task_free
  *
  * Free task structure.
@@ -480,7 +486,7 @@ static void bg_task_free(struct bgtask *bt)
 	wfree(bt, sizeof(*bt));
 }
 
-/*
+/**
  * bg_task_terminate
  *
  * Terminate the task, invoking the completion callback if defined.
@@ -572,7 +578,7 @@ static void bg_task_terminate(struct bgtask *bt)
 	dead_tasks = g_slist_prepend(dead_tasks, bt);
 }
 
-/*
+/**
  * bg_task_exit
  *
  * Called by user code to "exit" the task.
@@ -596,7 +602,7 @@ void bg_task_exit(gpointer h, gint code)
 	longjmp(bt->env, 1);		/* Will call bg_task_terminate() */
 }
 
-/*
+/**
  * bg_task_sendsig
  *
  * Deliver signal via the user's signal handler.
@@ -614,11 +620,12 @@ static void bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 	bt->signal = BG_SIG_ZERO;
 }
 
-/*
+/**
  * bg_task_kill
  *
  * Send a signal to the given task.
- * Returns -1 if the task could not be signalled.
+ *
+ * @returns -1 if the task could not be signalled.
  */
 static gint bg_task_kill(gpointer h, bgsig_t sig)
 {
@@ -676,11 +683,12 @@ static gint bg_task_kill(gpointer h, bgsig_t sig)
 	return 1;
 }
 
-/*
+/**
  * bg_task_signal
  *
  * Install user-level signal handler for a task signal.
- * Returns previously installed signal handler.
+ *
+ * @returns previously installed signal handler.
  */
 bgsig_cb_t bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
 {
@@ -696,7 +704,7 @@ bgsig_cb_t bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
 	return oldhandler;
 }
 
-/*
+/**
  * bg_task_deliver_signals
  *
  * Deliver all the signals queued so far for the task.
@@ -714,7 +722,7 @@ static void bg_task_deliver_signals(struct bgtask *bt)
 
 	while (bt->signals != NULL) {
 		GSList *lnk = bt->signals;
-		bgsig_t sig = (bgsig_t) lnk->data;
+		bgsig_t sig = (bgsig_t) GPOINTER_TO_UINT(lnk->data);
 
 		/*
 		 * If signal kills the thread (it calls bg_task_exit() from the
@@ -728,7 +736,7 @@ static void bg_task_deliver_signals(struct bgtask *bt)
 	}
 }
 
-/*
+/**
  * bg_task_cancel
  *
  * Cancel a given task.
@@ -780,7 +788,7 @@ void bg_task_cancel(gpointer h)
 	g_assert(bt->flags & TASK_F_EXITED);	/* Task is now terminated */
 }
 
-/*
+/**
  * bg_task_ticks_used
  *
  * This routine can be called by the task when a single step is not using
@@ -802,7 +810,7 @@ void bg_task_ticks_used(gpointer h, gint used)
 		bt->flags |= TASK_F_NOTICK;			/* Won't update tick info */
 }
 
-/*
+/**
  * bg_reclaim_dead
  *
  * Reclaim all dead tasks
@@ -818,7 +826,7 @@ static void bg_reclaim_dead(void)
 	dead_tasks = NULL;
 }
 
-/*
+/**
  * bg_task_ended
  *
  * Called when a task has ended its processing.
@@ -875,14 +883,14 @@ static void bg_task_ended(struct bgtask *bt)
 	}
 }
 
-/*
+/**
  * bg_sched_timer
  *
  * Main task scheduling timer, called once per second.
  */
 void bg_sched_timer(void)
 {
-	struct bgtask *bt;
+	struct bgtask * volatile bt;
 	volatile gint remain = MAX_LIFE;
 	gint target;
 	volatile gint ticks;
@@ -1032,7 +1040,7 @@ void bg_sched_timer(void)
 		bg_reclaim_dead();			/* Free dead tasks */
 }
 
-/*
+/**
  * bg_close
  *
  * Called at shutdown time.
