@@ -1,10 +1,8 @@
 /*
- * $Id: crc.h,v 1.1 2004/08/24 21:37:15 rmanfredi Exp $
+ * $Id: crc.h,v 1.3 2005/09/07 17:35:30 cbiere Exp $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * CRC computation.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,15 +23,25 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * CRC computation.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #ifndef _crc_h_
 #define _crc_h_
 
 #include <glib.h>
 
 void crc_init(void);
-guint32 crc32_update_crc(guint32 crc_accum, const gchar *data, gint len);
+guint32 crc32_update_crc(guint32 crc_accum, gconstpointer data, size_t len);
 
 #endif	/* _crc_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
 
