@@ -3,8 +3,9 @@ static char yysccsid[] = "@(#)yaccpar	1.8 (Berkeley) 01/20/91";
 #endif
 #define YYBYACC 1
 #line 2 "getdate.y"
+
 /*
- * $Id: getdate.c,v 1.2 2004/11/13 14:08:39 rmanfredi Exp $
+ * $Id: getdate.c,v 1.6 2005/08/18 20:40:38 cbiere Exp $
  *
  * Date parsing.
  *
@@ -648,6 +649,9 @@ static TABLE const MilitaryTable[] = {
 static int yyerror(char *unused_s)
 {
 	(void) unused_s;
+#ifndef lint
+	(void) yysccsid;
+#endif
     return 0;
 }
 
@@ -702,7 +706,7 @@ static int LookupWord(char *buff)
     /* Make it lowercase. */
     for (p = (unsigned char *) buff; *p; p++)
 	if (ISUPPER(*p))
-	    *p = tolower(*p);
+	    *p += 32;
 
     if (strcmp(buff, "am") == 0 || strcmp(buff, "a.m.") == 0) {
 	yylval.Meridian = MERam;
@@ -1047,7 +1051,7 @@ yyparse()
     *yyssp = yystate = 0;
 
 yyloop:
-    if (yyn = yydefred[yystate]) goto yyreduce;
+    if (0 != (yyn = yydefred[yystate])) goto yyreduce;
     if (yychar < 0)
     {
         if ((yychar = yylex()) < 0) yychar = 0;
@@ -1090,12 +1094,16 @@ yyloop:
 #ifdef lint
     goto yynewerror;
 #endif
+#if lint 
 yynewerror:
+#endif
     yyerror("syntax error");
 #ifdef lint
     goto yyerrlab;
 #endif
+#ifdef lint
 yyerrlab:
+#endif
     ++yynerrs;
 yyinrecovery:
     if (yyerrflag < 3)
