@@ -1,5 +1,5 @@
 /*
- * $Id: getline.c,v 1.1 2004/08/24 21:57:13 rmanfredi Exp $
+ * $Id: getline.c,v 1.3 2005/06/25 01:37:42 daichik Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -24,21 +24,25 @@
  */
 
 /**
+ * @ingroup lib
  * @file
  *
  * Line-oriented parsing from memory buffer.
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: getline.c,v 1.1 2004/08/24 21:57:13 rmanfredi Exp $");
+RCSID("$Id: getline.c,v 1.3 2005/06/25 01:37:42 daichik Exp $");
 
 #include <string.h>		/* For memmove() */
 
 #include "getline.h"
 #include "walloc.h"
 #include "misc.h"		/* For RCSID */
-#include "override.h"		/* Must be the last header included */
+#include "override.h"	/* Must be the last header included */
 
 /*
  * Those govern the allocation policy for the getline buffer.
@@ -173,7 +177,7 @@ getline_read(getline_t *o, gchar *data, gint len, gint *used)
 }
 
 /**
- * Returns a C string (NUL-terminated) corresponding to the line we currently
+ * @return a C string (NUL-terminated) corresponding to the line we currently
  * have in the buffer.
  */
 gchar *
@@ -186,7 +190,7 @@ getline_str(getline_t *o)
 }
 
 /**
- * Return the length of the currently accumulated line.
+ * @return the length of the currently accumulated line.
  */
 gint
 getline_length(getline_t *o)
