@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 2002, ko (ko-@wanadoo.fr)
+ * $Id: inputevt.h,v 1.17 2005/09/27 20:11:30 rmanfredi Exp $
  *
- * Input I/O notification.
+ * Copyright (c) 2002, ko (ko-@wanadoo.fr)
+ * Copyright (c) 2005, Christian Biere
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -23,21 +24,39 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Input I/O notification.
+ *
+ * @author ko (ko-@wanadoo.fr)
+ * @date 2002
+ * @author Christian Biere
+ * @date 2005
+ */
+
 #ifndef _inputevt_h_
 #define _inputevt_h_
 
 #include <glib.h>
 
-/*
+/**
  * This mimics the GDK input condition type.
  */
 typedef enum {
-	INPUT_EVENT_READ		= 1 << 0,
-	INPUT_EVENT_WRITE		= 1 << 1,
-	INPUT_EVENT_EXCEPTION	= 1 << 2,
+	INPUT_EVENT_R			= 1 << 0,	/* poll for Read events */
+	INPUT_EVENT_W			= 1 << 1,	/* poll for Write events */
+	INPUT_EVENT_EXCEPTION	= 1 << 2,	/* poll for exceptions */
+
+	INPUT_EVENT_RX = ((guint) INPUT_EVENT_R | (guint) INPUT_EVENT_EXCEPTION),
+	INPUT_EVENT_WX = ((guint) INPUT_EVENT_W | (guint) INPUT_EVENT_EXCEPTION),
+	
+	INPUT_EVENT_RW = ((guint) INPUT_EVENT_R | (guint) INPUT_EVENT_W),
+	INPUT_EVENT_RWX = ((guint) INPUT_EVENT_RW | (guint) INPUT_EVENT_EXCEPTION),
 } inputevt_cond_t;
 
-/*
+/**
  * And the handler function type.
  */
 typedef void (*inputevt_handler_t) (
@@ -47,19 +66,27 @@ typedef void (*inputevt_handler_t) (
 );
 
 /*
- * Module initialization and cleanup functions. 
+ * Module initialization and cleanup functions.
  * These don't do anything and are not called (yet).
  */
 void inputevt_init(void);
 void inputevt_close(void);
 
-/*
+/**
  * This emulates the GDK input interface.
  */
 guint inputevt_add(gint source, inputevt_cond_t condition,
-	inputevt_handler_t handler, gpointer data) ;
+	inputevt_handler_t handler, gpointer data);
 
+const gchar *inputevt_cond_to_string(inputevt_cond_t cond);
+
+#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
+void inputevt_remove(guint id);
+#else /* !(HAS_EPOLL || HAS_KQUEUE) */
 #define inputevt_remove(source) (g_source_remove(source))
+#define inputevt_timer()
+#endif /* HAS_EPOLL || HAS_KQUEUE */
 
 #endif  /* _inputevt_h_ */
 
+/* vi: set ts=4 sw=4 cindent: */
