@@ -1,5 +1,5 @@
 /*
- * $Id: iprange.c,v 1.3 2004/10/20 15:38:58 cbiere Exp $
+ * $Id: iprange.c,v 1.9 2005/06/29 14:24:26 daichik Exp $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -24,15 +24,21 @@
  */
 
 /**
+ * @ingroup lib
  * @file
  *
+ * Needs brief description here.
+ *
  * Load, parse and lookup IP addresses from a set of IP ranges defined
  * by a list of addresses in CIDR (Classless Internet Domain Routing) format.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: iprange.c,v 1.3 2004/10/20 15:38:58 cbiere Exp $");
+RCSID("$Id: iprange.c,v 1.9 2005/06/29 14:24:26 daichik Exp $");
 
 #include "iprange.h"
 #include "misc.h"
@@ -58,7 +64,7 @@ RCSID("$Id: iprange.c,v 1.3 2004/10/20 15:38:58 cbiere Exp $");
  * the time the range was added to the database.
  */
 
-#define IPRANGE_MARK	1		/* Magic tagging of user values */
+#define IPRANGE_MARK	1		/**< Magic tagging of user values */
 
 /*
  * A "database" descriptor, holding the CIDR networks and their attached value.
@@ -68,43 +74,43 @@ RCSID("$Id: iprange.c,v 1.3 2004/10/20 15:38:58 cbiere Exp $");
 #define IPAGESZ			256
 
 struct iprange_db {
-	guint32 magic;			/* Magic number */
-	gpointer lvl1[IPAGESZ];	/* Level-1 index */
+	guint32 magic;			/**< Magic number */
+	gpointer lvl1[IPAGESZ];	/**< Level-1 index */
 	/* Allocation */
-	iprange_clone_t clone;	/* Cloning callback for values */
-	iprange_free_t free;	/* Freeing callback for values */
+	iprange_clone_t clone;	/**< Cloning callback for values */
+	iprange_free_t free;	/**< Freeing callback for values */
 	/* Statistics */
-	gint range_count;		/* Amount of network ranges registered */
-	gint lvl2_count;		/* Amount of level-2 pages, for stats */
-	gint head_count;		/* Amount of heads in level-2 pages, for stats */
-	gint list_count;		/* Amount of items held in lists */
+	gint range_count;		/**< Amount of network ranges registered */
+	gint lvl2_count;		/**< Amount of level-2 pages, for stats */
+	gint head_count;		/**< Amount of heads in level-2 pages, for stats */
+	gint list_count;		/**< Amount of items held in lists */
 };
 
-/*
+/**
  * A CIDR network description.  Note that the network IP is already masked,
  * so to determine whether some IP falls within that network, one needs
  * to compare (IP & netmask) with the ip_masked field.
  */
 struct iprange_net {
-	guint32 ip_masked;	/* The IP of the network (meaningful bits) */
-	guint32 netmask;	/* The network bit mask, selecting meaningful bits */
-	gpointer value;		/* Value held */
-};
-
-/*
- * Error code stings.
- */
-static const gchar *iprange_errstr[] = {
-	"OK",									/* IPR_ERR_OK */
-	"Incorrect network prefix",				/* IPR_ERR_BAD_PREFIX */
-	"CIDR range clash",						/* IPR_ERR_RANGE_CLASH */
-	"Duplicate range",						/* IPR_ERR_RANGE_DUP */
-	"Range is subnet of existing range",	/* IPR_ERR_RANGE_SUBNET */
-	"Range is overlapping existing range",	/* IPR_ERR_RANGE_OVERLAP */
+	guint32 ip_masked;	/**< The IP of the network (meaningful bits) */
+	guint32 netmask;	/**< The network bit mask, selecting meaningful bits */
+	gpointer value;		/**< Value held */
 };
 
 /**
- * Returns human-readable error string for given error code.
+ * Error code stings.
+ */
+static const gchar *iprange_errstr[] = {
+	"OK",									/**< IPR_ERR_OK */
+	"Incorrect network prefix",				/**< IPR_ERR_BAD_PREFIX */
+	"CIDR range clash",						/**< IPR_ERR_RANGE_CLASH */
+	"Duplicate range",						/**< IPR_ERR_RANGE_DUP */
+	"Range is subnet of existing range",	/**< IPR_ERR_RANGE_SUBNET */
+	"Range is overlapping existing range",	/**< IPR_ERR_RANGE_OVERLAP */
+};
+
+/**
+ * @return human-readable error string for given error code.
  */
 const gchar *
 iprange_strerror(iprange_err_t errnum)
@@ -167,6 +173,7 @@ iprange_free_each(gpointer db, gpointer udata)
 	struct iprange_db *idb = (struct iprange_db *) db;
 	gint i;
 
+	g_assert(db);
 	g_assert(idb->magic == IPRANGE_MAGIC);
 
 	/*
@@ -221,7 +228,7 @@ iprange_free_each(gpointer db, gpointer udata)
 static iprange_err_t
 iprange_add_level1(
 	struct iprange_db *idb,
-	guint32 net, gint8 bits, gpointer udata, gboolean force)
+	guint32 net, guint bits, gpointer udata, gboolean force)
 {
 	guint8 first;
 	guint8 mask;
@@ -274,7 +281,7 @@ iprange_add_level1(
 static iprange_err_t
 iprange_add_level2(
 	struct iprange_db *idb,
-	guint32 net, gint8 bits, gpointer *lvl2, gpointer udata, gboolean force)
+	guint32 net, guint bits, gpointer *lvl2, gpointer udata, gboolean force)
 {
 	guint8 second;
 	guint8 mask;
@@ -336,7 +343,7 @@ iprange_add_level2(
  */
 static iprange_err_t
 iprange_add_cidr_internal(
-	gpointer db, guint32 net, gint8 bits, gpointer udata,
+	gpointer db, guint32 net, guint bits, gpointer udata,
 	gboolean force, gpointer cdata)
 {
 	struct iprange_db *idb = (struct iprange_db *) db;
@@ -351,6 +358,7 @@ iprange_add_cidr_internal(
 	GSList *to_remove = NULL;
 	struct iprange_net *ipn;
 
+	g_assert(db);
 	g_assert(idb->magic == IPRANGE_MAGIC);
 	g_assert(bits >= 1 && bits <= 32);
 	g_assert(0 == (GPOINTER_TO_UINT(udata) & IPRANGE_MARK));
@@ -507,7 +515,7 @@ iprange_add_cidr_internal(
 	lvl2[second] = g_slist_prepend(head, ipn);
 	idb->list_count++;
 	idb->range_count++;
-	
+
 	return IPR_ERR_OK;			/* Successfully inserted */
 }
 
@@ -522,7 +530,7 @@ iprange_add_cidr_internal(
  * @return IPR_ERR_OK if successful, an error code otherwise.
  */
 iprange_err_t
-iprange_add_cidr(gpointer db, guint32 net, gint8 bits, gpointer udata)
+iprange_add_cidr(gpointer db, guint32 net, guint bits, gpointer udata)
 {
 	return iprange_add_cidr_internal(db, net, bits, udata, FALSE, NULL);
 }
@@ -541,7 +549,7 @@ iprange_add_cidr(gpointer db, guint32 net, gint8 bits, gpointer udata)
  */
 iprange_err_t
 iprange_add_cidr_force(
-	gpointer db, guint32 net, gint8 bits, gpointer udata, gpointer cdata)
+	gpointer db, guint32 net, guint bits, gpointer udata, gpointer cdata)
 {
 	return iprange_add_cidr_internal(db, net, bits, udata, TRUE, cdata);
 }
@@ -560,8 +568,9 @@ iprange_get(gpointer db, guint32 ip)
 	GSList *head;
 	GSList *l;
 
+	g_assert(db);
 	g_assert(idb->magic == IPRANGE_MAGIC);
-	
+
 	first = ip >> 24;
 	lvl2 = idb->lvl1[first];
 
@@ -610,6 +619,7 @@ iprange_get_stats(gpointer db, iprange_stats_t *stats)
 {
 	struct iprange_db *idb = (struct iprange_db *) db;
 
+	g_assert(db);
 	g_assert(idb->magic == IPRANGE_MAGIC);
 
 	stats->count = idb->range_count;
