@@ -1,10 +1,8 @@
 /*
- * $Id: malloc.c,v 1.8 2004/11/13 14:09:03 rmanfredi Exp $
+ * $Id: malloc.c,v 1.15 2005/11/15 14:37:25 cbiere Exp $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
- * Debugging malloc, to supplant dmalloc which is not satisfactory.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,16 +23,28 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Debugging malloc, to supplant dmalloc which is not satisfactory.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
+ */
 
 #include "common.h"		/* For RCSID */
 
 #include "atoms.h"		/* For binary_hash() */
-/*
+#include "misc.h"		/* For concat_strings() */
+#include "tm.h"			/* For tm_time() */
+
+/**
  * Routines in this file are defined either for TRACK_MALLOC or TRACK_ZALLOC
  */
 
 #if defined(TRACK_MALLOC) || defined(TRACK_ZALLOC)
-RCSID("$Id: malloc.c,v 1.8 2004/11/13 14:09:03 rmanfredi Exp $");
+RCSID("$Id: malloc.c,v 1.15 2005/11/15 14:37:25 cbiere Exp $");
 
 /*
  * When MALLOC_FRAMES is supplied, we keep information about the allocation
@@ -54,7 +64,7 @@ RCSID("$Id: malloc.c,v 1.8 2004/11/13 14:09:03 rmanfredi Exp $");
 #define MALLOC_STATS
 #endif
 
-#define FRAME_DEPTH		8	/* Size of allocation frame we keep around */
+#define FRAME_DEPTH		8	/**< Size of allocation frame we keep around */
 
 #endif /* MALLOC_FRAMES */
 #endif /* TRACK_MALLOC || TRACK_ZALLOC */
@@ -64,17 +74,17 @@ RCSID("$Id: malloc.c,v 1.8 2004/11/13 14:09:03 rmanfredi Exp $");
 #include "hashlist.h"
 #include "misc.h"
 #include "glib-missing.h"
-#define MALLOC_SOURCE	/* Avoid nasty remappings, but include signatures */
+#define MALLOC_SOURCE	/**< Avoid nasty remappings, but include signatures */
 #include "override.h"
 
 #if 0
-#define TRANSPARENT	/* To make sure our macros have no side effect */
+#define TRANSPARENT		/**< To make sure our macros have no side effect */
 #endif
 
 static time_t init_time = 0;
 static time_t reset_time = 0;
 
-/*
+/**
  * Structure keeping track of allocated blocks.
  *
  * Each block is inserted into a hash table, the key being the block's
@@ -94,17 +104,17 @@ static void free_record(gpointer o, gchar *file, gint line);
 
 #ifdef MALLOC_FRAMES
 
-/*
+/**
  * Structure keeping track of the allocation/free stack frames.
  *
  * Counts are signed because for realloc() frames, we count algebric
  * quantities (in case the blocks are shrunk).
  */
 struct frame {
-	void *stack[FRAME_DEPTH];	/* PC of callers */
-	gint len;					/* Number of valid entries in stack */
-	gint32 count;				/* Bytes allocated/freed since reset */
-	gint32 total_count;			/* Grand total for this stack frame */
+	void *stack[FRAME_DEPTH];	/**< PC of callers */
+	gint len;					/**< Number of valid entries in stack */
+	gint32 count;				/**< Bytes allocated/freed since reset */
+	gint32 total_count;			/**< Grand total for this stack frame */
 };
 
 /**
@@ -114,7 +124,7 @@ static guint
 frame_hash(gconstpointer key)
 {
 	struct frame *f = (struct frame *) key;
-	
+
 	return binary_hash((gconstpointer) f->stack, f->len * sizeof(void *));
 }
 
@@ -133,7 +143,9 @@ frame_eq(gconstpointer a, gconstpointer b)
 
 #endif /* MALLOC_FRAMES */
 
-/*
+/**
+ * @struct stats
+ *
  * When MALLOC_STATS is supplied, we keep information about the amount
  * of bytes allocated from a single point in the code, and the amount
  * of it that has been freed.
@@ -145,24 +157,24 @@ frame_eq(gconstpointer a, gconstpointer b)
 #ifdef MALLOC_STATS
 
 struct stats {
-	gchar *file;				/* Place where allocation took place */
-	gint line;					/* Line number */
-	gint blocks;				/* Live blocks since last "reset" */
-	gint total_blocks;			/* Total live blocks */
-	guint32 allocated;			/* Total allocated since last "reset" */
-	guint32 freed;				/* Total freed since last "reset" */
-	guint32 total_allocated;	/* Total allocated overall */
-	guint32 total_freed;		/* Total freed overall */
-	gint32 reallocated;			/* Total reallocated since last "reset" */
-	gint32 total_reallocated;	/* Total reallocated overall (algebric!) */
+	gchar *file;				/**< Place where allocation took place */
+	gint line;					/**< Line number */
+	gint blocks;				/**< Live blocks since last "reset" */
+	gint total_blocks;			/**< Total live blocks */
+	guint32 allocated;			/**< Total allocated since last "reset" */
+	guint32 freed;				/**< Total freed since last "reset" */
+	guint32 total_allocated;	/**< Total allocated overall */
+	guint32 total_freed;		/**< Total freed overall */
+	gint32 reallocated;			/**< Total reallocated since last "reset" */
+	gint32 total_reallocated;	/**< Total reallocated overall (algebric!) */
 #ifdef MALLOC_FRAMES
-	GHashTable *alloc_frames;	/* The frames where allocation took place */
-	GHashTable *free_frames;	/* The frames where free took place */
-	GHashTable *realloc_frames;	/* The frames where realloc took place */
+	GHashTable *alloc_frames;	/**< The frames where allocation took place */
+	GHashTable *free_frames;	/**< The frames where free took place */
+	GHashTable *realloc_frames;	/**< The frames where realloc took place */
 #endif /* MALLOC_FRAMES */
 };
 
-static GHashTable *stats = NULL; /* maps stats(file, line) -> stats */
+static GHashTable *stats = NULL; /**< maps stats(file, line) -> stats */
 
 /**
  * Hashing routine for "struct stats".
@@ -190,12 +202,13 @@ stats_eq(gconstpointer a, gconstpointer b)
 }
 #endif /* MALLOC_STATS */
 
-/*
+/**
  * malloc_init
  *
  * Called at first allocation to initialize tracking structures.
  */
-static void malloc_init(void)
+static void
+malloc_init(void)
 {
 	blocks = g_hash_table_new(g_direct_hash, g_direct_equal);
 
@@ -203,15 +216,16 @@ static void malloc_init(void)
 	stats = g_hash_table_new(stats_hash, stats_eq);
 #endif
 
-	init_time = reset_time = time(NULL);
+	init_time = reset_time = tm_time();
 }
 
-/*
+/**
  * malloc_log_block		-- hash table iterator callback
  *
  * Log used block, and record it among the `leaksort' set for future summary.
  */
-static void malloc_log_block(gpointer k, gpointer v, gpointer leaksort)
+static void
+malloc_log_block(gpointer k, gpointer v, gpointer leaksort)
 {
 	struct block *b = (struct block *) v;
 
@@ -229,12 +243,11 @@ static void malloc_log_block(gpointer k, gpointer v, gpointer leaksort)
 	}
 }
 
-/*
- * malloc_close
- *
+/**
  * Dump all the blocks that are still used.
  */
-void malloc_close(void)
+void
+malloc_close(void)
 {
 	gpointer leaksort;
 
@@ -254,13 +267,12 @@ void malloc_close(void)
 	leak_close(leaksort);
 }
 
-/*
- * malloc_record
- *
+/**
  * Record object `o' allocated at `file' and `line' of size `s'.
- * Returns argument `o'.
+ * @return argument `o'.
  */
-gpointer malloc_record(gpointer o, guint32 sz, gchar *file, gint line)
+gpointer
+malloc_record(gpointer o, guint32 sz, gchar *file, gint line)
 {
 	struct block *b;
 	struct block *ob;
@@ -287,7 +299,7 @@ gpointer malloc_record(gpointer o, guint32 sz, gchar *file, gint line)
 	b->size = sz;
 	b->realloc = NULL;
 
-	/*
+	/**
 	 * It can happen that we track the allocation of a block somewhere
 	 * but the freeing happens somewhere we either we forgot to include
 	 * "override.h", or happens in some library (e.g. in GTK+) where we
@@ -355,12 +367,11 @@ gpointer malloc_record(gpointer o, guint32 sz, gchar *file, gint line)
 	return o;
 }
 
-/*
- * malloc_track
- *
+/**
  * Allocate `s' bytes.
  */
-gpointer malloc_track(guint32 s, gchar *file, gint line)
+gpointer
+malloc_track(guint32 s, gchar *file, gint line)
 {
 	gpointer o;
 
@@ -371,27 +382,25 @@ gpointer malloc_track(guint32 s, gchar *file, gint line)
 	return malloc_record(o, s, file, line);
 }
 
-/*
- * malloc0_track
- *
+/**
  * Allocate `s' bytes, zero the allocated zone.
  */
-gpointer malloc0_track(guint32 s, gchar *file, gint line)
+gpointer
+malloc0_track(guint32 s, gchar *file, gint line)
 {
 	gpointer o;
 
 	o = malloc_track(s, file, line);
 	memset(o, 0, s);
-	
+
 	return o;
 }
 
-/*
- * free_record
- *
+/**
  * Record freeing of allocated block.
  */
-static void free_record(gpointer o, gchar *file, gint line)
+static void
+free_record(gpointer o, gchar *file, gint line)
 {
 	struct block *b;
 	gpointer k;
@@ -474,12 +483,11 @@ static void free_record(gpointer o, gchar *file, gint line)
 	free(b);
 }
 
-/*
- * free_track
- *
+/**
  * Free allocated block.
  */
-void free_track(gpointer o, gchar *file, gint line)
+void
+free_track(gpointer o, gchar *file, gint line)
 {
 #ifndef TRANSPARENT
 	free_record(o, file, line);
@@ -487,12 +495,11 @@ void free_track(gpointer o, gchar *file, gint line)
 	free(o);
 }
 
-/*
- * strfreev_track
- *
+/**
  * Free NULL-terminated vector of strings, and the vector.
  */
-void strfreev_track(gchar **v, gchar *file, gint line)
+void
+strfreev_track(gchar **v, gchar *file, gint line)
 {
 	gchar *x;
 	gchar **iv = v;
@@ -503,9 +510,7 @@ void strfreev_track(gchar **v, gchar *file, gint line)
 	free_track(v, file, line);
 }
 
-/*
- * realloc_record
- *
+/**
  * Update data structures to record that block `o' was re-alloced into
  * a block of `s' bytes at `n'.
  */
@@ -590,7 +595,8 @@ realloc_record(gpointer o, gpointer n, guint32 s, gchar *file, gint line)
 /**
  * Realloc object `o' to `s' bytes.
  */
-gpointer realloc_track(gpointer o, guint32 s, gchar *file, gint line)
+gpointer
+realloc_track(gpointer o, guint32 s, gchar *file, gint line)
 {
 	gpointer n;
 
@@ -609,12 +615,11 @@ gpointer realloc_track(gpointer o, guint32 s, gchar *file, gint line)
 	return realloc_record(o, n, s, file, line);
 }
 
-/*
- * memdup_track
- *
+/**
  * Duplicate buffer `p' of length `size'.
  */
-gpointer memdup_track(gconstpointer p, guint size, gchar *file, gint line)
+gpointer
+memdup_track(gconstpointer p, guint size, gchar *file, gint line)
 {
 	gpointer o;
 
@@ -627,12 +632,11 @@ gpointer memdup_track(gconstpointer p, guint size, gchar *file, gint line)
 	return o;
 }
 
-/*
- * strdup_track
- *
+/**
  * Duplicate string `s'.
  */
-gchar *strdup_track(const gchar *s, gchar *file, gint line)
+gchar *
+strdup_track(const gchar *s, gchar *file, gint line)
 {
 	gpointer o;
 	guint32 len;
@@ -647,12 +651,11 @@ gchar *strdup_track(const gchar *s, gchar *file, gint line)
 	return o;
 }
 
-/*
- * strndup_track
- *
+/**
  * Duplicate string `s', on at most `n' chars.
  */
-gchar *strndup_track(const gchar *s, gint n, gchar *file, gint line)
+gchar *
+strndup_track(const gchar *s, gint n, gchar *file, gint line)
 {
 	gpointer o;
 	gchar *p;
@@ -672,26 +675,24 @@ gchar *strndup_track(const gchar *s, gint n, gchar *file, gint line)
 	return o;
 }
 
-/*
- * strjoinv_track
- *
+/**
  * Join items in `vec' with `s' in-between.
  */
-gchar *strjoinv_track(const gchar *s, gchar **vec, gchar *file, gint line)
+gchar *
+strjoinv_track(const gchar *s, gchar **vec, gchar *file, gint line)
 {
 	gchar *o;
-	
+
 	o = g_strjoinv(s, vec);
 
 	return malloc_record(o, strlen(o) + 1, file, line);
 }
 
-/*
- * m_strconcatv
- *
+/**
  * The internal implementation of a vectorized g_strconcat().
  */
-static gchar *m_strconcatv(const gchar *s, va_list args)
+static gchar *
+m_strconcatv(const gchar *s, va_list args)
 {
 	gchar *res;
 	gchar *add;
@@ -711,12 +712,11 @@ static gchar *m_strconcatv(const gchar *s, va_list args)
 	return res;
 }
 
-/*
- * strconcat_track
- *
+/**
  * Perform string concatenation, returning newly allocated string.
  */
-gchar *strconcat_track(gchar *file, gint line, const gchar *s, ...)
+gchar *
+strconcat_track(gchar *file, gint line, const gchar *s, ...)
 {
 	va_list args;
 	gchar *o;
@@ -728,12 +728,11 @@ gchar *strconcat_track(gchar *file, gint line, const gchar *s, ...)
 	return malloc_record(o, strlen(o) + 1, file, line);
 }
 
-/*
- * strdup_printf_track
- *
+/**
  * Perform printf into newly allocated string.
  */
-gchar *strdup_printf_track(gchar *file, gint line, const gchar *fmt, ...)
+gchar *
+strdup_printf_track(gchar *file, gint line, const gchar *fmt, ...)
 {
 	va_list args;
 	gchar *o;
@@ -745,13 +744,11 @@ gchar *strdup_printf_track(gchar *file, gint line, const gchar *fmt, ...)
 	return malloc_record(o, strlen(o) + 1, file, line);
 }
 
-/*
- * strsplit_track
- *
+/**
  * Perform a g_strplit() operation, tracking all returned strings.
  */
-gchar **strsplit_track(
-	const gchar *s, const gchar *d, gint m, gchar *file, gint line)
+gchar **
+strsplit_track(const gchar *s, const gchar *d, gint m, gchar *file, gint line)
 {
 	gchar **v;
 	gchar **iv;
@@ -767,13 +764,12 @@ gchar **strsplit_track(
 	return v;
 }
 
-/*
- * string_record
- *
+/**
  * Record string `s' allocated at `file' and `line'.
- * Returns argument `s'.
+ * @return argument `s'.
  */
-gpointer string_record(const gchar *s, gchar *file, gint line)
+gpointer
+string_record(const gchar *s, gchar *file, gint line)
 {
 	if (s == NULL)
 		return NULL;
@@ -781,13 +777,11 @@ gpointer string_record(const gchar *s, gchar *file, gint line)
 	return malloc_record((gpointer) s, strlen(s) + 1, file, line);
 }
 
-/*
- * hashtable_new_track
- *
+/**
  * Wrapper over g_hash_table_new() to track allocation of hash tables.
  */
-GHashTable *hashtable_new_track(
-	GHashFunc h, GCompareFunc y, gchar *file, gint line)
+GHashTable *
+hashtable_new_track(GHashFunc h, GCompareFunc y, gchar *file, gint line)
 {
 	GHashTable *o;
 
@@ -795,33 +789,30 @@ GHashTable *hashtable_new_track(
 	return malloc_record(o, 24, file, line);	/* Size not right, don't care */
 }
 
-/*
- * hashtable_destroy_track
- *
+/**
  * Wrapper over g_hash_Table_destroy() to track destruction of hash tables.
  */
-void hashtable_destroy_track(GHashTable *h, gchar *file, gint line)
+void
+hashtable_destroy_track(GHashTable *h, gchar *file, gint line)
 {
 	free_record(h, file, line);
 	g_hash_table_destroy(h);
 }
 
-/*
- * hash_list_new_track
- *
+/**
  * Wrapper over hash_list_new().
  */
-hash_list_t *hash_list_new_track(gchar *file, gint line)
+hash_list_t *
+hash_list_new_track(gchar *file, gint line)
 {
 	return malloc_record(hash_list_new(), 28, file, line);	/* Approx. size */
 }
 
-/*
- * hash_list_free_track
- *
+/**
  * Wrapper over hash_list_free().
  */
-void hash_list_free_track(hash_list_t *h, gchar *file, gint line)
+void
+hash_list_free_track(hash_list_t *h, gchar *file, gint line)
 {
 	free_record(h, file, line);
 	hash_list_free(h);
@@ -834,12 +825,14 @@ void hash_list_free_track(hash_list_t *h, gchar *file, gint line)
 #define GSLIST_LINK_SIZE	8		/* Random size */
 #define GLIST_LINK_SIZE		12		/* Random size */
 
-GSList *slist_alloc_track(gchar *file, gint line)
+GSList *
+slist_alloc_track(gchar *file, gint line)
 {
 	return malloc_record(g_slist_alloc(), GSLIST_LINK_SIZE, file, line);
 }
 
-GSList *slist_append_track(GSList *l, gpointer data, gchar *file, gint line)
+GSList *
+slist_append_track(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *new;
 
@@ -854,7 +847,8 @@ GSList *slist_append_track(GSList *l, gpointer data, gchar *file, gint line)
 		return new;
 }
 
-GSList *slist_prepend_track(GSList *l, gpointer data, gchar *file, gint line)
+GSList *
+slist_prepend_track(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *new;
 
@@ -865,7 +859,8 @@ GSList *slist_prepend_track(GSList *l, gpointer data, gchar *file, gint line)
 	return new;
 }
 
-GSList *slist_copy_track(GSList *list, gchar *file, gint line)
+GSList *
+slist_copy_track(GSList *list, gchar *file, gint line)
 {
 	GSList *new;
 	GSList *l;
@@ -878,7 +873,8 @@ GSList *slist_copy_track(GSList *list, gchar *file, gint line)
 	return new;
 }
 
-void slist_free_track(GSList *l, gchar *file, gint line)
+void
+slist_free_track(GSList *l, gchar *file, gint line)
 {
 	GSList *lk;
 
@@ -888,7 +884,8 @@ void slist_free_track(GSList *l, gchar *file, gint line)
 	g_slist_free(l);
 }
 
-void slist_free1_track(GSList *l, gchar *file, gint line)
+void
+slist_free1_track(GSList *l, gchar *file, gint line)
 {
 	if (l == NULL)
 		return;
@@ -897,7 +894,8 @@ void slist_free1_track(GSList *l, gchar *file, gint line)
 	g_slist_free_1(l);
 }
 
-GSList *slist_remove_track(GSList *l, gpointer data, gchar *file, gint line)
+GSList *
+slist_remove_track(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *lk;
 
@@ -909,8 +907,8 @@ GSList *slist_remove_track(GSList *l, gpointer data, gchar *file, gint line)
 	return g_slist_remove(l, data);
 }
 
-GSList *slist_insert_track(
-	GSList *l, gpointer data, gint pos, gchar *file, gint line)
+GSList *
+slist_insert_track(GSList *l, gpointer data, gint pos, gchar *file, gint line)
 {
 	GSList *lk;
 
@@ -926,8 +924,9 @@ GSList *slist_insert_track(
 		return slist_insert_after_track(l, lk, data, file, line);
 }
 
-GSList *slist_insert_sorted_track(
-	GSList *l, gpointer d, GCompareFunc c, gchar *file, gint line)
+GSList *
+slist_insert_sorted_track(GSList *l, gpointer d, GCompareFunc c,
+	gchar *file, gint line)
 {
 	gint cmp;
 	GSList *tmp = l;
@@ -962,8 +961,9 @@ GSList *slist_insert_sorted_track(
 	return new;
 }
 
-GSList *slist_insert_after_track(
-	GSList *l, GSList *lk, gpointer data, gchar *file, gint line)
+GSList *
+slist_insert_after_track(GSList *l, GSList *lk, gpointer data,
+	gchar *file, gint line)
 {
 	GSList *new;
 
@@ -979,12 +979,14 @@ GSList *slist_insert_after_track(
 	return l;
 }
 
-GList *list_alloc_track(gchar *file, gint line)
+GList *
+list_alloc_track(gchar *file, gint line)
 {
 	return malloc_record(g_list_alloc(), GLIST_LINK_SIZE, file, line);
 }
 
-GList *list_append_track(GList *l, gpointer data, gchar *file, gint line)
+GList *
+list_append_track(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *new;
 
@@ -1000,7 +1002,8 @@ GList *list_append_track(GList *l, gpointer data, gchar *file, gint line)
 		return new;
 }
 
-GList *list_prepend_track(GList *l, gpointer data, gchar *file, gint line)
+GList *
+list_prepend_track(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *new;
 
@@ -1019,7 +1022,8 @@ GList *list_prepend_track(GList *l, gpointer data, gchar *file, gint line)
 	return new;
 }
 
-GList *list_copy_track(GList *list, gchar *file, gint line)
+GList *
+list_copy_track(GList *list, gchar *file, gint line)
 {
 	GList *new;
 	GList *l;
@@ -1032,7 +1036,8 @@ GList *list_copy_track(GList *list, gchar *file, gint line)
 	return new;
 }
 
-void list_free_track(GList *l, gchar *file, gint line)
+void
+list_free_track(GList *l, gchar *file, gint line)
 {
 	GList *lk;
 
@@ -1042,7 +1047,8 @@ void list_free_track(GList *l, gchar *file, gint line)
 	g_list_free(l);
 }
 
-void list_free1_track(GList *l, gchar *file, gint line)
+void
+list_free1_track(GList *l, gchar *file, gint line)
 {
 	if (l == NULL)
 		return;
@@ -1051,7 +1057,8 @@ void list_free1_track(GList *l, gchar *file, gint line)
 	g_list_free_1(l);
 }
 
-GList *list_remove_track(GList *l, gpointer data, gchar *file, gint line)
+GList *
+list_remove_track(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *lk;
 
@@ -1062,8 +1069,8 @@ GList *list_remove_track(GList *l, gpointer data, gchar *file, gint line)
 	return list_delete_link_track(l, lk, file, line);
 }
 
-GList *list_insert_track(
-	GList *l, gpointer data, gint pos, gchar *file, gint line)
+GList *
+list_insert_track(GList *l, gpointer data, gint pos, gchar *file, gint line)
 {
 	GList *lk;
 
@@ -1079,8 +1086,9 @@ GList *list_insert_track(
 		return list_insert_after_track(l, lk, data, file, line);
 }
 
-GList *list_insert_sorted_track(
-	GList *l, gpointer d, GCompareFunc c, gchar *file, gint line)
+GList *
+list_insert_sorted_track(GList *l, gpointer d, GCompareFunc c,
+	gchar *file, gint line)
 {
 	gint cmp;
 	GList *tmp = l;
@@ -1117,8 +1125,9 @@ GList *list_insert_sorted_track(
 	return (tmp == l) ? new : l;
 }
 
-GList *list_insert_after_track(
-	GList *l, GList *lk, gpointer data, gchar *file, gint line)
+GList *
+list_insert_after_track(GList *l, GList *lk, gpointer data,
+	gchar *file, gint line)
 {
 	GList *new;
 
@@ -1127,7 +1136,7 @@ GList *list_insert_after_track(
 
 	new = list_alloc_track(file, line);
 	new->data = data;
-	
+
 	new->prev = lk;
 	new->next = lk->next;
 
@@ -1139,7 +1148,8 @@ GList *list_insert_after_track(
 	return l;
 }
 
-GList *list_delete_link_track(GList *l, GList *lk, gchar *file, gint line)
+GList *
+list_delete_link_track(GList *l, GList *lk, gchar *file, gint line)
 {
 	GList *new;
 
@@ -1155,13 +1165,14 @@ GList *list_delete_link_track(GList *l, GList *lk, gchar *file, gint line)
 
 #define GSTRING_OBJ_SIZE	8		/* Random size */
 
-/*
+/**
  * string_str_track
  *
  * Track changes to the internal string object.
- * Returns GString object.
+ * @return GString object.
  */
-static GString *string_str_track(GString *s, gchar *old, gchar *file, gint line)
+static GString *
+string_str_track(GString *s, gchar *old, gchar *file, gint line)
 {
 	if (s->str != old) {
 		free_record(old, file, line);
@@ -1172,7 +1183,8 @@ static GString *string_str_track(GString *s, gchar *old, gchar *file, gint line)
 	return s;
 }
 
-GString *string_new_track(const gchar *p, gchar *file, gint line)
+GString *
+string_new_track(const gchar *p, gchar *file, gint line)
 {
 	GString *result = g_string_new(p);
 
@@ -1182,7 +1194,8 @@ GString *string_new_track(const gchar *p, gchar *file, gint line)
 	return result;
 }
 
-GString *string_sized_new_track(guint size, gchar *file, gint line)
+GString *
+string_sized_new_track(guint size, gchar *file, gint line)
 {
 	GString *result = g_string_sized_new(size);
 
@@ -1192,8 +1205,8 @@ GString *string_sized_new_track(guint size, gchar *file, gint line)
 	return result;
 }
 
-GString *string_append_track(
-	GString *s, const gchar *p, gchar *file, gint line)
+GString *
+string_append_track(GString *s, const gchar *p, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1201,8 +1214,8 @@ GString *string_append_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_append_c_track(
-	GString *s, gchar c, gchar *file, gint line)
+GString *
+string_append_c_track(GString *s, gchar c, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1210,8 +1223,18 @@ GString *string_append_c_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_assign_track(
-	GString *s, const gchar *p, gchar *file, gint line)
+GString *
+string_append_len_track(GString *s, const gchar *val, gssize len,
+	gchar *file, gint line)
+{
+	gchar *old = s->str;
+
+	s = g_string_append_len(s, val, len);
+	return string_str_track(s, old, file, line);
+}
+
+GString *
+string_assign_track(GString *s, const gchar *p, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1219,7 +1242,8 @@ GString *string_assign_track(
 	return string_str_track(s, old, file, line);
 }
 
-void string_free_track(GString *s, gint freestr, gchar *file, gint line)
+void
+string_free_track(GString *s, gint freestr, gchar *file, gint line)
 {
 	free_record(s, file, line);
 	if (freestr)
@@ -1228,8 +1252,8 @@ void string_free_track(GString *s, gint freestr, gchar *file, gint line)
 	g_string_free(s, freestr);
 }
 
-GString *string_prepend_track(
-	GString *s, const gchar *p, gchar *file, gint line)
+GString *
+string_prepend_track(GString *s, const gchar *p, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1237,8 +1261,8 @@ GString *string_prepend_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_prepend_c_track(
-	GString *s, gchar c, gchar *file, gint line)
+GString *
+string_prepend_c_track(GString *s, gchar c, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1246,8 +1270,9 @@ GString *string_prepend_c_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_insert_track(
-	GString *s, gint pos, const gchar *p, gchar *file, gint line)
+GString *
+string_insert_track(GString *s, gint pos, const gchar *p,
+	gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1255,8 +1280,8 @@ GString *string_insert_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_insert_c_track(
-	GString *s, gint pos, gchar c, gchar *file, gint line)
+GString *
+string_insert_c_track(GString *s, gint pos, gchar c, gchar *file, gint line)
 {
 	gchar *old = s->str;
 
@@ -1264,8 +1289,8 @@ GString *string_insert_c_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_sprintf_track(
-	GString *s, gchar *file, gint line, const gchar *fmt, ...)
+GString *
+string_sprintf_track(GString *s, gchar *file, gint line, const gchar *fmt, ...)
 {
 	va_list args;
 	gchar *o;
@@ -1280,8 +1305,8 @@ GString *string_sprintf_track(
 	return string_str_track(s, old, file, line);
 }
 
-GString *string_sprintfa_track(
-	GString *s, gchar *file, gint line, const gchar *fmt, ...)
+GString *
+string_sprintfa_track(GString *s, gchar *file, gint line, const gchar *fmt, ...)
 {
 	va_list args;
 	gchar *o;
@@ -1314,27 +1339,24 @@ struct leak_set {
 	GHashTable *places;		/* Maps "file:4" -> leak_record */
 };
 
-/*
- * leak_init
- *
+/**
  * Initialize the leak accumulator by "file:line"
  */
 gpointer leak_init(void)
 {
 	struct leak_set *ls;
 
-	ls = malloc(sizeof(struct leak_set));
+	ls = malloc(sizeof *ls);
 	ls->places = g_hash_table_new(g_str_hash, g_str_equal);
 
 	return ls;
 }
 
-/*
- * leak_free_kv
- *
+/**
  * Get rid of the key/value tupple in the leak table.
  */
-static gboolean leak_free_kv(gpointer key, gpointer value, gpointer unused_user)
+static gboolean
+leak_free_kv(gpointer key, gpointer value, gpointer unused_user)
 {
 	(void) unused_user;
 	free(key);
@@ -1342,12 +1364,11 @@ static gboolean leak_free_kv(gpointer key, gpointer value, gpointer unused_user)
 	return TRUE;
 }
 
-/*
- * leak_close
- *
+/**
  * Dispose of the leaks accumulated.
  */
-void leak_close(gpointer o)
+void
+leak_close(gpointer o)
 {
 	struct leak_set *ls = (struct leak_set *) o;
 
@@ -1357,14 +1378,13 @@ void leak_close(gpointer o)
 	free(ls);
 }
 
-/*
- * leak_add
- *
+/**
  * Record a new leak of `size' bytes allocated at `file', line `line'.
  */
-void leak_add(gpointer o, guint32 size, gchar *file, gint line)
+void
+leak_add(gpointer o, guint32 size, gchar *file, gint line)
 {
-	struct leak_set *ls = (struct leak_set *) o;
+	struct leak_set *ls = o;
 	gchar key[1024];
 	struct leak_record *lr;
 	gboolean found;
@@ -1373,7 +1393,9 @@ void leak_add(gpointer o, guint32 size, gchar *file, gint line)
 
 	g_assert(file);
 	g_assert(line >= 0);
-	gm_snprintf(key, sizeof key, "%s:%d", file, line);
+
+	concat_strings(key, sizeof key,
+		file, ":", uint64_to_string(line), (void *) 0);
 	found = g_hash_table_lookup_extended(ls->places, key, &k, &v);
 
 	if (found) {
@@ -1393,13 +1415,14 @@ struct leak {			/* A memory leak, for sorting purposes */
 	struct leak_record *lr;
 };
 
-/*
+/**
  * leak_size_cmp		-- qsort() callback
  *
  * Compare two pointers to "struct leak" based on their size value,
  * in reverse order.
  */
-static gint leak_size_cmp(const void *p1, const void *p2)
+static gint
+leak_size_cmp(const void *p1, const void *p2)
 {
 	guint32 i1 = ((struct leak *) p1)->lr->size;
 	guint32 i2 = ((struct leak *) p2)->lr->size;
@@ -1415,16 +1438,17 @@ struct filler {			/* Used by hash table iterator to fill leak array */
 	gint idx;			/* Next index to be filled */
 };
 
-/*
+/**
  * fill_array			-- hash table iterator
  *
  * Append current hash table entry at the end of the "leaks" array.
  */
-static void fill_array(gpointer key, gpointer value, gpointer user)
+static void
+fill_array(gpointer key, gpointer value, gpointer user)
 {
-	struct filler *filler = (struct filler *) user;
+	struct filler *filler = user;
 	struct leak *l;
-	struct leak_record *lr = (struct leak_record *) value;
+	struct leak_record *lr = value;
 
 	g_assert(filler->idx < filler->count);
 
@@ -1433,12 +1457,11 @@ static void fill_array(gpointer key, gpointer value, gpointer user)
 	l->lr = lr;
 }
 
-/*
- * leak_dump
- *
+/**
  * Dump the links sorted by decreasing leak size.
  */
-void leak_dump(gpointer o)
+void
+leak_dump(gpointer o)
 {
 	struct leak_set *ls = (struct leak_set *) o;
 	gint count;
@@ -1450,7 +1473,7 @@ void leak_dump(gpointer o)
 	if (count == 0)
 		return;
 
-	filler.leaks = (struct leak *) malloc(sizeof(struct leak) * count);
+	filler.leaks = malloc(sizeof(struct leak) * count);
 	filler.count = count;
 	filler.idx = 0;
 
@@ -1500,7 +1523,8 @@ struct afiller {		/* Used by hash table iterator to fill alloc array */
  * Compare two pointers to "struct stat" based on their allocation value,
  * in reverse order. -- qsort() callback
  */
-static gint stats_allocated_cmp(const void *p1, const void *p2)
+static gint
+stats_allocated_cmp(const void *p1, const void *p2)
 {
 	guint32 i1 = (*(struct stats **) p1)->allocated;
 	guint32 i2 = (*(struct stats **) p2)->allocated;
@@ -1514,7 +1538,8 @@ static gint stats_allocated_cmp(const void *p1, const void *p2)
  * Compare two pointers to "struct stat" based on their total allocation value,
  * in reverse order. -- qsort() callback
  */
-static gint stats_total_allocated_cmp(const void *p1, const void *p2)
+static gint
+stats_total_allocated_cmp(const void *p1, const void *p2)
 {
 	guint32 i1 = (*(struct stats **) p1)->total_allocated;
 	guint32 i2 = (*(struct stats **) p2)->total_allocated;
@@ -1528,7 +1553,8 @@ static gint stats_total_allocated_cmp(const void *p1, const void *p2)
  * Compare two pointers to "struct stat" based on their residual value,
  * in reverse order. -- qsort() callback
  */
-static gint stats_residual_cmp(const void *p1, const void *p2)
+static gint
+stats_residual_cmp(const void *p1, const void *p2)
 {
 	struct stats *s1 = *(struct stats **) p1;
 	struct stats *s2 = *(struct stats **) p2;
@@ -1544,7 +1570,8 @@ static gint stats_residual_cmp(const void *p1, const void *p2)
  * Compare two pointers to "struct stat" based on their total residual value,
  * in reverse order. -- qsort() callback
  */
-static gint stats_total_residual_cmp(const void *p1, const void *p2)
+static gint
+stats_total_residual_cmp(const void *p1, const void *p2)
 {
 	struct stats *s1 = *(struct stats **) p1;
 	struct stats *s2 = *(struct stats **) p2;
@@ -1560,7 +1587,8 @@ static gint stats_total_residual_cmp(const void *p1, const void *p2)
  * Append current hash table entry at the end of the "stats" array
  * in the supplied filler structure.  -- hash table iterator
  */
-static void stats_fill_array(gpointer unused_key, gpointer value, gpointer user)
+static void
+stats_fill_array(gpointer unused_key, gpointer value, gpointer user)
 {
 	struct afiller *filler = (struct afiller *) user;
 	struct stats *st = (struct stats *) value;
@@ -1577,7 +1605,8 @@ static void stats_fill_array(gpointer unused_key, gpointer value, gpointer user)
 /**
  * Dump the stats held in the specified array.
  */
-static void stats_array_dump(FILE *f, struct afiller *filler)
+static void
+stats_array_dump(FILE *f, struct afiller *filler)
 {
 	gint i;
 
@@ -1645,7 +1674,8 @@ static void stats_array_dump(FILE *f, struct afiller *filler)
  * When `total' is TRUE, sorting is made on the total stats instead of
  * the incremental ones.
  */
-void alloc_dump(FILE *f, gboolean total)
+void
+alloc_dump(FILE *f, gboolean total)
 {
 	gint count;
 	struct afiller filler;
@@ -1656,7 +1686,7 @@ void alloc_dump(FILE *f, gboolean total)
 	if (count == 0)
 		return;
 
-	now = time(NULL);
+	now = tm_time();
 	fprintf(f, "--- distinct allocation spots found: %d at %s\n",
 		count, short_time(now - init_time));
 
@@ -1703,9 +1733,10 @@ void alloc_dump(FILE *f, gboolean total)
 }
 
 /**
- * Reset incrementel allocation and free counters. -- hash table iterator
+ * Reset incremental allocation and free counters. -- hash table iterator
  */
-static void stats_reset(gpointer uu_key, gpointer value, gpointer uu_user)
+static void
+stats_reset(gpointer uu_key, gpointer value, gpointer uu_user)
 {
 	struct stats *st = (struct stats *) value;
 
@@ -1719,9 +1750,10 @@ static void stats_reset(gpointer uu_key, gpointer value, gpointer uu_user)
  * Atomically dump the allocation stats and reset the incremental allocation
  * statistics.
  */
-void alloc_reset(FILE *f, gboolean total)
+void
+alloc_reset(FILE *f, gboolean total)
 {
-	time_t now = time(NULL);
+	time_t now = tm_time();
 
 	alloc_dump(f, total);
 	g_hash_table_foreach(stats, stats_reset, NULL);
@@ -1734,4 +1766,4 @@ void alloc_reset(FILE *f, gboolean total)
 
 #endif /* MALLOC_STATS */
 
-/* vi: set ts=4:  */
+/* vi: set ts=4 sw=4 cindent:  */
