@@ -1,5 +1,5 @@
 /*
- * $Id: url.c,v 1.4 2004/10/20 15:38:58 cbiere Exp $
+ * $Id: url.c,v 1.23 2005/10/20 07:17:12 cbiere Exp $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -23,29 +23,39 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * URL handling of specific formats.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: url.c,v 1.4 2004/10/20 15:38:58 cbiere Exp $");
+RCSID("$Id: url.c,v 1.23 2005/10/20 07:17:12 cbiere Exp $");
 
+#include "host_addr.h"
 #include "url.h"
 #include "misc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
-// XXX -- needs to initialze the debug level of the lib functions -- RAM
-static gboolean url_debug = FALSE;
+extern guint32 lib_debug;
 
 #define ESCAPE_CHAR		'%'
 #define TRANSPARENT_CHAR(x,m) \
 	((x) >= 32 && (x) < 128 && (is_transparent[(x)-32] & (m)))
 
-/*
- * Reserved chars: ";", "/", "?", ":", "@", "=" and "&"
- * Unsafe chars  : " ", '"', "<", ">", "#", and "%"
- * Misc chars    : "{", "}", "|", "\", "^", "~", "[", "]" and "`"
+/**
+ * - Reserved chars: ";", "/", "?", ":", "@", "=" and "&"
+ * - Unsafe chars  : " ", '"', "<", ">", "#", and "%"
+ * - Misc chars    : "{", "}", "|", "\", "^", "~", "[", "]" and "`"
  *
- * Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
- * Bit 1 encodes regular transparent set minus '+' (query string).
+ * - Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
+ * - Bit 1 encodes regular transparent set minus '+' (query string).
  */
 static const guint8 is_transparent[96] = {
 /*  0 1 2 3 4 5 6 7 8 9 a b c d e f */	/* 0123456789abcdef -            */
@@ -64,7 +74,9 @@ static const char hex_alphabet[] = "0123456789ABCDEF";
 
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
- * `mask' tells us whether we're escaping an URL path or a query string.
+ *
+ * @param `url' no brief description.
+ * @param `mask' tells us whether we're escaping an URL path or a query string.
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
@@ -82,7 +94,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 			need_escape++;
 
 	if (need_escape == 0)
-		return (gchar *) url; /* Override const */
+		return deconstify_gchar(url);
 
 	new = g_malloc(p - url + (need_escape << 1));
 
@@ -256,15 +268,15 @@ url_unescape(gchar *url, gboolean inplace)
 		unescaped_memory = p - url - (need_unescape << 1) + 1;
 		new = g_malloc(unescaped_memory);
 	}
-	
+
 	for (p = url, q = new, c = *p++; c; c = *p++) {
 		if (c != ESCAPE_CHAR)
 			*q++ = c;
 		else {
 			if ((c = *p++)) {
-				gint v = (hex2dec(c) << 4);
+				gint v = hex2int_inline(c) << 4;
 				if ((c = *p++))
-					v += hex2dec(c);
+					v += hex2int_inline(c);
 				else
 					g_assert_not_reached();	/* Handled in pre-scan above */
 
@@ -301,7 +313,7 @@ url_params_parse(gchar *query)
 	gchar *value = NULL;
 	gboolean in_value = FALSE;
 
-	up = walloc(sizeof(*up));
+	up = walloc(sizeof *up);
 	up->params = g_hash_table_new(g_str_hash, g_str_equal);
 	up->count = 0;
 
@@ -352,8 +364,8 @@ url_params_parse(gchar *query)
  * Get the value of a parameter, or NULL if the parameter is not present.
  * The value returned has already been URL-unescaped.
  */
-gchar *
-url_params_get(url_params_t *up, gchar *name)
+const gchar *
+url_params_get(url_params_t *up, const gchar *name)
 {
 	g_assert(up != NULL);
 	g_assert(up->params != NULL);
@@ -380,42 +392,38 @@ url_params_free(url_params_t *up)
 	g_hash_table_foreach(up->params, free_params_kv, NULL);
 	g_hash_table_destroy(up->params);
 
-	wfree(up, sizeof(*up));
+	wfree(up, sizeof *up);
 }
 
 static gboolean
-url_safe_char(gint c, url_policy_t p)
+url_safe_char(gchar c, url_policy_t p)
 {
-	if (p & URL_POLICY_GWC_RULES) {
-		/* Reject allow anything unreasonable */
-		if (!isascii(c)) {
-			if (url_debug)
-				g_warning("Non-ASCII character in GWC URL; rejected");
-			return FALSE;
-		}
+	if (!isascii(c) || is_ascii_cntrl(c))
+		return FALSE;
 
-    	if (!isalnum(c)
-        	&& c != '/' && c != '.' && c != '_' && c != '-' && c != '~'
-      	) {
-			if (url_debug)
+	if (!(p & URL_POLICY_ALLOW_ANY_CHAR)) {
+    	if (
+			!is_ascii_lower(c) &&
+			!is_ascii_digit(c) &&
+			NULL == strchr("/._-~", c)
+		) {
+			if (lib_debug)
       			g_warning("Unsafe character in GWC URL; rejected");
       		return FALSE;
     	}
-	} else {
-		/* XXX: Figure out what's the correct set for this */
-		if (!isascii(c) || iscntrl(c)) {
-			return FALSE;
-		}
 	}
+
 	return TRUE;
 }
 
 /**
+ * @attention
  * NB: May modify ``url'' in all cased; pass a copy if necessary!
  *
- * Returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
+ * @returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
  * it returns either a pointer to the original URL or a g_malloc()ed
  * modified copy which has to be g_free()ed.
+ *
  * The URL is validated according to the specified policy. Unnecessary
  * parts such as ":80" (for the port) and "/./" are removed: the hostname
  * is forced to lowercase; a base URI "/" is appended if missing.
@@ -425,80 +433,81 @@ url_safe_char(gint c, url_policy_t p)
 gchar *
 url_normalize(gchar *url, url_policy_t pol)
 {
-	gint c, dots = 0;
-	gchar *endptr;
-	const gchar *tld = NULL;
-	guint16 port = 0;
-	gchar *p, *q = url;
-	const gchar *uri;
 	static const char http_prefix[] = "http://";
-	static gboolean url_debug = FALSE;
-	gchar *warn = NULL;
+	const gchar *p, *uri, *endptr, *tld = NULL;
+	gchar c, *q, *warn = NULL;
+	host_addr_t addr;
 
 	g_assert(url);
 
-	if (0 != strncmp(q, http_prefix, sizeof http_prefix - 1)) {
-		if (url_debug)
+	if (NULL == (q = is_strcaseprefix(url, http_prefix))) {
+		if (lib_debug)
 			g_warning("URL \"%s\" isn't preceded by \"%s\"", url, http_prefix);
 		return NULL;
 	}
-	q += sizeof http_prefix - 1;
+	/* Make sure the prefix is all-lowercase */
+	memcpy(url, http_prefix, CONST_STRLEN(http_prefix));
 
-	if (!isalnum((guchar) *q)) {
+	if (!is_ascii_alnum(*q)) {
 		warn = "HTTP prefix MUST be followed by an alphanum";
 		goto bad;
 	}
 
-	if (gchar_to_ip_strict(q, NULL, (const gchar **) &endptr)) {
+	if (
+		string_to_host_or_addr(q, &endptr, &addr) &&
+		is_host_addr(addr) &&
+		('/' == *endptr || ':' == *endptr || '\0' == *endptr)
+	) {
 		if (!(pol & URL_POLICY_ALLOW_IP_AS_HOST)) {
 			warn = "URLs without hostnames have been disabled";
 			goto bad;
 		}
-		q = endptr;
+		q = deconstify_gchar(endptr);
 
 	} else {
-		/* The ``host'' part is not an IP address */  
+		size_t dots = 0;
+
+		/* The ``host'' part is not an IP address */
 
 		for (/* NOTHING */; *q != '\0'; q++) {
-			gint c;
 
-		    for (/* NOTHING */; (c = (guchar) *q) != '\0'; q++) {
-        		if (isalnum(c)) {
-          			*q = tolower(c);
-        		} else if (c != '-') {
+		    for (/* NOTHING */; '\0' != (c = *q); q++) {
+        		if (is_ascii_alnum(c)) {
+          			*q = ascii_tolower(c);
+        		} else if ('-' != c) {
           			break;
         		}
-      		}   
+      		}
 
-			if (c == '\0') {
+			if ('\0' == c) {
 				if (dots < 1 && !(pol & URL_POLICY_ALLOW_LOCAL_HOSTS)) {
 					warn = "current URL policy forbids local hosts";
 					goto bad;
 				}
 				break;
-			} else if (c == '.') {
+			} else if ('.' == c) {
 
-				if (!(isalnum((guchar) *(q - 1)) && isalnum((guchar) q[1]))) {
+				if ( !(is_ascii_alnum(*(q - 1)) && is_ascii_alnum(q[1]))) {
 					warn = "a dot must be preceded and followed by an alphanum";
 					goto bad;
 				}
 				dots++;
-				if (q[1] == '\0' || q[1] == ':' || q[1] == '/')
+				if ('\0' == q[1] || ':' == q[1] || '/' == q[1])
 					break;
 
 				tld = &q[1];
-			} else if (c == '\0' || c == '/' || c == ':') {
+			} else if ('\0' == c || '/' == c || ':' == c) {
 				break;
 			} else {
-				if (url_debug)
+				if (lib_debug)
 					g_warning("Bad URL \"%s\": "
 						"invalid character '%c' in ``host:port'' part.",
-						url, isprint(c) ? c : '?');
+						url, is_ascii_print(c) ? c : '?');
 				return NULL;
 			}
 		}
 
-		if (!tld || !(isalpha((guchar) tld[0]) && isalpha((guchar) tld[1]))) {
+		if (!tld || !(is_ascii_alpha(tld[0]) && is_ascii_alpha(tld[1]))) {
 			warn = "no or invalid top-level domain";
 			goto bad;
 		}
@@ -506,127 +515,112 @@ url_normalize(gchar *url, url_policy_t pol)
 	}
 
 	p = q;
-	if (*q == ':' ) {
-		gulong v = 0;
+	if (':' == *q) {
+		guint32 port;
+		gint error;
 
-		q++;
-		/* strtoul() allows leading spaces, +, - and zeroes */
-		if (NULL != strchr("123456789", *q)) {
-			errno = 0;
-			v = strtoul(q, &endptr, 10);
+		q++; /* Skip ':' */
+
+		/* Reject port numbers with leading zeros */
+		if (!is_ascii_digit(*q) || '0' == *q) {
+			error = EINVAL;
+			port = 0;
+		} else {
+			port = parse_uint32(q, &endptr, 10, &error);
 		}
-		if (v < 1 || v > 65535 || errno) {
+
+		if (error || port < 1 || port > 65535) {
 			warn = "':' MUST be followed a by port value (1-65535)";
 			goto bad;
 		}
-		port = (guint16) v;
+
+		if (!(URL_POLICY_ALLOW_ANY_PORT & pol) && 80 != port && port < 1024) {
+			warn = "Ports below 1024 other than 80 are disallowed";
+			goto bad;
+		}
+
 		p = endptr;
 		if (port == /* HTTP_PORT */ 80) {
 			/* We don't want the default port in a URL;
 			 * this does also prevents duplicates. */
 			q--;
 		} else {
-			q = endptr;
+			q = deconstify_gchar(endptr);
 		}
 	}
 
-	if (*p != '/' && *p != '\0') {
+	if ('/' != *p && '\0' != *p) {
 		warn = "host must be followed by ':', '/' or NUL";
 		goto bad;
 	}
 
-	uri = q;
+	uri = p;
 
 	/* Scan path */
-	for (/* NOTHING */; (c = *(guchar *) p) != '\0'; q++, p++) {
+	for (/* NOTHING */; '\0' != (c = *p); p++) {
 		if (!url_safe_char(c, pol)) {
 			warn = "URL contains characters prohibited by policy";
 			goto bad;
 		}
-      
-		/* Handle relative paths i.e., /. and /.. */
-		if (c != '/') {
-				*q = c;
-			continue;
-		}
-
-		/* Special handling for '/' follows */
-		do {
-
-			*q = '/';
-
-			while (p[1] == '/')
-				p++;
-
-			if (0 == strcmp(p, "/.")) {
-				p++;
-				if (url_debug)
-					g_message("ignoring trailing \"/.\" in URI");
-			} else if (0 == strcmp(p, "/..")) {
-				if (url_debug)
-					g_message("trailing \"/..\" in URI; rejected");
-				return NULL;
-			} else if (0 == strncmp(p, "/./", sizeof "/./" - 1)) {
-				p += 2;
-				if (url_debug)
-					g_message("ignoring unnecessary \"/./\" in URI");
-			} else if (0 == strncmp(p, "/../", sizeof "/../" - 1)) {
-				p += 3;
-				if (url_debug)
-					g_message("ascending one component in URI");
-				while (*--q != '/')
-					if (q <= uri) {
-						warn = "URI ascends beyond root per \"/../\"";
-						goto bad;
-					}
-			} else {
-				break;
-			}
-
-		} while (*p == '/' && (p[1] == '/' || p[1] == '.'));
-
 	}
-	*q = '\0';
 
-	if (pol & URL_POLICY_GWC_RULES) {
+	if (q != uri) {
+		size_t len;
+
+		len = strlen(uri);
+		memmove(q, uri, len);
+		q[len] = '\0';
+	}
+	uri = q;
+
+	if (0 != canonize_path(q, q)) {
+		warn = "Could not canonize URI";
+		goto bad;
+	}
+
+	if (!(URL_POLICY_ALLOW_STATIC_FILES & pol)) {
 		static const struct {
-			ssize_t len;
 			const gchar *ext;
+			size_t len;
 		} static_types[] = {
-			{ 5, ".html" },
-			{ 4, ".htm" },
-			{ 4, ".txt" }
+#define D(x) { (x), CONST_STRLEN(x) }
+			D(".html"),
+			D(".htm"),
+			D(".txt"),
+#undef D
 		};
 		guint i;
 
 		for (i = 0; i < G_N_ELEMENTS(static_types); i++)
-    		if (!strcasecmp(q - static_types[i].len, static_types[i].ext)) {
+    		if (
+				0 == ascii_strcasecmp(q - static_types[i].len,
+						static_types[i].ext)
+			) {
 				warn = "URL points probably to static data; rejected";
       			goto bad;
     		}
 	}
 
 	/* Add a trailing slash; if the URI is empty (to prevent dupes) */
-	if (*uri == '\0') {
+	if ('\0' == uri[0]) {
 		ssize_t len = q - url;
- 
-		g_assert(len > 0); 
-		p = g_malloc(len + sizeof "/");
-		if (p) {
-			memcpy(p, url, len);
-			p[len] = '/';
-			p[len + 1] = '\0';
-		}
-		url = p;
+		gchar *s;
+
+		g_assert(len > 0);
+		s = g_malloc(len + sizeof "/");
+		memcpy(s, url, len);
+		s[len] = '/';
+		s[len + 1] = '\0';
+		url = s;
 	}
 
-	if (url_debug)
+	if (lib_debug > 1)
 		g_message("url=\"%s\"", url);
 
 	return url;
 
 bad:
-	if (url_debug)
+	if (lib_debug)
 		g_warning("invalid URL \"%s\": %s", url, warn);
 	return NULL;
 }
