@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.c,v 1.3 2004/11/26 14:52:42 rmanfredi Exp $
+ * $Id: vendors.c,v 1.14 2005/11/13 10:36:12 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Vendor code management.
+ *
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: vendors.c,v 1.3 2004/11/26 14:52:42 rmanfredi Exp $");
+RCSID("$Id: vendors.c,v 1.14 2005/11/13 10:36:12 cbiere Exp $");
 
 #include "vendors.h"
 #include "endian.h"
@@ -74,6 +84,7 @@ struct vendor {
     { T_LIME, "LimeWire" },
     { T_LION, "LionShare" },
     { T_MACT, "Mactella" },
+    { T_MESH, "iMesh" },
     { T_MIRT, "Mirtella" },
     { T_MLDK, "MLDonkey" },
     { T_MMMM, "Morpheus-v2" },
@@ -94,6 +105,7 @@ struct vendor {
     { T_RASP, "Rasputin" },
     { T_RAZA, "Shareaza" },
     { T_SHNB, "Shinobu" },
+    { T_SNOW, "FrostWire" },
     { T_SNUT, "SwapNut" },
     { T_STRM, "Storm" },
     { T_SWAP, "Swapper" },
@@ -111,113 +123,96 @@ struct vendor {
 	/* Above line intentionally left blank (for "!}sort" on vi) */
 };
 
-#define END(v)		(v - 1 + sizeof(v) / sizeof(v[0]))
-
-/*
- * vendor_code_cmp
- *
- * Compare two codes, alphanumerically (i.e. "ACQX" < "GTKG").
- * Returns -1/0/+1 depending on comparison's sign.
- * Note that this comparison is case-sensitive.
- */
-inline gint vendor_code_cmp(guint32 a, guint32 b)
-{
-	return a < b ? -1 : a > b ? 1 : 0;
-}
-
-/*
- * find_vendor
- *
+/**
  * Find vendor name, given vendor code.
- * Returns vendor string if found, NULL otherwise.
+ *
+ * @returns vendor string if found, NULL otherwise.
  */
-static gchar *find_vendor(guchar raw[4])
+static gchar *
+find_vendor(guint32 code)
 {
-	struct vendor *low = vendor_map;
-	struct vendor *high = END(vendor_map);
-	guint32 code;
+#define GET_KEY(i) (vendor_map[(i)].code)
+#define FOUND(i) G_STMT_START { \
+	return vendor_map[(i)].name; \
+	/* NOTREACHED */ \
+} G_STMT_END
 
-    READ_GUINT32_BE(raw, code);
+	BINARY_SEARCH(guint32, code, G_N_ELEMENTS(vendor_map), VENDOR_CODE_CMP,
+		GET_KEY, FOUND);
 
-	while (low <= high) {
-		struct vendor *mid = low + (high - low) / 2;
-		gint c = vendor_code_cmp(mid->code,  code);
-
-		if (c == 0)
-			return mid->name;
-		else if (c < 0)
-			low = mid + 1;
-		else
-			high = mid - 1;
-	}
-
-	return NULL;		/* Not found */
+#undef FOUND
+#undef GET_KEY
+	return NULL; /* not found */
 }
 
-/*
- * is_vendor_known:
- *
- * Return true is gtk-gnutella knows the given 4-byte vendor code.
+/**
+ * @return true is gtk-gnutella knows the given 4-byte vendor code.
  */
-gboolean is_vendor_known(guchar raw[4])
+gboolean
+is_vendor_known(union vendor_code code)
 {
-    if (raw[0] == '\0')
+    if (code.be32 == 0)
         return FALSE;
 
-	return find_vendor(raw) != NULL;
+	return find_vendor(ntohl(code.be32)) != NULL;
 }
 
-/*
- * vendor_code_str
- *
+/**
  * Make up a printable version of the vendor code.
- * Returns pointer to static data.
+ *
+ * @param code A 4-letter Gnutella vendor ID in host-endian order thus
+ *        after peek_be32() or ntohl().
+ *
+ * @return pointer to static data.
  */
-gchar *vendor_code_str(guint32 code)
+const gchar *
+vendor_code_str(guint32 code)
 {
-	static gchar temp[5];
-    gint i;
+	static gchar temp[1 + sizeof code];
+    guint i;
 
-	if (code == 0)
+	STATIC_ASSERT(5 == G_N_ELEMENTS(temp));
+
+    if (code == 0)
 		return "null";
 
-	WRITE_GUINT32_BE(code, temp);
-	temp[4] = '\0';
-
-	for (i = 0; i < 4; i++) {
-        guchar c = temp[i];
-		if (!is_ascii_print(c))
+	poke_be32(&temp[0], code);
+	for (i = 0; i < G_N_ELEMENTS(temp) - 1; i++) {
+		if (!is_ascii_print(temp[i]))
 			temp[i] = '.';
 	}
 
+	temp[4] = '\0';
+
 	return temp;
 }
 
-/*
- * lookup_vendor_name
- *
+/**
  * Return the "human readable" name associated with the 4-byte vendor code.
  * If we can't understand the code return NULL or if the 4-byte code
  * consists only of printable characters, return the code as a string.
  */
-gchar *lookup_vendor_name(guchar raw[4])
+const gchar *
+lookup_vendor_name(union vendor_code code)
 {
-	static gchar temp[5];
+	static gchar temp[1 + G_N_ELEMENTS(code.b)];
 	gchar *name;
     guint i;
 
-    if (raw[0] == '\0')
+	STATIC_ASSERT(5 == G_N_ELEMENTS(temp));
+	
+    if (code.be32 == 0)
         return NULL;
 
-	name = find_vendor(raw);
+	name = find_vendor(ntohl(code.be32));
 	if (name != NULL)
 		return name;
 
+	
 	/* Unknown type, look whether we have all printable ASCII */
-	for (i = 0; i < sizeof(raw); i++) {
-        guchar c = raw[i];
-		if (is_ascii_print(c))
-            temp[i] = c;
+	for (i = 0; i < G_N_ELEMENTS(code.b); i++) {
+		if (is_ascii_print(code.b[i]))
+            temp[i] = code.b[i];
 		else {
             temp[0] = '\0';
 			break;
@@ -228,4 +223,14 @@ gchar *lookup_vendor_name(guchar raw[4])
 	return temp[0] ? temp : NULL;
 }
 
-/* vi: set ts=4: */
+/**
+ * Initialize the vendor lookup.
+ */
+void
+vendor_init(void)
+{
+	BINARY_ARRAY_SORTED(vendor_map, struct vendor, code,
+		VENDOR_CODE_CMP, vendor_code_str);
+}
+
+/* vi: set ts=4 sw=4 cindent: */
