@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats_common.c,v 1.14 2004/10/25 04:29:16 cbiere Exp $
+ * $Id: gnet_stats_common.c,v 1.21 2005/08/11 06:32:14 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,19 +25,22 @@
 
 #include "gui.h"
 
-RCSID("$Id: gnet_stats_common.c,v 1.14 2004/10/25 04:29:16 cbiere Exp $");
+RCSID("$Id: gnet_stats_common.c,v 1.21 2005/08/11 06:32:14 cbiere Exp $");
+
 #include "gnet_stats_common.h"
+#include "gtk-missing.h"
+
 #include "if/core/net_stats.h"
 #include "if/bridge/ui2c.h"
+
 #include "lib/glib-missing.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-/*
- * msg_type_str
- *
+/**
  * Gets the string associated with the message type.
  */
-const gchar *msg_type_str(int value)
+const gchar *
+msg_type_str(gint value)
 {
 	static const char * const strs[] = {
 		N_("Unknown"),
@@ -46,11 +49,11 @@ const gchar *msg_type_str(int value)
 		N_("Bye"),
 		N_("QRP"),
 		N_("HSEP"),
-		N_("Vendor Spec."),
-		N_("Vendor Std."),
+		N_("Vendor spec."),
+		N_("Vendor std."),
 		N_("Push"),
 		N_("Query"),
-		N_("Query Hit"),
+		N_("Query hit"),
 		N_("Total"),
 	};
 
@@ -64,17 +67,17 @@ const gchar *msg_type_str(int value)
 	return _(strs[value]);
 }
 
-int msg_type_str_size(void)
+gint
+msg_type_str_size(void)
 {
 	return MSG_TYPE_COUNT;
 }
 
-/*
- * msg_drop_str
- *
+/**
  * Gets the string associated with the drop reason.
  */
-const gchar *msg_drop_str(int value)
+const gchar *
+msg_drop_str(gint value)
 {
 	static const char * const strs[] = {
 		N_("Bad size"),
@@ -107,7 +110,7 @@ const gchar *msg_drop_str(int value)
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(strs) == MSG_DROP_REASON_COUNT);
-	
+
 	if ((guint) value >= G_N_ELEMENTS(strs)) {
 		g_warning("Requested general_type_str %d is invalid", value);
 		return "";
@@ -116,12 +119,11 @@ const gchar *msg_drop_str(int value)
 	return _(strs[value]);
 }
 
-/*
- * general_type_str
- *
- * Gets the string associated with the general message
+/**
+ * Gets the string associated with the general message.
  */
-const gchar *general_type_str(int value)
+const gchar *
+general_type_str(gint value)
 {
 	static const char * const strs[] = {
 		N_("Routing errors"),
@@ -165,7 +167,7 @@ const gchar *general_type_str(int value)
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(strs) == GNR_TYPE_COUNT);
-	
+
 	if ((guint) value >= G_N_ELEMENTS(strs)) {
 		g_warning("Requested general_type_str %d is invalid", value);
 		return "";
@@ -174,18 +176,19 @@ const gchar *general_type_str(int value)
 	return _(strs[value]);
 }
 
-/*
- * horizon_stat_str
+/**
+ * @returns the cell contents for the horizon stats table.
  *
- * Returns the cell contents for the horizon stats table.
+ * @warning
  * NB: The static buffers for each column are disjunct.
  */
-const gchar *horizon_stat_str(gint row,	c_horizon_t column)
+const gchar *
+horizon_stat_str(gint row, c_horizon_t column)
 {
     switch (column) {
     case c_horizon_hops:
 		{
-    		static gchar buf[21];
+    		static gchar buf[UINT64_DEC_BUFLEN];
 
 			gm_snprintf(buf, sizeof(buf), "%d", row);
            	return buf;
@@ -209,20 +212,18 @@ const gchar *horizon_stat_str(gint row,	c_horizon_t column)
     return NULL;
 }
 
-/*
- * gnet_stats_gui_horizon_update
- *
+/**
  * Updates the horizon statistics in the statusbar.
+ *
  * This is an event-driven callback called from the HSEP code
  * using the event listener framework. In addition to taking into account
  * the HSEP information, the number of established non-HSEP nodes and
  * their library size (if provided) are added to the values displayed.
  */
-
-void gnet_stats_gui_horizon_update(hsep_triple *table, guint32 triples)
+void
+gnet_stats_gui_horizon_update(hsep_triple *table, guint32 triples)
 {
 	const guint32 hops = 4U;      /* must be <= HSEP_N_MAX */
-	gchar s[64];
 	guint64 val;
 	hsep_triple other;
 
@@ -238,21 +239,19 @@ void gnet_stats_gui_horizon_update(hsep_triple *table, guint32 triples)
 	 */
 
 	val = table[hops][HSEP_IDX_NODES] + other[HSEP_IDX_NODES];
-	gm_snprintf(s, sizeof(s),
-	            "%" PRIu64 " %s", val, val == 1 ? _("node") : _("nodes"));
-	gtk_label_set_text(GTK_LABEL(lookup_widget(main_window,
-	                   "label_statusbar_horizon_node_count")), s);
+	gtk_label_printf(GTK_LABEL(
+			lookup_widget(main_window, "label_statusbar_horizon_node_count")),
+		"%s %s", uint64_to_string(val), NG_("node", "nodes", val));
 
 	val = table[hops][HSEP_IDX_FILES] + other[HSEP_IDX_FILES];
-	gm_snprintf(s, sizeof(s), "%" PRIu64 " %s",
-	            val, val == 1 ? _("file") : _("files"));
-	gtk_label_set_text(GTK_LABEL(lookup_widget(main_window,
-	                   "label_statusbar_horizon_file_count")), s);
+	gtk_label_printf(GTK_LABEL(
+			lookup_widget(main_window, "label_statusbar_horizon_file_count")),
+		"%s %s", uint64_to_string(val), NG_("file", "files", val));
 
 	val = table[hops][HSEP_IDX_KIB] + other[HSEP_IDX_KIB];
-	gm_snprintf(s, sizeof(s), "%s", short_kb_size(val));
-	gtk_label_set_text(GTK_LABEL(lookup_widget(main_window,
-	                   "label_statusbar_horizon_kb_count")), s);
+	gtk_label_printf(GTK_LABEL(
+			lookup_widget(main_window, "label_statusbar_horizon_kb_count")),
+		"%s", short_kb_size(val));
 }
 
 /* vi: set ts=4 sw=4 cindent: */
