@@ -1,5 +1,5 @@
 /*
- * $Id: callbacks.c,v 1.2 2004/11/13 09:45:23 cbiere Exp $
+ * $Id: callbacks.c,v 1.8 2005/11/15 21:25:28 rmanfredi Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -25,11 +25,12 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: callbacks.c,v 1.2 2004/11/13 09:45:23 cbiere Exp $");
+RCSID("$Id: callbacks.c,v 1.8 2005/11/15 21:25:28 rmanfredi Exp $");
 
 #include "gtk/misc.h"
-#include "gtk/statusbar.h"
+#include "gtk/search.h"
 #include "gtk/search_stats.h"
+#include "gtk/statusbar.h"
 
 #include "if/bridge/ui2c.h"
 #include "if/gui_property.h"
@@ -37,7 +38,7 @@ RCSID("$Id: callbacks.c,v 1.2 2004/11/13 09:45:23 cbiere Exp $");
 
 #include "lib/override.h"		/* Must be the last header included */
 
-/* 
+/*
  * Create a function for the focus out signal and make it call
  * the callback for the activate signal.
  */
@@ -82,7 +83,7 @@ on_progressbar_bws_in_button_press_event(GtkWidget *unused_widget,
 		GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -96,7 +97,7 @@ on_progressbar_bws_out_button_press_event(GtkWidget *unused_widget,
 	GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -106,11 +107,11 @@ on_progressbar_bws_out_button_press_event(GtkWidget *unused_widget,
 }
 
 gboolean
-on_progressbar_bws_gin_button_press_event(GtkWidget *unused_widget, 
+on_progressbar_bws_gin_button_press_event(GtkWidget *unused_widget,
 	GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -120,11 +121,11 @@ on_progressbar_bws_gin_button_press_event(GtkWidget *unused_widget,
 }
 
 gboolean
-on_progressbar_bws_gout_button_press_event(GtkWidget *unused_widget, 
+on_progressbar_bws_gout_button_press_event(GtkWidget *unused_widget,
 	GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -134,11 +135,11 @@ on_progressbar_bws_gout_button_press_event(GtkWidget *unused_widget,
 }
 
 gboolean
-on_progressbar_bws_lin_button_press_event(GtkWidget *unused_widget, 
+on_progressbar_bws_lin_button_press_event(GtkWidget *unused_widget,
 	GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -152,7 +153,7 @@ on_progressbar_bws_lout_button_press_event(GtkWidget *unused_widget,
 		GdkEventButton *unused_event, gpointer unused_udata)
 {
     gboolean val;
-    
+
 	(void) unused_widget;
 	(void) unused_event;
 	(void) unused_udata;
@@ -198,7 +199,7 @@ on_button_hostcache_clear_bad_clicked(GtkButton *unused_button,
 
 /***
  *** Search Stats
- ***/ 
+ ***/
 
 void
 on_button_search_stats_reset_clicked(GtkButton *unused_button,
@@ -220,7 +221,7 @@ on_clist_search_stats_resize_column(GtkCList *unused_clist, gint column,
 
 /***
  *** Config pane
- ***/ 
+ ***/
 
 
 /* While downloading, store files to */
@@ -252,9 +253,8 @@ button_fs_save_path_clicked(GtkButton *unused_button, gpointer user_data)
 
 		if (is_directory(name)) {
             gnet_prop_set_string(PROP_SAVE_FILE_PATH, name);
-        } else {
-            G_FREE_NULL(name);
-        }
+		}
+        G_FREE_NULL(name);
 	}
 
 	gtk_widget_destroy(save_path_filesel);
@@ -320,13 +320,12 @@ button_fs_move_path_clicked(GtkButton *unused_button, gpointer user_data)
 
 		if (is_directory(name)) {
             gnet_prop_set_string(PROP_MOVE_FILE_PATH, name);
-        } else {
-            G_FREE_NULL(name);
         }
+        G_FREE_NULL(name);
 	}
 
 	gtk_widget_destroy(move_path_filesel);
-	move_path_filesel = (GtkWidget *) NULL;
+	move_path_filesel = NULL;
 }
 
 void
@@ -378,7 +377,7 @@ void
 button_fs_bad_path_clicked(GtkButton *unused_button, gpointer user_data)
 {
 	(void) unused_button;
-	
+
 	if (user_data) {
 		gchar *name;
 
@@ -387,13 +386,12 @@ button_fs_bad_path_clicked(GtkButton *unused_button, gpointer user_data)
 
 		if (is_directory(name)) {
             gnet_prop_set_string(PROP_BAD_FILE_PATH, name);
-        } else {
-            G_FREE_NULL(name);
         }
+        G_FREE_NULL(name);
 	}
 
 	gtk_widget_destroy(bad_path_filesel);
-	bad_path_filesel = (GtkWidget *) NULL;
+	bad_path_filesel = NULL;
 }
 
 void
@@ -402,7 +400,7 @@ on_button_config_bad_path_clicked(GtkButton *unused_button,
 {
 	(void) unused_button;
 	(void) unused_udata;
-	
+
 	if (!bad_path_filesel) {
 		bad_path_filesel =
 			gtk_file_selection_new(
@@ -444,10 +442,10 @@ void
 button_fs_add_dir_clicked(GtkButton *unused_button, gpointer user_data)
 {
 	(void) unused_button;
-	
+
 	if (user_data) {
 		gchar *name;
-     
+
         name = g_strdup(gtk_file_selection_get_filename
             (GTK_FILE_SELECTION(add_dir_filesel)));
 
@@ -467,7 +465,7 @@ on_button_config_add_dir_clicked(GtkButton *unused_button,
 {
 	(void) unused_button;
 	(void) unused_udata;
-	
+
 	if (!add_dir_filesel) {
 		add_dir_filesel =
 			gtk_file_selection_new(_("Please choose a directory to share"));
@@ -495,7 +493,7 @@ on_button_config_rescan_dir_clicked(GtkButton *unused_button,
 {
 	(void) unused_button;
 	(void) unused_udata;
-	
+
 	gui_allow_rescan_dir(FALSE);
 	guc_share_scan();
 	gui_allow_rescan_dir(TRUE);
@@ -506,7 +504,7 @@ void
 on_entry_config_netmask_activate(GtkEditable *editable, gpointer unused_data)
 {
     gchar *buf;
-    
+
 	(void) unused_data;
     buf = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
     gnet_prop_set_string(PROP_LOCAL_NETMASKS_STRING, buf);
@@ -526,108 +524,24 @@ on_clist_search_resize_column(GtkCList *unused_clist, gint column,
     *(gint *) &search_list_col_widths[column] = width;
 }
 
-#ifdef USE_GTK2
+void
+on_popup_search_browse_host_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_menuitem;
+	(void) unused_udata;
+
+	search_gui_browse_selected();
+}
 
 void
-on_hb_searches_child_detached(GtkHandleBox *unused_handlebox,
-	GtkWidget *widget, gpointer unused_udata)
+on_popup_search_sort_defaults_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
 {
-	gint width, height;
-
-	(void) unused_handlebox;
+	(void) unused_menuitem;
 	(void) unused_udata;
 	
-	/*
-	 * Before the GtkHandleBox is detached, request the current size
-	 * of the widget inside to preserve dimensions. Otherwise, the
-	 * detached GtkHandleBox becomes pretty small.
-	 */
-
-	width = widget->allocation.width;
-	height = widget->allocation.height;
-	gtk_widget_set_size_request(widget, width, height);
+	search_gui_set_sort_defaults();
 }
 
-void
-on_menu_wizard_connection_unknown_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_t3_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_t1_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_sdsl_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_adsl_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_isdn128_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_isdn64_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-void
-on_menu_wizard_connection_modem_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	/* TODO: Implement */
-	(void) unused_menuitem;
-	(void) unused_udata;
-}
-
-
-#endif /* USE_GTK2 */
 /* vi: set ts=4 sw=4 cindent: */
