@@ -1,10 +1,8 @@
 /*
- * $Id: fileinfo.c,v 1.4 2004/11/13 09:45:23 cbiere Exp $
+ * $Id: fileinfo.c,v 1.23 2006/02/06 14:35:54 cbiere Exp $
  *
  * Copyright (c) 2003, Richard Eckart
  *
- * Displaying of file information in the gui.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Displaying of file information in the GUI.
+ *
+ * @author Richard Eckart
+ * @date 2003
+ */
+
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c,v 1.4 2004/11/13 09:45:23 cbiere Exp $");
+RCSID("$Id: fileinfo.c,v 1.23 2006/02/06 14:35:54 cbiere Exp $");
 
 #include "gtk/filter.h"
 #include "gtk/statusbar.h"
@@ -39,20 +47,116 @@ RCSID("$Id: fileinfo.c,v 1.4 2004/11/13 09:45:23 cbiere Exp $");
 #include "if/bridge/ui2c.h"
 
 #include "lib/glib-missing.h"
+#include "lib/url.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 static gnet_fi_t last_shown = 0;
 static gboolean  last_shown_valid = FALSE;
+static GHashTable *fi_updates = NULL;
 
-/* 
+/*
  * Together visible_fi and hidden_fi are a list of all fileinfo handles
- * the the gui knows about. 
+ * the the gui knows about.
  */
 static GSList *visible_fi = NULL;
 static GSList *hidden_fi  = NULL;
 
 static regex_t filter_re;
 
+static GtkCList *clist_fileinfo = NULL;		/* Cached lookup_widget() */
+
+static void
+drag_begin(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
+	gpointer udata)
+{
+	gchar **url_ptr = udata;
+
+	(void) unused_widget;
+	(void) unused_drag_ctx;
+
+	g_assert(url_ptr != NULL);
+	G_FREE_NULL(*url_ptr);
+	
+	if (last_shown_valid) {
+		gnet_fi_info_t *fi = NULL;
+    	gnet_fi_status_t fis;
+
+    	guc_fi_get_status(last_shown, &fis);
+
+		/* Allow partials but not unstarted files */
+		if (fis.done > 0) {
+			const gchar *path;
+			gchar *save_path = NULL;
+
+			fi = guc_fi_get_info(last_shown);
+			g_assert(fi != NULL);
+			if (fis.done < fis.size) {
+				path = fi->path;
+			} else {
+				/* XXX: This is a hack since the final destination might
+				 *		might be different e.g., due to a filename clash
+				 *		or because the PROP_MOVE_FILE_PATH changed in the
+				 *		meantime. */
+				save_path = gnet_prop_get_string(PROP_MOVE_FILE_PATH, NULL, 0);
+				path = save_path;
+			}
+
+			if (path && fi->file_name) {
+				gchar *escaped;
+				gchar *pathname;
+
+				pathname = make_pathname(path, fi->file_name);
+				escaped = url_escape(pathname);
+				if (escaped != pathname)
+					G_FREE_NULL(pathname);
+
+				*url_ptr = g_strconcat("file://", escaped, (void *) 0);
+
+				G_FREE_NULL(escaped);
+			}
+			G_FREE_NULL(save_path);
+
+    		guc_fi_free_info(fi);
+		}
+	}
+}
+
+static void
+drag_data_get(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
+	GtkSelectionData *data, guint unused_info, guint unused_stamp,
+	gpointer udata)
+{
+	gchar **url_ptr = udata;
+
+	(void) unused_widget;
+	(void) unused_drag_ctx;
+	(void) unused_info;
+	(void) unused_stamp;
+
+	g_assert(url_ptr != NULL);
+	if (*url_ptr) {
+		const gchar *drag_data = *url_ptr;
+		
+    	gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
+			8 /* CHAR_BIT */, cast_to_gconstpointer(drag_data),
+			strlen(drag_data));
+		G_FREE_NULL(*url_ptr);
+	}
+}
+
+static void
+drag_end(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
+	gpointer udata)
+{
+	gchar **url_ptr = udata;
+
+	(void) unused_widget;
+	(void) unused_drag_ctx;
+
+	g_assert(url_ptr != NULL);
+	G_FREE_NULL(*url_ptr);
+}
+
 void
 on_clist_fileinfo_resize_column(GtkCList *unused_clist,
 	gint column, gint width, gpointer unused_udata)
@@ -66,25 +170,24 @@ on_clist_fileinfo_resize_column(GtkCList *unused_clist,
  * when fi_gui_shutdown is called. */
 static gnet_fi_info_t *last_fi = NULL;
 
-/*
- * fi_gui_fill_info:
- *
+/**
  * Fill in the cell data. Calling this will always break the data
  * it filled in last time!
  *
- * WARNING: returns pointer to global data: the gnet_fi_info_t structure
+ * @warning
+ * Returns pointer to global data: the gnet_fi_info_t structure
  * filled from the given `fih'.
  */
 static gnet_fi_info_t *
 fi_gui_fill_info(gnet_fi_t fih, gchar *titles[c_fi_num])
 {
     /* Clear info from last call. We keep this around so we don't
-     * have to strdup entries from it when passing them to the 
+     * have to strdup entries from it when passing them to the
      * outside through titles[]. */
 
     if (last_fi != NULL)
         guc_fi_free_info(last_fi);
-        
+
     /* Fetch new info */
     last_fi = guc_fi_get_info(fih);
     g_assert(last_fi != NULL);
@@ -94,6 +197,7 @@ fi_gui_fill_info(gnet_fi_t fih, gchar *titles[c_fi_num])
 	return last_fi;
 }
 
+/* XXX -- factorize this code with GTK2's one */
 static void
 fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 {
@@ -110,27 +214,60 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
     titles[c_fi_sources] = fi_sources;
 
     if (s.done) {
-        gm_snprintf(fi_done, sizeof(fi_done), "%s (%.1f%%)", 
+        gm_snprintf(fi_done, sizeof(fi_done), "%s (%.1f%%)",
             short_size(s.done), ((float) s.done / s.size)*100.0);
         titles[c_fi_done] = fi_done;
     } else {
         titles[c_fi_done] = "-";
     }
-        
+
     g_strlcpy(fi_size, short_size(s.size), sizeof(fi_size));
     titles[c_fi_size]    = fi_size;
 
     if (s.recvcount) {
-        gm_snprintf(fi_status, sizeof(fi_status), 
-            _("Downloading (%.1f k/s)"), s.recv_last_rate / 1024.0);
+		guint32 secs = 0;
+
+		if (s.recv_last_rate)
+			secs = (s.size - s.done) / s.recv_last_rate;
+
+        gm_snprintf(fi_status, sizeof(fi_status),
+            _("Downloading (%s)  TR: %s"),
+			short_rate(s.recv_last_rate),
+			secs ? short_time(secs) : "-");
+
+        titles[c_fi_status] = fi_status;
+    } else if (s.size && s.done == s.size){
+		gint rw;
+
+		rw = gm_snprintf(fi_status, sizeof(fi_status),
+				"%s", _("Finished"));
+
+		if (s.has_sha1) {
+			if (s.sha1_hashed == s.size)
+				rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
+						"; SHA1 %s", s.sha1_matched ? _("OK") : _("failed"));
+			else if (s.sha1_hashed == 0)
+				rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
+						"; %s", _("Waiting for SHA1 check"));
+			else
+				rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
+						"; %s %s (%.1f%%)", _("Computing SHA1"),
+						short_size(s.sha1_hashed),
+						((float) s.sha1_hashed / s.size) * 100.0);
+		}
+
+		if (s.copied > 0 && s.copied < s.size) 
+			rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
+					"; %s %s (%.1f%%)", _("Moving"),
+					short_size(s.copied),
+					((float) s.copied / s.size) * 100.0);
+
         titles[c_fi_status] = fi_status;
-    } else if (s.done == s.size){
-        titles[c_fi_status] = _("Finished");
     } else if (s.lifecount == 0) {
         titles[c_fi_status] = _("No sources");
     } else if (s.aqueued_count || s.pqueued_count) {
-        gm_snprintf(fi_status, sizeof(fi_status), 
-            _("Queued (%d active/ %d passive)"),
+        gm_snprintf(fi_status, sizeof(fi_status),
+            _("Queued (%d active, %d passive)"),
             s.aqueued_count, s.pqueued_count);
         titles[c_fi_status] = fi_status;
     } else {
@@ -151,7 +288,7 @@ fi_gui_set_details(gnet_fi_t fih)
     gchar **aliases;
     guint n;
     GtkCList *cl_aliases;
-	gboolean in_progress = FALSE;
+	gchar bytes[32];
 
     fi = guc_fi_get_info(fih);
     g_assert(fi != NULL);
@@ -164,9 +301,10 @@ fi_gui_set_details(gnet_fi_t fih)
     gtk_label_set_text(
         GTK_LABEL(lookup_widget(main_window, "label_fi_filename")),
         fi->file_name);
+	uint64_to_string_buf(fis.size, bytes, sizeof bytes);
     gtk_label_printf(
         GTK_LABEL(lookup_widget(main_window, "label_fi_size")),
-        _("%s (%" PRIu64 " bytes)"), short_size(fis.size), (guint64) fis.size);
+        _("%s (%s bytes)"), short_size(fis.size), bytes);
 
     gtk_clist_freeze(cl_aliases);
     gtk_clist_clear(cl_aliases);
@@ -174,8 +312,6 @@ fi_gui_set_details(gnet_fi_t fih)
         gtk_clist_append(cl_aliases, &aliases[n]);
     gtk_clist_thaw(cl_aliases);
 
-	in_progress = (fis.done != fis.size);
-    
     g_strfreev(aliases);
     guc_fi_free_info(fi);
 
@@ -185,7 +321,7 @@ fi_gui_set_details(gnet_fi_t fih)
 	vp_draw_fi_progress(last_shown_valid, last_shown);
 
 	gtk_widget_set_sensitive(lookup_widget(main_window, "button_fi_purge"),
-							 in_progress);
+							 TRUE);
 }
 
 /**
@@ -223,7 +359,7 @@ fi_gui_match_filter(const gchar *s)
 
     if (n == REG_ESPACE) {
         g_warning("fi_gui_match_filter: regexp memory overflow");
-    } 
+    }
 
     return n == 0;
 }
@@ -237,7 +373,7 @@ fi_gui_match_filter(const gchar *s)
 static void
 fi_gui_add_row(gnet_fi_t fih)
 {
-    GtkCList *clist;
+    GtkCList *clist = clist_fileinfo;
     gint row;
     guint n;
     gchar *titles[c_fi_num];
@@ -256,7 +392,7 @@ fi_gui_add_row(gnet_fi_t fih)
 	filter_match = fi_gui_match_filter(info->file_name);
 
 	for (l = info->aliases; !filter_match && l; l = g_slist_next(l)) {
-		const gchar *alias = (const gchar *) l->data;
+		const gchar *alias = l->data;
 		filter_match = fi_gui_match_filter(alias);
 	}
 
@@ -273,8 +409,6 @@ fi_gui_add_row(gnet_fi_t fih)
 
     fi_gui_fill_status(fih, titles);
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
-
     for (n = 0; n < G_N_ELEMENTS(titles); n ++) {
         if (titles[n] == NULL)
             titles[n] = "";
@@ -292,11 +426,9 @@ fi_gui_add_row(gnet_fi_t fih)
 static void
 fi_gui_remove_row(gnet_fi_t fih, gboolean hide)
 {
-    GtkCList *clist;
+    GtkCList *clist = clist_fileinfo;
     gint row;
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
-
     row = gtk_clist_find_row_from_data(clist, GUINT_TO_POINTER(fih));
     gtk_clist_remove(clist, row);
 
@@ -321,12 +453,12 @@ fi_gui_set_filter_regex(gchar *s)
     gint row;
     GSList *old_hidden = g_slist_copy(hidden_fi);
     GtkCList *clist_fi;
-    char *fallback_re = ".";	
+    char *fallback_re = ".";
 
     if (s == NULL) {
         s = fallback_re;
     }
- 
+
     /* Recompile the row filter*/
     err = regcomp(&filter_re, s,
                   REG_EXTENDED|REG_NOSUB|(fi_regex_case ? 0 : REG_ICASE));
@@ -342,8 +474,7 @@ fi_gui_set_filter_regex(gchar *s)
         g_assert(!err);
     }
 
-    clist_fi = GTK_CLIST(
-        lookup_widget(main_window, "clist_fileinfo"));
+    clist_fi = clist_fileinfo;
 
     /* now really apply the filter */
     gtk_clist_unselect_all(clist_fi);
@@ -367,27 +498,25 @@ fi_gui_set_filter_regex(gchar *s)
             row ++;
         }
     }
-		
+
     /* now add matching hidden to list */
     for (sl = old_hidden; NULL != sl; sl = g_slist_next(sl)) {
         /* We simply try to add all hidden rows. If they match
          * the new filter they will be unhidden */
         fi_gui_add_row(GPOINTER_TO_UINT(sl->data));
     }
-    
+
     gtk_clist_thaw(clist_fi);
 }
 
 static void
 fi_gui_update(gnet_fi_t fih, gboolean full)
 {
-    GtkCList *clist;
+    GtkCList *clist = clist_fileinfo;
 	gchar    *titles[c_fi_num];
     gint      row;
     guint     n;
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
-
     row = gtk_clist_find_row_from_data(clist, GUINT_TO_POINTER(fih));
     if (row == -1) {
         /* This can happen if we get an update event for a hidden row. */
@@ -404,10 +533,12 @@ fi_gui_update(gnet_fi_t fih, gboolean full)
             gtk_clist_set_text(clist, row, n, titles[n]);
     }
 
-    /* 
+    /*
      * If this entry is currently selected we should also update the progress
      */
-	vp_draw_fi_progress(last_shown_valid, last_shown);
+
+	if (fih == last_shown)
+		vp_draw_fi_progress(last_shown_valid, last_shown);
 }
 
 static void
@@ -419,13 +550,44 @@ fi_gui_fi_added(gnet_fi_t fih)
 static void
 fi_gui_fi_removed(gnet_fi_t fih)
 {
+	g_hash_table_remove(fi_updates, GUINT_TO_POINTER(fih));
+	if (fih == last_shown)
+		last_shown_valid = FALSE;
+
     fi_gui_remove_row(fih, FALSE);
 }
 
 static void
 fi_gui_fi_status_changed(gnet_fi_t fih)
 {
-    fi_gui_update(fih, FALSE);
+	/*
+	 * Buffer update, delaying GUI refresh.
+	 */
+
+	g_hash_table_insert(fi_updates, GUINT_TO_POINTER(fih), GINT_TO_POINTER(1));
+}
+
+static void
+fi_gui_fi_status_changed_transient(gnet_fi_t fih)
+{
+	if (fih == last_shown)
+		fi_gui_fi_status_changed(fih);
+}
+
+/**
+ * Hash table iterator to update the display for each queued entry.
+ */
+/* XXX -- move to new fileinfo_common.c */
+static gboolean
+fi_gui_update_queued(gpointer key, gpointer unused_value, gpointer unused_udata)
+{
+	gnet_fi_t fih = GPOINTER_TO_UINT(key);
+
+	(void) unused_value;
+	(void) unused_udata;
+
+	fi_gui_update(fih, FALSE);
+	return TRUE;	/* Remove the handle from the hashtable */
 }
 
 void
@@ -433,7 +595,7 @@ on_clist_fileinfo_select_row(GtkCList *clist, gint row, gint unused_column,
     GdkEvent *unused_event, gpointer unused_udata)
 {
     gnet_fi_t fih;
-    
+
 	(void) unused_column;
 	(void) unused_event;
 	(void) unused_udata;
@@ -444,7 +606,7 @@ on_clist_fileinfo_select_row(GtkCList *clist, gint row, gint unused_column,
 void
 on_clist_fileinfo_unselect_row(GtkCList *clist, gint unused_row,
 	gint unused_column, GdkEvent *unused_event, gpointer unused_udata)
-{ 
+{
 	(void) unused_row;
 	(void) unused_column;
 	(void) unused_event;
@@ -456,18 +618,25 @@ on_clist_fileinfo_unselect_row(GtkCList *clist, gint unused_row,
 void
 on_button_fi_purge_clicked(GtkButton *unused_button, gpointer unused_udata)
 {
-    GSList *sl = NULL;
-    GtkCList *clist = GTK_CLIST(
-        lookup_widget(main_window, "clist_fileinfo"));
-		
+    GSList *sl_handles = NULL;
+    GtkCList *clist = clist_fileinfo;
+
 	(void) unused_button;
 	(void) unused_udata;
-    sl = clist_collect_data(clist, TRUE, NULL);
-    if (sl) {
-        guc_fi_purge_by_handle_list(sl);
+    sl_handles = clist_collect_data(clist, TRUE, NULL);
+    if (sl_handles) {
+		GSList *sl;
+
+		for (sl = sl_handles; sl != NULL; sl = g_slist_next(sl))
+			if (GPOINTER_TO_UINT(sl->data) == last_shown) {
+				last_shown_valid = FALSE;
+				break;
+			}
+
+        guc_fi_purge_by_handle_list(sl_handles);
     }
-				    
-    g_slist_free(sl);
+
+    g_slist_free(sl_handles);
 }
 
 void
@@ -484,24 +653,44 @@ on_entry_fi_regex_activate(GtkEditable *editable, gpointer unused_udata)
 }
 
 void
-fi_gui_init(void) 
+fi_gui_init(void)
 {
-    GtkCList *clist;
+    static const GtkTargetEntry targets[] = {
+        { "STRING", 0, 23 },
+        { "text/plain", 0, 23 },
+    };
+	static gchar *dnd_url; /* Holds the URL to set the drag data */
+
+	fi_updates = g_hash_table_new(NULL, NULL);
 
     guc_fi_add_listener(fi_gui_fi_added, EV_FI_ADDED, FREQ_SECS, 0);
     guc_fi_add_listener(fi_gui_fi_removed, EV_FI_REMOVED, FREQ_SECS, 0);
     guc_fi_add_listener(fi_gui_fi_status_changed, EV_FI_STATUS_CHANGED,
 		FREQ_SECS, 0);
+    guc_fi_add_listener(fi_gui_fi_status_changed_transient,
+		EV_FI_STATUS_CHANGED_TRANSIENT, FREQ_SECS, 0);
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
+    clist_fileinfo = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
 
-    gtk_clist_set_column_justification(clist, 
+    gtk_clist_set_column_justification(clist_fileinfo,
         c_fi_size, GTK_JUSTIFY_RIGHT);
 
-    gtk_clist_column_titles_passive(clist);
+    gtk_clist_column_titles_passive(clist_fileinfo);
 
     /* Initialize the row filter */
     fi_gui_set_filter_regex(NULL);
+	
+	/* Initialize drag support */
+	gtk_drag_source_set(GTK_WIDGET(clist_fileinfo),
+		GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, targets, G_N_ELEMENTS(targets),
+		GDK_ACTION_DEFAULT | GDK_ACTION_COPY | GDK_ACTION_ASK);
+
+    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-data-get",
+        drag_data_get, &dnd_url);
+    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-begin",
+        drag_begin, &dnd_url);
+    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-end",
+        drag_end, &dnd_url);
 }
 
 void
@@ -517,6 +706,7 @@ fi_gui_shutdown(void)
     if (last_fi != NULL)
         guc_fi_free_info(last_fi);
 
+	g_hash_table_destroy(fi_updates);
     regfree(&filter_re);
 }
 
@@ -524,7 +714,9 @@ fi_gui_shutdown(void)
  * Update all the fileinfo at the same time.
  */
 
-/* FIXME: we should remember for every node when it was last
+/**
+ * @bug
+ * FIXME: We should remember for every node when it was last
  *        updated and only refresh every node at most once every
  *        second. This information should be kept in a struct pointed
  *        to by the row user_data and should be automatically freed
@@ -533,9 +725,13 @@ fi_gui_shutdown(void)
 void
 fi_gui_update_display(time_t now)
 {
+    gtk_clist_freeze(clist_fileinfo);
+	g_hash_table_foreach_remove(fi_updates, fi_gui_update_queued, NULL);
+    gtk_clist_thaw(clist_fileinfo);
+
 #if 0
     static time_t last_update = 0;
-	GtkCList *clist;
+	GtkCList *clist = clist_fileinfo;
 	GList *l;
 	gint row = 0;
 
@@ -544,7 +740,6 @@ fi_gui_update_display(time_t now)
 
     last_update = now;
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
     gtk_clist_freeze(clist);
 
 	for (l = clist->row_list, row = 0; l; l = l->next, row++) {
