@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c,v 1.6 2004/11/13 09:45:23 cbiere Exp $
+ * $Id: gnet_stats.c,v 1.13 2005/11/12 11:29:52 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c,v 1.6 2004/11/13 09:45:23 cbiere Exp $");
+RCSID("$Id: gnet_stats.c,v 1.13 2005/11/12 11:29:52 cbiere Exp $");
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
@@ -37,10 +37,9 @@ RCSID("$Id: gnet_stats.c,v 1.6 2004/11/13 09:45:23 cbiere Exp $");
 #include "if/bridge/ui2c.h"
 
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-static gint selected_type = MSG_TOTAL;
-
 /***
  *** Callbacks
  ***/
@@ -53,7 +52,7 @@ on_clist_gnet_stats_msg_resize_column(GtkCList *unused_clist, gint column,
 
 	(void) unused_clist;
 	(void) unused_udata;
-	
+
     if (lock)
         return;
 
@@ -77,7 +76,7 @@ on_clist_gnet_stats_fc_ttl_resize_column(GtkCList *unused_clist,
 	(void) unused_clist;
 	(void) unused_column;
 	(void) unused_udata;
-	
+
     if (lock)
         return;
 
@@ -103,7 +102,7 @@ on_clist_gnet_stats_fc_hops_resize_column(GtkCList *unused_clist,
 	(void) unused_clist;
 	(void) unused_column;
 	(void) unused_udata;
-	
+
     if (lock)
         return;
 
@@ -127,7 +126,7 @@ on_clist_gnet_stats_horizon_resize_column(GtkCList *unused_clist,
 
 	(void) unused_clist;
 	(void) unused_udata;
-	
+
     if (lock)
         return;
 
@@ -147,9 +146,9 @@ on_clist_gnet_stats_drop_reasons_resize_column(GtkCList *unused_clist,
 
 	(void) unused_clist;
 	(void) unused_udata;
-	
+
     /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS, 
+    gui_prop_set_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
         &buf, column, 1);
 }
 
@@ -161,22 +160,12 @@ on_clist_gnet_stats_general_resize_column(GtkCList *unused_clist, gint column,
 
 	(void) unused_clist;
 	(void) unused_udata;
-	
+
     /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS, 
+    gui_prop_set_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS,
         &buf, column, 1);
 }
 
-static void
-on_gnet_stats_type_selected(GtkItem *unused_item, gpointer data)
-{
-	(void) unused_item;
-
-    selected_type = GPOINTER_TO_INT(data);
-    gnet_stats_gui_update(time(NULL));
-}
-
-
 /***
  *** Private functions
  ***/
@@ -184,16 +173,16 @@ on_gnet_stats_type_selected(GtkItem *unused_item, gpointer data)
 const gchar *
 pkt_stat_str(const guint64 *val_tbl, gint type)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
 
     if (val_tbl[type] == 0)
         return gnet_stats_perc ? "-  " : "-";
 
     if (gnet_stats_perc)
-        gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%", 
+        gm_snprintf(strbuf, sizeof strbuf, "%.2f%%",
             (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
     else
-        gm_snprintf(strbuf, sizeof(strbuf), "%" PRIu64, val_tbl[type]);
+        uint64_to_string_buf(val_tbl[type], strbuf, sizeof strbuf);
 
     return strbuf;
 }
@@ -202,7 +191,7 @@ pkt_stat_str(const guint64 *val_tbl, gint type)
 const gchar *
 byte_stat_str(const guint64 *val_tbl, const guint64 *nb_packets, gint type)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
 	guint64 size = val_tbl[type];
 
     if (val_tbl[type] == 0)
@@ -215,7 +204,7 @@ byte_stat_str(const guint64 *val_tbl, const guint64 *nb_packets, gint type)
 		guint64 total_size = val_tbl[MSG_TOTAL];
 		if (!gnet_stats_with_headers)
 			size -= nb_packets[MSG_TOTAL] * GTA_HEADER_SIZE;
-        gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%", 
+        gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%",
             (gfloat) size / total_size * 100.0);
         return strbuf;
     } else
@@ -225,18 +214,19 @@ byte_stat_str(const guint64 *val_tbl, const guint64 *nb_packets, gint type)
 const gchar *
 drop_stat_str(const gnet_stats_t *stats, gint reason)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
     guint32 total = stats->pkg.dropped[MSG_TOTAL];
+	guint i = gnet_stats_drop_reasons_type;
 
-    if (stats->drop_reason[reason][selected_type] == 0)
-        return gnet_stats_drop_perc ? "-  " : "-";
+    if (stats->drop_reason[reason][i] == 0)
+        return gnet_stats_perc ? "-  " : "-";
 
-    if (gnet_stats_drop_perc)
-        gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%", 
-            (gfloat) stats->drop_reason[reason][selected_type] / total * 100);
+    if (gnet_stats_perc)
+        gm_snprintf(strbuf, sizeof strbuf, "%.2f%%",
+            (gfloat) stats->drop_reason[reason][i] / total * 100);
     else
-        gm_snprintf(strbuf, sizeof(strbuf), "%" PRIu64, 
-            stats->drop_reason[reason][selected_type]);
+        uint64_to_string_buf(stats->drop_reason[reason][i],
+			strbuf, sizeof strbuf);
 
     return strbuf;
 }
@@ -244,7 +234,7 @@ drop_stat_str(const gnet_stats_t *stats, gint reason)
 const gchar *
 general_stat_str(const gnet_stats_t *stats, gint type)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
 
     if (stats->general[type] == 0)
         return "-";
@@ -252,7 +242,7 @@ general_stat_str(const gnet_stats_t *stats, gint type)
     if (type == GNR_QUERY_COMPACT_SIZE) {
         return compact_size(stats->general[type]);
     } else {
-        gm_snprintf(strbuf, sizeof(strbuf), "%" PRIu64, stats->general[type]);
+        uint64_to_string_buf(stats->general[type], strbuf, sizeof strbuf);
         return strbuf;
     }
 }
@@ -260,16 +250,16 @@ general_stat_str(const gnet_stats_t *stats, gint type)
 const gchar *
 flowc_stat_str_pkg(const guint64 *val_tbl, gint type)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
 
     if (val_tbl[type] == 0)
         return gnet_stats_perc ? "-  " : "-";
 
 	if (gnet_stats_perc) {
-		gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%", 
-            (gfloat) val_tbl[type]/val_tbl[MSG_TOTAL] * 100.0);
+		gm_snprintf(strbuf, sizeof strbuf, "%.2f%%",
+            (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
     } else {
-       	gm_snprintf(strbuf, sizeof(strbuf), "%" PRIu64, val_tbl[type]);
+       	uint64_to_string_buf(val_tbl[type], strbuf, sizeof strbuf);
     }
 
     return strbuf;
@@ -278,13 +268,13 @@ flowc_stat_str_pkg(const guint64 *val_tbl, gint type)
 const gchar *
 flowc_stat_str_byte(const guint64 *val_tbl, gint type)
 {
-    static gchar strbuf[21];
+    static gchar strbuf[UINT64_DEC_BUFLEN];
 
     if (val_tbl[type] == 0)
         return gnet_stats_perc ? "-  " : "-";
 
 	if (gnet_stats_perc) {
-		gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%", 
+		gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%",
             (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
     } else {
        	return compact_size(val_tbl[type]);
@@ -306,7 +296,6 @@ gnet_stats_gui_init(void)
     GtkCList *clist_general;
     GtkCList *clist_reason;
     GtkCList *clist_horizon;
-    GtkCombo *combo_types;
     gchar *titles[10];
     guint n;
 
@@ -325,8 +314,6 @@ gnet_stats_gui_init(void)
         lookup_widget(main_window, "clist_gnet_stats_general"));
     clist_horizon = GTK_CLIST(
         lookup_widget(main_window, "clist_gnet_stats_horizon"));
-    combo_types = GTK_COMBO(
-        lookup_widget(main_window, "combo_gnet_stats_type"));
 
     /*
      * Set column justification for numeric columns to GTK_JUSTIFY_RIGHT.
@@ -345,7 +332,7 @@ gnet_stats_gui_init(void)
         gtk_clist_set_column_justification(
             clist_stats_msg, n, GTK_JUSTIFY_RIGHT);
     }
-    
+
     for (n = 1; n < 10; n ++) {
         gtk_clist_set_column_justification(
             clist_stats_fc_ttl, n, GTK_JUSTIFY_RIGHT);
@@ -368,8 +355,6 @@ gnet_stats_gui_init(void)
      * Initialize stats tables.
      */
     for (n = 0; n < MSG_TYPE_COUNT; n ++) {
-        GtkWidget *list_item;
-        GList *l;
         gint row;
 
         titles[0] = (gchar *) msg_type_str(n);
@@ -380,22 +365,6 @@ gnet_stats_gui_init(void)
         gtk_clist_set_selectable(clist_stats_fc_ttl, row, FALSE);
         row = gtk_clist_append(clist_stats_fc_hops, titles);
         gtk_clist_set_selectable(clist_stats_fc_hops, row, FALSE);
-
-        list_item = gtk_list_item_new_with_label(msg_type_str(n));
-
-        gtk_widget_show(list_item);
-
-        gtk_signal_connect(
-            GTK_OBJECT(list_item), "select",
-            GTK_SIGNAL_FUNC(on_gnet_stats_type_selected),
-            GINT_TO_POINTER(n));
-
-        l = g_list_prepend(NULL, (gpointer) list_item);
-        gtk_list_append_items(GTK_LIST(GTK_COMBO(combo_types)->list), l);
-
-        if (n == MSG_TOTAL)
-            gtk_list_select_child(
-                GTK_LIST(GTK_COMBO(combo_types)->list), list_item);
     }
 
     for (n = 0; n < MSG_DROP_REASON_COUNT; n ++) {
@@ -498,46 +467,46 @@ gnet_stats_gui_update(time_t now)
     for (n = 0; n < MSG_TYPE_COUNT; n ++) {
         int m;
 
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_received, 
-            gnet_stats_bytes ? 
-                byte_stat_str(xstats->byte.received, xstats->pkg.received, n) : 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_received,
+            gnet_stats_bytes ?
+                byte_stat_str(xstats->byte.received, xstats->pkg.received, n) :
                 pkt_stat_str(xstats->pkg.received, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_gen_queued, 
-            gnet_stats_bytes ? 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_gen_queued,
+            gnet_stats_bytes ?
                 byte_stat_str(xstats->byte.gen_queued,
-					xstats->pkg.gen_queued, n) : 
+					xstats->pkg.gen_queued, n) :
                 pkt_stat_str(xstats->pkg.gen_queued, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_generated, 
-            gnet_stats_bytes ? 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_generated,
+            gnet_stats_bytes ?
                 byte_stat_str(xstats->byte.generated,
-					xstats->pkg.generated, n) : 
+					xstats->pkg.generated, n) :
                 pkt_stat_str(xstats->pkg.generated, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_dropped, 
-            gnet_stats_bytes ? 
-                byte_stat_str(xstats->byte.dropped, xstats->pkg.dropped, n) : 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_dropped,
+            gnet_stats_bytes ?
+                byte_stat_str(xstats->byte.dropped, xstats->pkg.dropped, n) :
                 pkt_stat_str(xstats->pkg.dropped, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_expired, 
-            gnet_stats_bytes ? 
-                byte_stat_str(xstats->byte.expired, xstats->pkg.expired, n) : 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_expired,
+            gnet_stats_bytes ?
+                byte_stat_str(xstats->byte.expired, xstats->pkg.expired, n) :
                 pkt_stat_str(xstats->pkg.expired, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_queued, 
-            gnet_stats_bytes ? 
-                byte_stat_str(xstats->byte.queued, xstats->pkg.queued, n) : 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_queued,
+            gnet_stats_bytes ?
+                byte_stat_str(xstats->byte.queued, xstats->pkg.queued, n) :
                 pkt_stat_str(xstats->pkg.queued, n));
-        gtk_clist_set_text(clist_stats_msg, n, c_gs_relayed, 
-            gnet_stats_bytes ? 
-                byte_stat_str(xstats->byte.relayed, xstats->pkg.relayed, n) : 
+        gtk_clist_set_text(clist_stats_msg, n, c_gs_relayed,
+            gnet_stats_bytes ?
+                byte_stat_str(xstats->byte.relayed, xstats->pkg.relayed, n) :
                 pkt_stat_str(xstats->pkg.relayed, n));
 
         for (m = 0; m < 9; m ++)
             gtk_clist_set_text(clist_stats_fc_ttl, n, m+1,
-                gnet_stats_bytes ? 
+                gnet_stats_bytes ?
                     flowc_stat_str_byte(stats.byte.flowc_ttl[m], n) :
                     flowc_stat_str_pkg(stats.pkg.flowc_ttl[m], n));
 
         for (m = 0; m < 9; m ++)
             gtk_clist_set_text(clist_stats_fc_hops, n, m+1,
-                gnet_stats_bytes ? 
+                gnet_stats_bytes ?
                     flowc_stat_str_byte(stats.byte.flowc_hops[m], n) :
                     flowc_stat_str_pkg(stats.pkg.flowc_hops[m], n));
     }
@@ -550,24 +519,24 @@ gnet_stats_gui_update(time_t now)
 
 	/*
 	 * Update horizon statistics table, but only if the values have changed.
-	 *		-- TNT 09/06/2004 
+	 *		-- TNT 09/06/2004
 	 *
 	 * Changed this check to update the table every 2 seconds, because not
 	 * only the HSEP table but also the PONG-based library sizes of direct
 	 * non-HSEP neighbors may have changed.
-	 *		-- TNT 14/06/2004 
+	 *		-- TNT 14/06/2004
 	 */
 
 	if (delta_time(now, last_horizon_update) >= 2) {
-		
+
 		gtk_clist_freeze(clist_horizon);
 
 		global_table_size = guc_hsep_get_table_size();
-		
+
 		for (n = 0; n < global_table_size; n ++) {
-			/* 
-			 * Note that we output hsep_table[1..global_table_size] 
-			 *		-- TNT 02/06/2004 
+			/*
+			 * Note that we output hsep_table[1..global_table_size]
+			 *		-- TNT 02/06/2004
 			 */
 			gtk_clist_set_text(clist_horizon, n, 1,
 			    horizon_stat_str(n + 1, 1));
