@@ -1,5 +1,5 @@
 /*
- * $Id: downloads_cb.c,v 1.2 2004/11/13 12:24:25 cbiere Exp $
+ * $Id: downloads_cb.c,v 1.15 2005/11/22 19:37:22 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: downloads_cb.c,v 1.2 2004/11/13 12:24:25 cbiere Exp $");
+RCSID("$Id: downloads_cb.c,v 1.15 2005/11/22 19:37:22 cbiere Exp $");
 
 #include "downloads_cb.h"
 #include "gtk/downloads.h"
@@ -33,10 +33,12 @@ RCSID("$Id: downloads_cb.c,v 1.2 2004/11/13 12:24:25 cbiere Exp $");
 #include "gtk/statusbar.h"
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/columns.h"
+#include "gtk/search_common.h"
 
 #include "if/gnet_property.h"
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
+#include "if/core/sockets.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -46,6 +48,7 @@ typedef enum {
 	DL_ACTION_ABORT_HOST,
 	DL_ACTION_ABORT_NAMED,
 	DL_ACTION_ABORT_SHA1,
+	DL_ACTION_BROWSE,
 	DL_ACTION_CONNECT,
 	DL_ACTION_COPY_URL,
 	DL_ACTION_PUSH,
@@ -59,9 +62,11 @@ typedef enum {
 	DL_ACTION_QUEUED_ABORT_NAMED,
 	DL_ACTION_QUEUED_ABORT_HOST,
 	DL_ACTION_QUEUED_ABORT_SHA1,
+	DL_ACTION_QUEUED_BROWSE,
 	DL_ACTION_QUEUED_CONNECT,
 	DL_ACTION_QUEUED_COPY_URL,
 	DL_ACTION_QUEUED_START,
+	DL_ACTION_QUEUED_PUSH,
 
 	NUM_DL_ACTION
 } dl_action_type_t;
@@ -90,6 +95,7 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 	case DL_ACTION_ABORT:
 	case DL_ACTION_ABORT_HOST:
 	case DL_ACTION_ABORT_NAMED:
+	case DL_ACTION_BROWSE:
 	case DL_ACTION_CONNECT:
 	case DL_ACTION_COPY_URL:
 	case DL_ACTION_PUSH:
@@ -100,8 +106,10 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 	/* Queued stuff goes here */
 	case DL_ACTION_QUEUED_ABORT_HOST:
 	case DL_ACTION_QUEUED_ABORT_NAMED:
+	case DL_ACTION_QUEUED_BROWSE:
 	case DL_ACTION_QUEUED_CONNECT:
 	case DL_ACTION_QUEUED_COPY_URL:
+	case DL_ACTION_QUEUED_PUSH:
 
 		switch (ctx->action) {
 		/* Active download stuff goes here */
@@ -110,6 +118,7 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 		case DL_ACTION_ABORT_NAMED:
 		case DL_ACTION_REMOVE_FILE:
 		case DL_ACTION_QUEUE:
+		case DL_ACTION_BROWSE:
 		case DL_ACTION_CONNECT:
 		case DL_ACTION_COPY_URL:
 		case DL_ACTION_ABORT:
@@ -122,9 +131,11 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 		case DL_ACTION_QUEUED_ABORT:
 		case DL_ACTION_QUEUED_ABORT_HOST:
 		case DL_ACTION_QUEUED_ABORT_NAMED:
+		case DL_ACTION_QUEUED_BROWSE:
 		case DL_ACTION_QUEUED_CONNECT:
 		case DL_ACTION_QUEUED_COPY_URL:
 		case DL_ACTION_QUEUED_START:
+		case DL_ACTION_QUEUED_PUSH:
 			column = c_queue_record;
 			break;
 
@@ -165,6 +176,7 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 		case DL_ACTION_ABORT_NAMED:
 		case DL_ACTION_REMOVE_FILE:
 		case DL_ACTION_QUEUE:
+		case DL_ACTION_BROWSE:
 		case DL_ACTION_CONNECT:
 		case DL_ACTION_COPY_URL:
 		case DL_ACTION_ABORT:
@@ -173,9 +185,11 @@ dl_action(GtkTreeModel *model, GtkTreePath *unused_path,
 		case DL_ACTION_QUEUED_ABORT:
 		case DL_ACTION_QUEUED_ABORT_HOST:
 		case DL_ACTION_QUEUED_ABORT_NAMED:
+		case DL_ACTION_QUEUED_BROWSE:
 		case DL_ACTION_QUEUED_CONNECT:
 		case DL_ACTION_QUEUED_COPY_URL:
 		case DL_ACTION_QUEUED_START:
+		case DL_ACTION_QUEUED_PUSH:
 		case NUM_DL_ACTION:
 			g_assert_not_reached();
 		}
@@ -225,7 +239,10 @@ dl_action_select(const gchar *treeview_name, dl_action_type_t action)
 {
 	GtkTreeView *tree_view;
 	GtkTreeSelection *selection;
-	dl_action_t ctx = { action, NULL };
+	dl_action_t ctx;
+
+	ctx.action = action;
+	ctx.sl = NULL;
 
 	g_assert((gint) action >= 0 && action <= NUM_DL_ACTION);
  	g_assert(treeview_name && treeview_name[0] != '\0');
@@ -234,7 +251,7 @@ dl_action_select(const gchar *treeview_name, dl_action_type_t action)
 	selection = gtk_tree_view_get_selection(tree_view);
 	gtk_tree_selection_selected_foreach(selection, dl_action, &ctx);
 	if (action != DL_ACTION_SELECT)
-		gtk_tree_selection_unselect_all(selection);	
+		gtk_tree_selection_unselect_all(selection);
 
 	return ctx.sl;
 }
@@ -248,31 +265,25 @@ static void
 show_removed(guint removed)
 {
     statusbar_gui_message(15,
-		removed == 1 ? _("Removed %u download") : _("Removed %u downloads"),
+		NG_("Removed %u download", "Removed %u downloads", removed),
 		removed);
 }
 
-/**
- *	Causes all selected active downloads to fall back to push
- */
-void
-on_popup_downloads_push_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
+static void
+push_activate(gboolean active)
 {
 	GSList *sl, *selected;
-	gboolean send_pushes;
-	gboolean firewalled;
+	gboolean send_pushes, firewalled;
 
-	(void) unused_menuitem;
-	(void) unused_udata;
+   	gnet_prop_get_boolean_val(PROP_SEND_PUSHES, &send_pushes);
+   	gnet_prop_get_boolean_val(PROP_IS_FIREWALLED, &firewalled);
 
-    gnet_prop_get_boolean_val(PROP_SEND_PUSHES, &send_pushes);
-    gnet_prop_get_boolean_val(PROP_IS_FIREWALLED, &firewalled);
+   	if (firewalled || !send_pushes)
+       	return;
 
-    if (firewalled || !send_pushes)
-        return;
-
-	selected = dl_action_select("treeview_downloads", DL_ACTION_PUSH);
+	selected = dl_action_select(
+					active ? "treeview_downloads" : "treeview_downloads_queue",
+					active ? DL_ACTION_PUSH : DL_ACTION_QUEUED_PUSH);
 	if (!selected)
 		return;
 
@@ -285,14 +296,94 @@ on_popup_downloads_push_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads, remove all downloads with the same name 
+ * Causes all selected active downloads to fall back to push.
+ */
+void
+on_popup_downloads_push_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_menuitem;
+	(void) unused_udata;
+
+	push_activate(TRUE);
+}
+
+/**
+ * Initiates a browse host request to the currently selected host.
+ */
+void
+on_popup_downloads_browse_host_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+	GSList *sl, *selected;
+
+	(void) unused_menuitem;
+	(void) unused_udata;
+
+   	selected = dl_action_select("treeview_downloads", DL_ACTION_BROWSE);
+	if (!selected)
+		return;
+
+	for (sl = selected; sl; sl = g_slist_next(sl)) {
+		const struct download *d = sl->data;
+   		search_gui_new_browse_host(
+			download_hostname(d), download_addr(d), download_port(d),
+			download_guid(d), FALSE, NULL);
+	}
+	g_slist_free(selected);
+}
+
+/**
+ * Initiates a browse host request to the currently selected host.
+ */
+void
+on_popup_queue_browse_host_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+	GSList *sl, *selected;
+
+	(void) unused_menuitem;
+	(void) unused_udata;
+
+   	selected = dl_action_select("treeview_downloads_queue",
+					DL_ACTION_QUEUED_BROWSE);
+	if (!selected)
+		return;
+
+	for (sl = selected; sl; sl = g_slist_next(sl)) {
+		const struct download *d = sl->data;
+   		search_gui_new_browse_host(
+			download_hostname(d), download_addr(d), download_port(d),
+			download_guid(d), FALSE, NULL);
+	}
+	g_slist_free(selected);
+}
+
+
+/**
+ * Causes all selected queued downloads to fall back to push.
+ */
+void
+on_popup_queue_push_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_menuitem;
+	(void) unused_udata;
+
+	push_activate(FALSE);
+}
+
+
+/**
+ * For all selected active downloads, remove all downloads with
+ * the same name.
  */
 void
 on_popup_downloads_abort_named_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
+    guint removed = 0;
 
 	(void) unused_menuitem;
 	(void) unused_udata;
@@ -312,16 +403,17 @@ on_popup_downloads_abort_named_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads, remove all downloads with the same host 
+ * For all selected active downloads, remove all downloads with
+ * the same host.
  */
 /* XXX: routing misnamed: we're "forgetting" here, not "aborting" */
 void
 on_popup_downloads_abort_host_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
-	
+    guint removed = 0;
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -331,26 +423,27 @@ on_popup_downloads_abort_host_activate(GtkMenuItem *unused_menuitem,
 
    	for (sl = selected; sl; sl = g_slist_next(sl)) {
 		struct download *d = sl->data;
-		removed += guc_download_remove_all_from_peer
-			(download_guid(d), download_ip(d), download_port(d), FALSE);
+		removed += guc_download_remove_all_from_peer(download_guid(d),
+						download_addr(d), download_port(d), FALSE);
 	}
 	g_slist_free(selected);
 
-    statusbar_gui_message(15, 
-		removed == 1 ? _("Forgot %d download") : _("Forgot %d downloads"),
+    statusbar_gui_message(15,
+		NG_("Forgot %u download", "Forgot %u downloads", removed),
 		removed);
 }
 
 
 /**
- *	For all selected active downloads, remove all downloads with the same sha1 
+ * For all selected active downloads, remove all downloads with
+ * the same sha1.
  */
 void
-on_popup_downloads_abort_sha1_activate(GtkMenuItem *unused_menuitem, 
-	gpointer unused_udata) 
+on_popup_downloads_abort_sha1_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
+    guint removed = 0;
 
 	(void) unused_menuitem;
 	(void) unused_udata;
@@ -361,8 +454,9 @@ on_popup_downloads_abort_sha1_activate(GtkMenuItem *unused_menuitem,
 
    	for (sl = selected; sl; sl = g_slist_next(sl)) {
 		struct download *d = sl->data;
-		removed += guc_download_remove_all_with_sha1
-			(d->file_info->sha1);
+
+		if (d->file_info->sha1)
+			removed += guc_download_remove_all_with_sha1(d->file_info->sha1);
 	}
 	g_slist_free(selected);
 
@@ -371,10 +465,10 @@ on_popup_downloads_abort_sha1_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads, remove file 
+ * For all selected active downloads, remove file.
  */
 void on_popup_downloads_remove_file_activate(GtkMenuItem *unused_menuitem,
-     gpointer unused_udata) 
+     gpointer unused_udata)
 {
 	GSList *sl, *selected;
 
@@ -406,7 +500,7 @@ void on_popup_downloads_remove_file_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads, send back to queue 
+ * For all selected active downloads, send back to queue.
  */
 void
 on_popup_downloads_queue_activate(GtkMenuItem *unused_menuitem,
@@ -455,11 +549,11 @@ copy_selection_to_clipboard(const gchar *treeview_name,
 }
 
 /**
- *	For selected download, copy URL to clipboard 
+ * For selected download, copy URL to clipboard.
  */
 void
 on_popup_downloads_copy_url_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	(void) unused_menuitem;
 	(void) unused_udata;
@@ -469,14 +563,14 @@ on_popup_downloads_copy_url_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads connect to host 
+ * For all selected active downloads connect to host.
  */
 void
 on_popup_downloads_connect_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-	
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -485,8 +579,8 @@ on_popup_downloads_connect_activate(GtkMenuItem *unused_menuitem,
 		return;
 
 	for (sl = selected; sl; sl = g_slist_next(sl)) {
-		struct download *d = sl->data;
-   		guc_node_add(download_ip(d), download_port(d));
+		const struct download *d = sl->data;
+   		guc_node_add(download_addr(d), download_port(d), CONNECT_F_FORCE);
    	}
 	g_slist_free(selected);
 }
@@ -499,14 +593,14 @@ on_popup_downloads_connect_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued downloads, activate them
+ * For all selected queued downloads, activate them.
  */
 void
 on_popup_queue_start_now_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-	
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -524,7 +618,7 @@ on_popup_queue_start_now_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued downloads, forget them
+ * For all selected queued downloads, forget them.
  */
 void
 on_popup_queue_abort_activate(GtkMenuItem *unused_menuitem,
@@ -549,15 +643,16 @@ on_popup_queue_abort_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued downloads, remove all downloads with same name 
+ * For all selected queued downloads, remove all downloads with
+ * same name.
  */
 void
 on_popup_queue_abort_named_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
-	
+    guint removed = 0;
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -577,15 +672,16 @@ on_popup_queue_abort_named_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued downloads, remove all downloads with same host 
+ * For all selected queued downloads, remove all downloads with
+ * same host.
  */
 void
 on_popup_queue_abort_host_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
-	
+    guint removed = 0;
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -597,7 +693,7 @@ on_popup_queue_abort_host_activate(GtkMenuItem *unused_menuitem,
 	for (sl = selected; sl; sl = g_slist_next(sl)) {
 		struct download *d = sl->data;
 		removed += guc_download_remove_all_from_peer(
-				download_guid(d), download_ip(d), download_port(d), FALSE);
+				download_guid(d), download_addr(d), download_port(d), FALSE);
 	}
 	g_slist_free(selected);
 
@@ -607,15 +703,16 @@ on_popup_queue_abort_host_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued downloads, remove all downloads with same sha1 
+ * For all selected queued downloads, remove all downloads with
+ * same sha1.
  */
 void
 on_popup_queue_abort_sha1_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	GSList *sl, *selected;
-    gint removed = 0;
-	
+    guint removed = 0;
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
@@ -626,8 +723,9 @@ on_popup_queue_abort_sha1_activate(GtkMenuItem *unused_menuitem,
 
 	for (sl = selected; sl; sl = g_slist_next(sl)) {
 		struct download *d = sl->data;
-   		removed += guc_download_remove_all_with_sha1
-			(d->file_info->sha1);
+
+		if (d->file_info->sha1)
+   			removed += guc_download_remove_all_with_sha1(d->file_info->sha1);
 	}
 	g_slist_free(selected);
 
@@ -636,11 +734,11 @@ on_popup_queue_abort_sha1_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued download, copy url to clipboard
+ * For all selected queued download, copy url to clipboard.
  */
 void
 on_popup_queue_copy_url_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
 	(void) unused_menuitem;
 	(void) unused_udata;
@@ -651,13 +749,14 @@ on_popup_queue_copy_url_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected queued download, connect to host 
+ * For all selected queued download, connect to host.
  */
 void
 on_popup_queue_connect_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+	gpointer unused_udata)
 {
-	GSList *sl, *selected;
+	GSList *selected;
+	const GSList *sl;
 
 	(void) unused_menuitem;
 	(void) unused_udata;
@@ -668,12 +767,12 @@ on_popup_queue_connect_activate(GtkMenuItem *unused_menuitem,
 		return;
 
 	for (sl = selected; sl; sl = g_slist_next(sl)) {
-		struct download *d = sl->data;
-    	guc_node_add(download_ip(d), download_port(d));
+		const struct download *d = sl->data;
+    	guc_node_add(download_addr(d), download_port(d), CONNECT_F_FORCE);
 	}
 	g_slist_free(selected);
 }
- 
+
 
 
 /***
@@ -682,13 +781,13 @@ on_popup_queue_connect_activate(GtkMenuItem *unused_menuitem,
 
 
 /**
- *	For all selected active downloads, forget them 
+ * For all selected active downloads, forget them.
  */
 void
 on_button_downloads_abort_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
 {
-	GSList *sl, *selected;
+	GSList *selected, *sl;
 
 	(void) unused_button;
 	(void) unused_udata;
@@ -707,7 +806,7 @@ on_button_downloads_abort_clicked(GtkButton *unused_button,
 
 
 /**
- *	For all selected active downloads, resume 
+ * For all selected active downloads, resume.
  */
 void
 on_button_downloads_resume_clicked(GtkButton *unused_button,
@@ -724,7 +823,7 @@ on_button_downloads_resume_clicked(GtkButton *unused_button,
 
 	for (sl = selected; sl; sl = g_slist_next(sl)) {
 		struct download *d = sl->data;
-     	guc_download_resume(d);	
+     	guc_download_resume(d);
 	}
 	g_slist_free(selected);
 
@@ -748,13 +847,13 @@ on_popup_downloads_config_cols_activate(GtkMenuItem *unused_menuitem,
 
 
 
-/*** 
+/***
  *** Queued downloads
  ***/
 
 
 /**
- *	Select all queued downloads that match given regex in editable
+ * Select all queued downloads that match given regex in editable.
  */
 void
 on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
@@ -776,8 +875,8 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
     regex = STRTRACK(gtk_editable_get_chars(GTK_EDITABLE(editable), 0, -1));
 
 	g_return_if_fail(regex != NULL);
-	
-    err = regcomp(&re, 
+
+    err = regcomp(&re,
                   regex,
                   REG_EXTENDED|REG_NOSUB|(queue_regex_case ? 0 : REG_ICASE));
 
@@ -788,30 +887,30 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
         statusbar_gui_warning(15,
 			"on_entry_queue_regex_activate: regex error %s", buf);
     } else {
-  
+
 		tree_view = GTK_TREE_VIEW
 			(lookup_widget(main_window, "treeview_downloads_queue"));
 		model = gtk_tree_view_get_model(tree_view);
-			
-		if (NULL != model) {		
+
+		if (NULL != model) {
 
 			selection = gtk_tree_view_get_selection(tree_view);
 			gtk_tree_selection_unselect_all(selection);
-			
+
 			if (!gtk_tree_model_get_iter_first(model, &iter))
 				return; /* tree is empty */
-			
+
 			for (
 				total_nodes = 0;
-				gtk_tree_model_iter_next(model, &iter); 
+				gtk_tree_model_iter_next(model, &iter);
 				total_nodes++
 			) {
-				
+
 				gtk_tree_model_get(model, &iter, c_queue_record, &d, (-1));
 
 				if (DL_GUI_IS_HEADER == d)
-					continue;				
-				
+					continue;
+
 				if (!d) {
 	                g_warning("on_entry_queue_regex_activate: "
 						"row has NULL data");
@@ -830,25 +929,25 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
         	        g_warning("on_entry_queue_regex_activate: "
 						"regexp memory overflow");
         	}
-        
-			statusbar_gui_message(15, 
-				total_nodes == 1
-					? _("Selected %u of %u queued download matching \"%s\".") 
-					: _("Selected %u of %u queued downloads matching \"%s\"."), 
+
+			statusbar_gui_message(15,
+				NG_("Selected %u of %u queued download matching \"%s\".",
+					"Selected %u of %u queued downloads matching \"%s\".",
+					total_nodes),
 				m, total_nodes, regex);
 
 			regfree(&re);
 	    }
 	}
-    
+
 	g_free(regex);
     gtk_entry_set_text(GTK_ENTRY(editable), "");
 }
 
 
 /**
- *	When the right mouse button is clicked on the active downloads treeview,
- *	show the popup with the context menu
+ * When the right mouse button is clicked on the active downloads
+ * treeview, show the popup with the context menu.
  */
 gboolean
 on_treeview_downloads_button_press_event(GtkWidget *widget,
@@ -864,7 +963,7 @@ on_treeview_downloads_button_press_event(GtkWidget *widget,
 		return FALSE;
 
     gtk_menu_popup(
-        GTK_MENU(popup_downloads), NULL, NULL, NULL, NULL, 
+        GTK_MENU(popup_downloads), NULL, NULL, NULL, NULL,
         event->button, event->time);
 
 	return TRUE;
@@ -872,8 +971,8 @@ on_treeview_downloads_button_press_event(GtkWidget *widget,
 
 
 /**
- *	When the right mouse button is clicked on the queued downloads treeview,
- *	show the popup with the context menu
+ * When the right mouse button is clicked on the queued downloads
+ * treeview, show the popup with the context menu.
  */
 gboolean
 on_treeview_downloads_queue_button_press_event(GtkWidget *widget,
@@ -889,14 +988,14 @@ on_treeview_downloads_queue_button_press_event(GtkWidget *widget,
 		return FALSE;
 
     gtk_menu_popup(
-        GTK_MENU(popup_queue), NULL, NULL, NULL, NULL, 
+        GTK_MENU(popup_queue), NULL, NULL, NULL, NULL,
         event->button, event->time);
 
 	return TRUE;
 }
 
 
-void on_treeview_downloads_select_row(GtkTreeView *tree_view, 
+void on_treeview_downloads_select_row(GtkTreeView *tree_view,
 	gpointer unused_udata)
 {
 	GtkTreeSelection *selection;
@@ -918,11 +1017,11 @@ void on_treeview_downloads_select_row(GtkTreeView *tree_view,
 	g_slist_free(selected);
 	selected = NULL;
 
-    gtk_widget_set_sensitive(lookup_widget(popup_downloads, 
+    gtk_widget_set_sensitive(lookup_widget(popup_downloads,
 			"popup_downloads_copy_url"), activate);
-   	gtk_widget_set_sensitive(lookup_widget(popup_downloads, 
+   	gtk_widget_set_sensitive(lookup_widget(popup_downloads,
 			"popup_downloads_connect"), activate);
-	
+
 	/* Takes care of other widgets */
 	gui_update_download_abort_resume();
 }
@@ -941,13 +1040,13 @@ static void queue_select_row_helper(GtkTreeModel *model,
 
 	(void) unused_path;
 
-	gtk_tree_model_get(model, iter, c_queue_record, &d, (-1));				
+	gtk_tree_model_get(model, iter, c_queue_record, &d, (-1));
 	q->is_header |= DL_GUI_IS_HEADER == d;
 	q->count++;
 }
 
 void
-on_treeview_downloads_queue_select_row(GtkTreeView *unused_tv, 
+on_treeview_downloads_queue_select_row(GtkTreeView *unused_tv,
 	gpointer unused_udata)
 {
 	GtkTreeSelection *selection;
@@ -962,11 +1061,11 @@ on_treeview_downloads_queue_select_row(GtkTreeView *unused_tv,
 	selection = gtk_tree_view_get_selection(
 		GTK_TREE_VIEW(lookup_widget(main_window, "treeview_downloads")));
 	gtk_tree_selection_unselect_all(selection);
-		
+
 	selection = gtk_tree_view_get_selection(
 		GTK_TREE_VIEW(lookup_widget(main_window, "treeview_downloads_queue")));
 	gtk_tree_selection_selected_foreach(selection, queue_select_row_helper, &q);
-	
+
 	gtk_widget_set_sensitive(
 		lookup_widget(popup_queue, "popup_queue_copy_url"),
 		!q.is_header && q.count == 1);
@@ -990,59 +1089,59 @@ on_treeview_downloads_queue_select_row(GtkTreeView *unused_tv,
 }
 
 
-/*
- *	on_popup_downloads_expand_all_activate
+/**
+ * on_popup_downloads_expand_all_activate
  */
 void on_popup_downloads_expand_all_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata)
 {
 	GtkTreeView *tree_view = GTK_TREE_VIEW
 		(lookup_widget(main_window, "treeview_downloads"));
-		
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
-	downloads_gui_expand_all(tree_view);	
+	downloads_gui_expand_all(tree_view);
 }
 
 
-void on_popup_downloads_collapse_all_activate(GtkMenuItem *unused_menuitem, 
+void on_popup_downloads_collapse_all_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata)
 {
 	GtkTreeView *tree_view = GTK_TREE_VIEW
 		(lookup_widget(main_window, "treeview_downloads"));
-	
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
-	downloads_gui_collapse_all(tree_view);	
+	downloads_gui_collapse_all(tree_view);
 }
 
 
 void
-on_popup_queue_expand_all_activate(GtkMenuItem *unused_menuitem, 
+on_popup_queue_expand_all_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata)
 {
 	GtkTreeView *tree_view = GTK_TREE_VIEW
 		(lookup_widget(main_window, "treeview_downloads_queue"));
-	
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
-	downloads_gui_expand_all(tree_view);	
+	downloads_gui_expand_all(tree_view);
 }
 
 void
-on_popup_queue_collapse_all_activate(GtkMenuItem *unused_menuitem, 
+on_popup_queue_collapse_all_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata)
 {
 	GtkTreeView *tree_view = GTK_TREE_VIEW
 		(lookup_widget(main_window, "treeview_downloads_queue"));
-	
+
 	(void) unused_menuitem;
 	(void) unused_udata;
 
-	downloads_gui_collapse_all(tree_view);	
+	downloads_gui_collapse_all(tree_view);
 }
 
 void
