@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c,v 1.8 2004/11/13 09:45:23 cbiere Exp $
+ * $Id: gnet_stats.c,v 1.21 2005/12/19 07:12:17 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,17 +25,19 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c,v 1.8 2004/11/13 09:45:23 cbiere Exp $");
+RCSID("$Id: gnet_stats.c,v 1.21 2005/12/19 07:12:17 cbiere Exp $");
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/misc.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 #include "if/core/gnutella.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/tm.h"
 #include "lib/glib-missing.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -55,40 +57,23 @@ static const gchar * const msg_stats_label[] = {
 	N_("Dropped"),
 	N_("Queued"),
 	N_("Relayed"),
-	N_("Gen. Queued"),
-	N_("Gen. Sent")
+	N_("Gen. queued"),
+	N_("Gen. sent")
 };
 
-enum {
+enum gnet_stats_nb_page {
+	GNET_STATS_NB_PAGE_STATS,
 	GNET_STATS_NB_PAGE_MESSAGES,
 	GNET_STATS_NB_PAGE_FLOWC,
 	GNET_STATS_NB_PAGE_RECV,
+	GNET_STATS_NB_PAGE_HORIZON,
 
-	GNET_STATS_NP_PAGE_NUMBER
+	NUM_GNET_STATS_NB_PAGES
 };
 
 static void hide_column_by_title(GtkTreeView *, const gchar *, gboolean);
 static void gnet_stats_update_drop_reasons(const gnet_stats_t *);
 
-
-/***
- *** Callbacks
- ***/
-
-static gint gnet_stats_drop_reasons_type = MSG_TOTAL;
-
-static void
-on_gnet_stats_type_selected(GtkItem *unused_item, gpointer data)
-{
-	static gnet_stats_t stats;
-
-	(void) unused_item;
-
-	gnet_stats_drop_reasons_type = GPOINTER_TO_INT(data);
-	guc_gnet_stats_get(&stats);
-	gnet_stats_update_drop_reasons(&stats);
-}
-
 /***
  *** Private functions
  ***/
@@ -100,9 +85,9 @@ hide_column_by_title(GtkTreeView *treeview, const gchar *header_title,
 	GList *list, *l;
 	const gchar *title;
 
-	g_assert(NULL != header_title); 
+	g_assert(NULL != header_title);
 	list = gtk_tree_view_get_columns(treeview);
-	g_assert(NULL != list); 
+	g_assert(NULL != list);
 
 	for (l = list; NULL != l; l = g_list_next(l))
 		if (NULL != l->data) {
@@ -118,85 +103,85 @@ hide_column_by_title(GtkTreeView *treeview, const gchar *header_title,
 }
 
 static gchar *
-pkt_stat_str(gchar *strbuf, gulong n, const guint64 *val_tbl,
+pkt_stat_str(gchar *dst, size_t size, const guint64 *val_tbl,
 	gint type, gboolean perc)
 {
-	if (val_tbl[type] == 0)
-		g_strlcpy(strbuf, "-", n);
+	if (0 == val_tbl[type])
+		g_strlcpy(dst, "-", size);
 	else {
 		if (!perc)
-			gm_snprintf(strbuf, n, "%" PRIu64, val_tbl[type]);
+			uint64_to_string_buf(val_tbl[type], dst, size);
 		else
-			gm_snprintf(strbuf, n, "%.2f%%", 
+			gm_snprintf(dst, size, "%.2f%%",
 			    (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
 	}
 
-	return strbuf;
+	return dst;
 }
 
 
 static const gchar *
-byte_stat_str(gchar *strbuf, gulong n, const guint64 *val_tbl,
+byte_stat_str(gchar *dst, gulong n, const guint64 *val_tbl,
 	gint type, gboolean perc)
 {
-	if (val_tbl[type] == 0)
-		g_strlcpy(strbuf, "-", n);
+	if (0 == val_tbl[type])
+		g_strlcpy(dst, "-", n);
 	else if (!perc)
-		g_strlcpy(strbuf, compact_size(val_tbl[type]), n);
+		g_strlcpy(dst, compact_size(val_tbl[type]), n);
 	else
-		gm_snprintf(strbuf, n, "%.2f%%", 
+		gm_snprintf(dst, n, "%.2f%%",
 		    (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
 
-	return strbuf;
+	return dst;
 }
 
 static const gchar *
-drop_stat_str(gchar *str, gulong n, const gnet_stats_t *stats, gint reason,
-	gint selected_type)
+drop_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint reason,
+	gint selected_type, gboolean percent)
 {
 	guint32 total = stats->pkg.dropped[MSG_TOTAL];
 
 	if (stats->drop_reason[reason][selected_type] == 0)
-		g_strlcpy(str, "-", n);
-	else if (gnet_stats_drop_perc)
-		gm_snprintf(str, n, "%.2f%%", 
+		g_strlcpy(dst, "-", size);
+	else if (percent)
+		gm_snprintf(dst, size, "%.2f%%",
 		    (gfloat) stats->drop_reason[reason][selected_type] / total * 100);
 	else
-		gm_snprintf(str, n, "%" PRIu64,
-			stats->drop_reason[reason][selected_type]);
+		uint64_to_string_buf(stats->drop_reason[reason][selected_type],
+			dst, size);
 
-	return str;
+	return dst;
 }
 
 static const gchar *
-general_stat_str(gchar *str, gulong n, const gnet_stats_t *stats, gint type)
+general_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint type)
 {
 	if (stats->general[type] == 0)
-		g_strlcpy(str, "-", n);
+		g_strlcpy(dst, "-", size);
 	else if (type == GNR_QUERY_COMPACT_SIZE)
-		g_strlcpy(str, compact_size(stats->general[type]), n);
+		g_strlcpy(dst, compact_size(stats->general[type]), size);
 	else
-		gm_snprintf(str, n, "%" PRIu64, stats->general[type]);
+		uint64_to_string_buf(stats->general[type], dst, size);
 
-	return str;
+	return dst;
 }
 
 static const gchar *
-type_stat_str(gchar *strbuf, gulong n, gulong value, gulong total,
+type_stat_str(gchar *dst, size_t size, gulong value, gulong total,
 	gboolean perc, gboolean bytes)
 {
 	if (value == 0 || total == 0)
-		g_strlcpy(strbuf, "-", n);
+		g_strlcpy(dst, "-", size);
 	else if (perc)
-		gm_snprintf(strbuf, n, "%.2f%%", (gfloat) value / total * 100.0);
+		gm_snprintf(dst, size, "%.2f%%", (gfloat) value / total * 100.0);
 	else {
 		if (bytes)
-			g_strlcpy(strbuf, compact_size(value), n);
+			g_strlcpy(dst, compact_size(value), size);
 		else
-			gm_snprintf(strbuf, n, "%lu", (gulong) value);
+			gm_snprintf(dst, size, "%lu", (gulong) value);
 	}
 
-	return strbuf;
+	return dst;
 }
 
 static void
@@ -212,7 +197,7 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 	g_object_set(renderer,
 		"xalign", xalign,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		NULL);
+		(void *) 0);
 	column = gtk_tree_view_column_new_with_attributes(label, renderer,
 		"text", column_id,
 		NULL);
@@ -221,26 +206,27 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 		"min-width", 1,
 		"reorderable", TRUE,
 		"resizable", TRUE,
-		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		NULL);
+		"sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE,
+		(void *) 0);
 	gtk_tree_view_append_column(treeview, column);
 }
 
 static void
 gnet_stats_update_general(const gnet_stats_t *stats)
 {
-	static gchar str[32];
-	GtkTreeView *treeview = treeview_gnet_stats_general;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gint n;
 
-	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(
+				treeview_gnet_stats_general));
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
 	for (n = 0; n < GNR_TYPE_COUNT; n++) {
-		general_stat_str(str, sizeof(str), stats, n);
-		gtk_list_store_set(store, &iter, 1, str, (-1));
+		gchar buf[32];
+
+		general_stat_str(buf, sizeof buf, stats, n);
+		gtk_list_store_set(store, &iter, 1, buf, (-1));
 		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
 	}
 }
@@ -248,18 +234,20 @@ gnet_stats_update_general(const gnet_stats_t *stats)
 static void
 gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 {
-	static gchar str[32];
-	GtkTreeView *treeview = treeview_gnet_stats_drop_reasons;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gint n;
 
-	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(
+				treeview_gnet_stats_drop_reasons));
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
 	for (n = 0; n < MSG_DROP_REASON_COUNT; n++) {
-		drop_stat_str(str, sizeof(str), stats, n, gnet_stats_drop_reasons_type);
-		gtk_list_store_set(store, &iter, 1, str, (-1));
+		gchar buf[32];
+
+		drop_stat_str(buf, sizeof buf, stats, n,
+			gnet_stats_drop_reasons_type, gnet_stats_bytes);
+		gtk_list_store_set(store, &iter, 1, buf, (-1));
 		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
 	}
 }
@@ -288,7 +276,7 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 		if (!bytes) {
 			gtk_list_store_set(store, &iter,
 				c_gs_received,	 pkt_stat_str(str[c_gs_received],
-									len, stats->pkg.received, n, perc), 
+									len, stats->pkg.received, n, perc),
 				c_gs_expired,	 pkt_stat_str(str[c_gs_expired],
 									len, stats->pkg.expired, n, perc),
 				c_gs_dropped,	 pkt_stat_str(str[c_gs_dropped],
@@ -325,13 +313,15 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 
 }
 
+typedef guint64 stat_counters_t[MSG_TYPE_COUNT];
+
 static void
 gnet_stats_update_types(
 	const gnet_stats_t *unused_stats,
 	GtkTreeView *treeview,
 	gint columns,
-	const guint64 (*byte_counters)[MSG_TYPE_COUNT],
-	const guint64 (*pkg_counters)[MSG_TYPE_COUNT])
+	const stat_counters_t *byte_counters,
+	const stat_counters_t *pkg_counters)
 {
 	static gchar str[MSG_TYPE_COUNT][32];
 	GtkListStore *store;
@@ -363,7 +353,7 @@ gnet_stats_update_types(
 			for (i = 0; i < columns; i++) {
 				gulong	value;
 				gulong	total;
-		
+
 				value = byte_counters[i][n];
 				total = byte_counters[i][MSG_TOTAL];
 				if (!with_headers) {
@@ -384,8 +374,7 @@ gnet_stats_update_types(
 static void
 gnet_stats_update_flowc(const gnet_stats_t *stats)
 {
-	const guint64 (*byte_counters)[MSG_TYPE_COUNT];
-	const guint64 (*pkg_counters)[MSG_TYPE_COUNT];
+	const stat_counters_t *byte_counters, *pkg_counters;
 	GtkTreeView *treeview = treeview_gnet_stats_flowc;
 	gboolean hops = FALSE;
 
@@ -405,11 +394,10 @@ gnet_stats_update_flowc(const gnet_stats_t *stats)
 static void
 gnet_stats_update_recv(const gnet_stats_t *stats)
 {
-	const guint64 (*byte_counters)[MSG_TYPE_COUNT];
-	const guint64 (*pkg_counters)[MSG_TYPE_COUNT];
+	const stat_counters_t *byte_counters, *pkg_counters;
 	GtkTreeView *treeview = treeview_gnet_stats_recv;
 	gboolean hops = FALSE;
-	
+
 	gui_prop_get_boolean_val(PROP_GNET_STATS_HOPS, &hops);
 	if (hops) {
 		pkg_counters = stats->pkg.received_hops;
@@ -431,9 +419,9 @@ gnet_stats_update_horizon(void)
 	GtkTreeIter iter;
 	gint i;
 	static time_t last_horizon_update = 0;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 	gint global_table_size;
-	
+
 	/*
 	 * Update horizon statistics table, but only if the values have changed.
 	 *      -- TNT 09/06/2004
@@ -441,12 +429,12 @@ gnet_stats_update_horizon(void)
 	 * Changed this check to update the table every 2 seconds, because not
 	 * only the HSEP table but also the PONG-based library sizes of direct
 	 * non-HSEP neighbors may have changed.
-	 *      -- TNT 14/06/2004 
+	 *      -- TNT 14/06/2004
 	 */
 
     if (delta_time(now, last_horizon_update) < 2)
 		return;
-	
+
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
@@ -561,8 +549,8 @@ gnet_stats_gui_flowc_init(void)
 	g_object_unref(model);
 }
 
-static
-void gnet_stats_gui_drop_reasons_init(void)
+static void
+gnet_stats_gui_drop_reasons_init(void)
 {
 	GtkTreeView *treeview;
 	GtkTreeModel *model;
@@ -592,7 +580,7 @@ void gnet_stats_gui_drop_reasons_init(void)
 		}
 
 		add_column(treeview, n, width[n], (gfloat) (n != 0),
-			n == 0 ? _("Type") : _("Count"));
+			n == 0 ? _("Reason") : _("Count"));
 	}
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
@@ -737,33 +725,6 @@ gnet_stats_gui_recv_init(void)
 	g_object_unref(model);
 }
 
-static void
-gnet_stats_gui_type_menu_init(void)
-{
-	GtkOptionMenu *option_menu;
-	GtkWidget *menu;
-	gint n;
-
-	option_menu = GTK_OPTION_MENU(
-		lookup_widget(main_window, "option_menu_gnet_stats_type"));
-	menu = gtk_menu_new();
-
-	for (n = 0; n < msg_type_str_size(); n++) {
-		GtkWidget *menu_item;
-
-		menu_item = gtk_menu_item_new_with_label(msg_type_str(n));
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-		g_signal_connect(
-			GTK_OBJECT(menu_item), "activate",
-			G_CALLBACK(on_gnet_stats_type_selected),
-			GINT_TO_POINTER(n));
-	}
-	gtk_option_menu_set_menu(option_menu, menu);
-	gtk_option_menu_set_history(option_menu, MSG_TOTAL);
-	gtk_widget_show_all(GTK_WIDGET(option_menu));
-}
-
-
 /***
  *** Public functions
  ***/
@@ -786,7 +747,6 @@ gnet_stats_gui_init(void)
 	gnet_stats_gui_general_init();
 	gnet_stats_gui_messages_init();
 	gnet_stats_gui_recv_init();
-	gnet_stats_gui_type_menu_init();
 
 	guc_hsep_add_global_table_listener(
 		(GCallback) gnet_stats_gui_horizon_update, FREQ_UPDATES, 0);
@@ -795,22 +755,22 @@ gnet_stats_gui_init(void)
 void
 gnet_stats_gui_shutdown(void)
 {
-	struct {
+	static const struct {
 		property_t prop;
-		GtkTreeView *tv;
+		GtkTreeView **tv;
 	} widths[] = {
 		{ 	PROP_GNET_STATS_GENERAL_COL_WIDTHS,
-			treeview_gnet_stats_general },
+			&treeview_gnet_stats_general },
 		{ 	PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
-			treeview_gnet_stats_drop_reasons },
+			&treeview_gnet_stats_drop_reasons },
 		{ 	PROP_GNET_STATS_MSG_COL_WIDTHS,
-			treeview_gnet_stats_messages },
+			&treeview_gnet_stats_messages },
 		{ 	PROP_GNET_STATS_FC_COL_WIDTHS,
-			treeview_gnet_stats_flowc },
+			&treeview_gnet_stats_flowc },
 		{ 	PROP_GNET_STATS_RECV_COL_WIDTHS,
-			treeview_gnet_stats_recv },
+			&treeview_gnet_stats_recv },
 		{ 	PROP_GNET_STATS_HORIZON_COL_WIDTHS,
-			treeview_gnet_stats_horizon },
+			&treeview_gnet_stats_horizon },
 	};
 	size_t i;
 
@@ -818,7 +778,7 @@ gnet_stats_gui_shutdown(void)
 	    (GCallback) gnet_stats_gui_horizon_update);
 
 	for (i = 0; i < G_N_ELEMENTS(widths); i++) {
-		tree_view_save_widths(widths->tv, widths->prop);
+		tree_view_save_widths(*(widths->tv), widths->prop);
 	}
 }
 
@@ -834,7 +794,7 @@ gnet_stats_gui_update(time_t now)
 		return;
 	last_update = now;
 	locked = TRUE;
-	
+
 	current_page = gtk_notebook_get_current_page(notebook_main);
 	if (current_page != nb_main_page_gnet_stats)
 		goto cleanup;
@@ -842,12 +802,14 @@ gnet_stats_gui_update(time_t now)
 	guc_gnet_stats_get(&stats);
 
 	current_page = gtk_notebook_get_current_page(notebook_gnet_stats);
-
-	gnet_stats_update_general(&stats);
-	gnet_stats_update_drop_reasons(&stats);
-	gnet_stats_update_horizon();
-
-	switch (current_page) {
+	switch ((enum gnet_stats_nb_page) current_page) {
+	case GNET_STATS_NB_PAGE_STATS:
+		gnet_stats_update_general(&stats);
+		gnet_stats_update_drop_reasons(&stats);
+		goto cleanup;
+	case GNET_STATS_NB_PAGE_HORIZON:
+		gnet_stats_update_horizon();
+		goto cleanup;
 	case GNET_STATS_NB_PAGE_MESSAGES:
 		switch (gnet_stats_source) {
 		case GNET_STATS_FULL:
@@ -862,16 +824,18 @@ gnet_stats_gui_update(time_t now)
 			g_assert_not_reached();
 		}
 		gnet_stats_update_messages(&stats);
-		break;
+		goto cleanup;
 	case GNET_STATS_NB_PAGE_FLOWC:
 		gnet_stats_update_flowc(&stats);
-		break;
+		goto cleanup;
 	case GNET_STATS_NB_PAGE_RECV:
 		gnet_stats_update_recv(&stats);
+		goto cleanup;
+
+	case NUM_GNET_STATS_NB_PAGES:
 		break;
-	default:
-		g_assert_not_reached();
 	}
+	g_assert_not_reached();
 
 cleanup:
 	locked = FALSE;
