@@ -1,7 +1,8 @@
 /*
- * $Id: upload_stats.c,v 1.3 2004/11/13 09:45:24 cbiere Exp $
+ * $Id: upload_stats.c,v 1.13 2005/11/16 13:35:14 cbiere Exp $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
+ * Copyright (c) 2002, Michael Tesch
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -21,35 +22,47 @@
  *  Foundation, Inc.:
  *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *----------------------------------------------------------------------
+ */
+
+/**
+ * @ingroup gtk
+ * @file
  *
- * upload_stats.c - keep track of which files we send away, and how often.
+ * Keep track of which files we send away, and how often.
  *
- *		Statistics are kept by _FILENAME_ and file size, 
- *		not by actual path, so two files with the same
- *		name and size will be counted in the same bin.  
- *		I dont see this as a limitation because the
- *		user wouldn't be able to differentiate the files anyway.
- *		This could be extended to keep the entire path to 
- *		each file and optionally show the entire path, but..
- *		
- *		the 'upload_history' file has the following format:
- *		<url-escaped filename> <file size> <attempts> <completions>
+ * Statistics are kept by _FILENAME_ and file size, not by actual
+ * path, so two files with the same name and size will be counted
+ * in the same bin.
  *
- *		TODO: add a check to make sure that all of the files still exist(?)
- *			grey them out if they dont, optionally remove them from the 
- *			stats list (when 'Clear Non-existent Files' is clicked)
+ * I dont see this as a limitation because the user wouldn't be able
+ * to differentiate the files anyway. This could be extended to keep
+ * the entire path to each file and optionally show the entire path,
+ * but..
  *
- *		(C) 2002 Michael Tesch, released with gtk-gnutella & its license
+ * The 'upload_history' file has the following format:
+ *
+ *		- "<url-escaped filename> <file size> <attempts> <completions>"
+ *
+ * @todo
+ * TODO: Add a check to make sure that all of the files still exist(?)
+ *       grey them out if they dont, optionally remove them from the
+ *       stats list (when 'Clear Non-existent Files' is clicked).
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2003
+ * @author Michael Tesch
+ * @date 2002
  */
 
 #include "gtk/gui.h"
 
-RCSID("$Id: upload_stats.c,v 1.3 2004/11/13 09:45:24 cbiere Exp $");
+RCSID("$Id: upload_stats.c,v 1.13 2005/11/16 13:35:14 cbiere Exp $");
 
 #include "interface-glade.h" /* for create_popup_upload_stats */
 
 #include "gtk/columns.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/misc.h"
 
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
@@ -62,8 +75,9 @@ RCSID("$Id: upload_stats.c,v 1.3 2004/11/13 09:45:24 cbiere Exp $");
 static GtkTreeView *upload_stats_treeview = NULL;
 static GtkWidget *popup_upload_stats = NULL;
 
-/* Private callbacks */
-
+/**
+ * Private callbacks.
+ */
 static gboolean
 on_button_press_event(GtkWidget *unused_widget, GdkEventButton *event,
 	gpointer unused_udata)
@@ -90,7 +104,7 @@ on_button_press_event(GtkWidget *unused_widget, GdkEventButton *event,
  * @param cell The cell renderer for the column.
  * @param model The model holding the upload stats info.
  * @param iter The iter of the row we are working with.
- * @param data user data passed to the function.
+ * @param unused_data user data passed to the function.
  *
  */
 static void
@@ -106,7 +120,7 @@ cell_render_size_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	g_assert(iter != NULL);
 
 	gtk_tree_model_get(model, iter, c_us_size, &val, (-1));
-	g_object_set(cell, "text", short_size(val), NULL);
+	g_object_set(cell, "text", short_size(val), (void *) 0);
 }
 
 /**
@@ -119,7 +133,7 @@ cell_render_size_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
  * @param cell The cell renderer for the column.
  * @param model The model holding the upload stats info.
  * @param iter The iter of the row we are working with.
- * @param data user data passed to the function.
+ * @param unused_data user data passed to the function.
  *
  */
 static void
@@ -128,7 +142,7 @@ cell_render_norm_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 {
 	gfloat val = 0.0;
 	gchar tmpstr[32];
-	
+
 	(void) unused_data;
 	g_assert(column != NULL);
 	g_assert(cell != NULL);
@@ -137,7 +151,7 @@ cell_render_norm_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 
 	gtk_tree_model_get(model, iter, c_us_norm, &val, (-1));
 	gm_snprintf(tmpstr, sizeof tmpstr, "%1.3f", val);
-	g_object_set(cell, "text", tmpstr, NULL);
+	g_object_set(cell, "text", tmpstr, (void *) 0);
 }
 
 /**
@@ -148,19 +162,18 @@ cell_render_norm_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
  * @param tree The GtkTreeView which will have a new column appended.
  * @param column_id The numerical tag for this column.
  * @param title The title displayed in the column header.
- * @param width The width of the column.
- * @param xalign A number between 0.0 (left) and 1.0 (right) 
+ * @param xalign A number between 0.0 (left) and 1.0 (right)
  * horizontal alignment.
  * @param cell_data_func The function which will render that data
  * to show in the cell.  If, NULL gtk will try to render the data
  * as appropriate for the type.
  *
  */
-static void add_column(
+static void
+add_column(
     GtkTreeView *tree,
 	gint column_id,
 	const gchar *title,
-	gint width,
 	gfloat xalign,
 	GtkTreeCellDataFunc cell_data_func)
 {
@@ -177,20 +190,20 @@ static void add_column(
 	if (cell_data_func != NULL) {
 		gtk_tree_view_column_set_cell_data_func(column, renderer,
 			cell_data_func, GINT_TO_POINTER(column_id), NULL);
-	}		
+	}
 
 	g_object_set(renderer,
 		"xalign", xalign,
 		"xpad", GUI_CELL_RENDERER_XPAD,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		NULL);
+		(void *) 0);
 	g_object_set(column,
-		"fixed-width", MAX(1, width),
+		"fixed-width", 1,
 		"min-width", 1,
 		"resizable", TRUE,
 		"reorderable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		NULL);
+		(void *) 0);
    	gtk_tree_view_column_set_sort_column_id(column, column_id);
     gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 }
@@ -210,14 +223,15 @@ static void add_column(
  * parameters.
  *
  */
-static struct ul_stats *upload_stats_gui_find(
+static struct ul_stats *
+upload_stats_gui_find(
 	const gchar *name,
 	guint64 size,
 	GtkTreeModel *model,
 	GtkTreeIter *iter)
 {
 	gboolean valid;
-	
+
 	g_assert(name != NULL);
 	g_assert(iter != NULL);
 	g_assert(model != NULL);
@@ -236,27 +250,27 @@ static struct ul_stats *upload_stats_gui_find(
 }
 
 /**
- * Initialize the upload statistics gui.
+ * Initialize the upload statistics GUI.
  *
- * Initialize the upload statistics gui.  Define the
+ * Initialize the upload statistics GUI.  Define the
  * GtkTreeModel used to store the information as well
  * as rendering and sorting functions to use on the
  * cells and columns.
  */
-static void upload_stats_gui_init_intern(gboolean intern)
+static void
+upload_stats_gui_init_intern(gboolean intern)
 {
 	static const struct {
 		const gint id;
 		const gchar * const title;
-		const gint width;
 		const gfloat align;
 		const GtkTreeCellDataFunc func;
 	} columns[] = {
-		{ c_us_filename, N_("Filename"),   300, 0.0, NULL },
-		{ c_us_size,	 N_("Size"),	   60,  1.0, cell_render_size_func },
-		{ c_us_attempts, N_("Attempts"),   60,  1.0, NULL },
-		{ c_us_complete, N_("Complete"),   60,  1.0, NULL },
-    	{ c_us_norm, 	 N_("Normalized"), 60,  1.0, cell_render_norm_func }
+		{ c_us_filename, N_("Filename"),   0.0, NULL },
+		{ c_us_size,	 N_("Size"),	   1.0, cell_render_size_func },
+		{ c_us_attempts, N_("Attempts"),   1.0, NULL },
+		{ c_us_complete, N_("Complete"),   1.0, NULL },
+    	{ c_us_norm, 	 N_("Normalized"), 1.0, cell_render_norm_func }
 	};
 	static gboolean initialized = FALSE;
 	GtkTreeModel *model;
@@ -270,7 +284,7 @@ static void upload_stats_gui_init_intern(gboolean intern)
 			G_TYPE_STRING,		/* Filename (UTF-8 encoded) */
 			G_TYPE_UINT,		/* Size */
 			G_TYPE_UINT,		/* Attempts */
-			G_TYPE_UINT,		/* Completed */  
+			G_TYPE_UINT,		/* Completed */
 			G_TYPE_FLOAT,		/* Normalized */
 			G_TYPE_POINTER)); 	/* struct ul_stats */
 		upload_stats_treeview = GTK_TREE_VIEW(
@@ -282,7 +296,6 @@ static void upload_stats_gui_init_intern(gboolean intern)
 			add_column(upload_stats_treeview,
 				columns[i].id,
 				_(columns[i].title),
-				columns[i].width,
 				columns[i].align,
 				columns[i].func);
 		}
@@ -296,27 +309,14 @@ static void upload_stats_gui_init_intern(gboolean intern)
 	}
 
 	if (!intern) {
-		guint32 width[G_N_ELEMENTS(columns)];
-		gboolean visible[G_N_ELEMENTS(columns)];
-		
 		/* upload_stats_gui_init_intern() might be called internally before
 		 * settings_gui_init(). If it's called externally it's called from
 		 * main_gui_init() and the GUI properties are intialized. */
 
-		gui_prop_get_guint32(PROP_UL_STATS_COL_WIDTHS, width, 0,
-			G_N_ELEMENTS(width));
-		gui_prop_get_boolean(PROP_UL_STATS_COL_VISIBLE, visible, 0,
-			G_N_ELEMENTS(visible));
-
-		for (i = 0; i < G_N_ELEMENTS(columns); i++) {
-			GtkTreeViewColumn *c;
-
-			c = gtk_tree_view_get_column(upload_stats_treeview, i);
-			g_object_set(G_OBJECT(c),
-        		"fixed-width", MAX(1, width[i]),
-        		"visible", visible[i],
-        		NULL);
-		}
+		tree_view_restore_widths(upload_stats_treeview,
+			PROP_UL_STATS_COL_WIDTHS);
+		tree_view_restore_visibility(upload_stats_treeview,
+			PROP_UL_STATS_COL_VISIBLE);
 	}
 
 }
@@ -331,29 +331,36 @@ static void upload_stats_gui_init_intern(gboolean intern)
  * upload statistics pane.
  *
  * @param us A ul_stats structure with new upload stats to add.
- * 
+ *
  */
-void upload_stats_gui_add(struct ul_stats *us)
+void
+upload_stats_gui_add(const struct ul_stats *us)
 {
     GtkListStore *store;
 	GtkTreeIter iter;
+	gchar *filename;
 
 	g_assert(us != NULL);
 
+	filename = filename_to_utf8_normalized(us->filename, UNI_NORM_GUI);
+
 	upload_stats_gui_init_intern(TRUE);
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(upload_stats_treeview));
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter,
-		c_us_filename, locale_to_utf8(us->filename, 0),
+		c_us_filename, filename,
 		c_us_attempts, us->attempts,
 		c_us_complete, us->complete,
 		c_us_size, (guint) us->size,
 		c_us_norm, (gfloat) us->norm,
 		c_us_stat, us,
 		(-1));
+	
+	G_FREE_NULL(filename);
 }
 
-void upload_stats_gui_init(void)
+void
+upload_stats_gui_init(void)
 {
 	upload_stats_gui_init_intern(FALSE);
 }
@@ -365,11 +372,12 @@ void upload_stats_gui_init(void)
  * @param size The size of that file.
  *
  */
-void upload_stats_gui_update(const gchar *name, guint64 size)
+void
+upload_stats_gui_update(const gchar *name, guint64 size)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
-	struct ul_stats *us; 
+	struct ul_stats *us;
 
 	g_assert(name != NULL);
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(upload_stats_treeview));
@@ -387,7 +395,8 @@ void upload_stats_gui_update(const gchar *name, guint64 size)
  * Clear all upload statistic entries from the GtkTreeModel.
  *
  */
-void upload_stats_gui_clear_all(void)
+void
+upload_stats_gui_clear_all(void)
 {
 	GtkListStore *store;
 
@@ -396,10 +405,11 @@ void upload_stats_gui_clear_all(void)
 	gtk_list_store_clear(store);
 }
 
-void upload_stats_gui_shutdown(void)
+void
+upload_stats_gui_shutdown(void)
 {
 	tree_view_save_widths(upload_stats_treeview, PROP_UL_STATS_COL_WIDTHS);
 	tree_view_save_visibility(upload_stats_treeview, PROP_UL_STATS_COL_VISIBLE);
 }
 
-/* vi: set ts=4 sw=4: */
+/* vi: set ts=4 sw=4 cindent: */
